@@ -24,7 +24,7 @@ from trosnoth.messages import (TaggingZoneMsg, ShootMsg, RespawnMsg,
         RespawnRequestMsg, PlayerKilledMsg, JoinRequestMsg, JoinSuccessfulMsg,
         CannotJoinMsg, UpdatePlayerStateMsg, AimPlayerAtMsg, RemovePlayerMsg,
         BuyUpgradeMsg, ChatMsg)
-from trosnoth.model.universe_base import GameState
+from trosnoth.model.universe_base import NeutralTeamId
 from trosnoth.utils.twist import WeakCallLater, WeakLoopingCall
 import trosnoth.ais
 from math import atan2
@@ -36,13 +36,19 @@ def listAIs(playableOnly=False):
     Returns a list of strings of the available AI classes.
     '''
     results = []
-    for f in os.listdir(os.path.dirname(trosnoth.ais.__file__)):
-        if f.endswith('.py'):
-            aiName = os.path.splitext(f)[0]
-            c = __import__('trosnoth.ais.%s' % (aiName,), fromlist=['AIClass'])
-            if hasattr(c, 'AIClass') and (c.AIClass.playable or not
-                    playableOnly):
-                results.append(aiName)
+    try:
+        files = os.listdir(os.path.dirname(trosnoth.ais.__file__))
+    except OSError:
+        # Probably frozen in a zip. Use a standard list.
+        aiNames = ['alpha', 'simple', 'john', 'test']
+    else:
+        aiNames = [os.path.splitext(f)[0] for f in files if f.endswith('.py')]
+
+    for aiName in aiNames:
+        c = __import__('trosnoth.ais.%s' % (aiName,), fromlist=['AIClass'])
+        if hasattr(c, 'AIClass') and (c.AIClass.playable or not
+                playableOnly):
+            results.append(aiName)
     return results
 
 def makeAIAgent(world, aiName):
@@ -77,13 +83,13 @@ class AIAgent(Component):
     def _tick(self):
         if self.ai is not None:
             return
-        if self.world.gameState not in (GameState.InProgress, GameState.Ended):
+        if not self.world.state.aisEnabled():
             return
 
         nick = self.aiClass.nick
 
         if self.team is None:
-            teamId = '\x00'
+            teamId = NeutralTeamId
         else:
             teamId = self.team.id
 
