@@ -24,7 +24,8 @@ from trosnoth.model.upgrades import allUpgrades
 from trosnoth.model.zone import ZoneState
 from trosnoth.messages import (AchievementUnlockedMsg, TaggingZoneMsg,
         PlayerKilledMsg, PlayerHasUpgradeMsg, ShotFiredMsg, DeleteUpgradeMsg,
-        ChatMsg, ShotAbsorbedMsg, RespawnMsg, RemovePlayerMsg, GameOverMsg)
+        ChatMsg, ShotAbsorbedMsg, RespawnMsg, RemovePlayerMsg, GameOverMsg,
+        AchievementProgressMsg)
 from trosnoth.utils.event import Event
 
 log = logging.getLogger('achievementlist')
@@ -49,6 +50,9 @@ class Achievement(object):
     keepProgress = False
     afterGame = False
 
+    oncePerGame = False
+    oncePerTeamPerGame = False
+
     def rejoined(self, player):
         pass
 
@@ -75,17 +79,22 @@ class Achievement(object):
     def __str__(self):
         return '%s' % self.idstring
 
-    def describe(self):
+    def __repr__(self):
+        return '%s' % self.idstring
+
+    @classmethod
+    def describe(cls):
         '''
         Used when saving achievement meta-information to file. Should be
         overwritten by higher level classes.
         '''
-        information = {'name': self.name,
-                       'description': self.description,
+        information = {'name': cls.name,
+                       'description': cls.description,
                        'type': 'boolean'}
-        return {self.idstring: information}
+        return {cls.idstring: information}
 
 class PlayerAchievement(Achievement):
+    perPlayer = True
     def __init__(self, player, world, outPlug):
         super(PlayerAchievement, self).__init__()
         self.player = player
@@ -96,23 +105,30 @@ class PlayerAchievement(Achievement):
         self.player = player
 
 
-class OncePerGame(Achievement):
+class PlayerlessAchievement(Achievement):
+    perPlayer = False
+
+    def achievementUnlocked(self, playerList):
+        if not self.unlocked:
+            self.unlocked = True
+            for player in playerList:
+                self._sendUnlocked(player)
+
+    def listeningForMessages(self):
+        return not self.unlocked
+
+class OncePerGame(PlayerlessAchievement):
+    oncePerGame = True
+
     def __init__(self, world, outPlug):
         super(OncePerGame, self).__init__()
         self.world = world
         self.outPlug = outPlug
         self.unlocked = False
 
-    def achievementUnlocked(self, playerList):
-        if not self.unlocked:
-            self.unlocked = True
-            for player in playerList:
-                self._sendUnlocked(player)
+class OncePerTeamPerGame(PlayerlessAchievement):
+    oncePerTeamPerGame = True
 
-    def listeningForMessages(self):
-        return not self.unlocked
-
-class OncePerTeamPerGame(Achievement):
     def __init__(self, world, team, outPlug):
         super(OncePerTeamPerGame, self).__init__()
         self.world = world
@@ -120,15 +136,6 @@ class OncePerTeamPerGame(Achievement):
         self.team = team
         self.unlocked = False
 
-    def achievementUnlocked(self, playerList):
-        if not self.unlocked:
-            self.unlocked = True
-            for player in playerList:
-                self._sendUnlocked(player)
-
-    def listeningForMessages(self):
-        return not self.unlocked
-
 class OncePerPlayerPerGame(PlayerAchievement):
     def __init__(self, player, world, outPlug):
         super(OncePerPlayerPerGame, self).__init__(player, world, outPlug)
@@ -164,7 +171,7 @@ class OnceEverPerPlayer(PlayerAchievement):
         super(OnceEverPerPlayer, self).__init__(player, world, outPlug)
         self.unlocked = False
         self.previouslyUnlocked = False
-        if self.player is not None:
+        if self.player is not None and not self.player.bot:
             self.readExistingData()
 
     def readExistingData(self):
@@ -215,6 +222,7 @@ class IncrementingAchievement(Achievement):
         return "%s: %d/%d" % (self.idstring, self.progress,
                 self.requiredQuantity)
 
+    @classmethod
     def describe(self):
         information = {'name': self.name,
                        'description': self.description,
@@ -320,10 +328,35 @@ class FireShotTrigger(Trigger):
 class EarnStarTrigger(PlayerKillTrigger, ZoneTagTrigger):
     messages = (PlayerKilledMsg, TaggingZoneMsg)
 
+class WinTrigger(Trigger):
+    messages = (GameOverMsg,)
+
+    def gotGameOverMsg(self, msg):
+        if (msg.teamId == '\x00' or self.world.getTeam(msg.teamId) !=
+                self.player.team):
+            return
+
+        playerCount = 0
+
+        for player in self.world.players:
+            if not player.bot:
+                playerCount += 1
+
+        if playerCount >= 6:
+            self.onTrigger.execute()
+
 ##########################
 # Achievement subclasses #
 ##########################
 
+class ListeningAchievement(Achievement):
+    messages = (AchievementProgressMsg,)
+
+    @checkPlayerId
+    def gotAchievementProgressMsg(self, msg):
+        if msg.achievementId == self.idstring:
+            self.achievementUnlocked()
+
 class KilledSomeoneAchievement(Achievement):
     messages = (PlayerKilledMsg,)
 
@@ -350,6 +383,7 @@ class ChecklistAchievement(Achievement):
         return "%s: %d/%d items" % (self.idstring, len(self.progress),
                 len(self.requiredItems))
 
+    @classmethod
     def describe(self):
         information = {'name': self.name,
                        'description': self.description,
@@ -385,7 +419,7 @@ class TimedAchievement(Achievement):
         self.rollingList = []
 
     def addToList(self):
-        now = self.world.getGameTime()
+        now = self.world.getElapsedTime()
         self.rollingList.append(now)
 
         while (len(self.rollingList) > 0 and (now - self.rollingList[0]) >
@@ -404,11 +438,52 @@ class QuickKillAchievement(TimedAchievement, KilledSomeoneAchievement):
         if msg.deathType in self.deathTypes:
             self.addToList()
 
+##################################
+# Collection of all achievements #
+##################################
+
+class AchievementSet(object):
+    def __init__(self, template=None):
+        self.all = []
+        self.byId = {}
+        self.oncePerGame = []
+        self.oncePerTeamPerGame = []
+        self.perPlayer = []
+        if template:
+            for achievement in template.all:
+                self.register(achievement)
+
+    def getAchievementDetails(self, idstring):
+        achievement = self.byId[idstring]
+        if achievement.name == "":
+            return idstring, achievement.description
+        else:
+            return achievement.name, achievement.description
+
+    def register(self, achievement):
+        if achievement.idstring in self.byId:
+            raise KeyError('achievement with id %r already registered' % (
+                    achievement.idstring,))
+
+        self.all.append(achievement)
+        self.byId[achievement.idstring] = achievement
+
+        if achievement.oncePerGame:
+            self.oncePerGame.append(achievement)
+        elif achievement.oncePerTeamPerGame:
+            self.oncePerTeamPerGame.append(achievement)
+        else:
+            assert achievement.perPlayer
+            self.perPlayer.append(achievement)
+        return achievement
+
+availableAchievements = AchievementSet()
 
 #########################
 # Concrete Achievements #
 #########################
 
+@availableAchievements.register
 class MultiTagSmall(Streak, OncePerPlayerPerGame):
     idstring = 'multiTagSmall'
     name = 'I Ain\'t Even Winded'
@@ -419,6 +494,7 @@ class MultiTagSmall(Streak, OncePerPlayerPerGame):
     streakTarget = 3
     messages = incrementTrigger.messages + resetTrigger.messages
 
+@availableAchievements.register
 class MultiTagMedium(Streak, OncePerPlayerPerGame):
     idstring = 'multiTagMedium'
     name = 'The Long Way Home'
@@ -429,6 +505,7 @@ class MultiTagMedium(Streak, OncePerPlayerPerGame):
     streakTarget = 5
     messages = incrementTrigger.messages + resetTrigger.messages
 
+@availableAchievements.register
 class MultiTagLarge(Streak, OncePerPlayerPerGame):
     idstring = 'multiTagLarge'
     name = 'Cross-Country Marathon'
@@ -439,24 +516,27 @@ class MultiTagLarge(Streak, OncePerPlayerPerGame):
     streakTarget = 8
     messages = incrementTrigger.messages + resetTrigger.messages
 
+@availableAchievements.register
 class TotalTagsSmall(Streak, OnceEverPerPlayer):
     idstring = 'totalTagsSmall'
-    name = 'Captured 20 Zones'
+    name = 'Cultural Assimilation'
     description = 'Capture 20 zones'
 
     incrementTrigger = ZoneTagTrigger
     streakTarget = 20
     messages = incrementTrigger.messages
 
+@availableAchievements.register
 class TotalTagsMedium(Streak, OnceEverPerPlayer):
     idstring = 'totalTagsMedium'
-    name = 'Captured 50 Zones'
+    name = 'Globalization'
     description = 'Capture 50 zones'
 
     incrementTrigger = ZoneTagTrigger
     streakTarget = 50
     messages = incrementTrigger.messages
 
+@availableAchievements.register
 class MultiKillSmall(Streak, OncePerPlayerPerGame):
     idstring = 'multikillSmall'
     name = 'Triple Threat'
@@ -465,8 +545,9 @@ class MultiKillSmall(Streak, OncePerPlayerPerGame):
     incrementTrigger = PlayerKillTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 3
-    messages = incrementTrigger.messages + resetTrigger.messages
+    messages = list(set(incrementTrigger.messages + resetTrigger.messages))
 
+@availableAchievements.register
 class MultiKillMedium(Streak, OncePerPlayerPerGame):
     idstring = 'multikillMedium'
     name = 'High Five'
@@ -475,18 +556,20 @@ class MultiKillMedium(Streak, OncePerPlayerPerGame):
     incrementTrigger = PlayerKillTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 5
-    messages = incrementTrigger.messages + resetTrigger.messages
+    messages = list(set(incrementTrigger.messages + resetTrigger.messages))
 
+@availableAchievements.register
 class MultiKillLarge(Streak, OncePerPlayerPerGame):
     idstring = 'multikillLarge'
-    name = 'Seriously Ten'
-    description = 'Kill ten enemies in a single life'
+    name = "That's the Badger"
+    description = 'Kill nine enemies in a single life'
 
     incrementTrigger = PlayerKillTrigger
     resetTrigger = PlayerDeathTrigger
-    streakTarget = 10
-    messages = incrementTrigger.messages + resetTrigger.messages
+    streakTarget = 9
+    messages = list(set(incrementTrigger.messages + resetTrigger.messages))
 
+@availableAchievements.register
 class TotalKillsSmall(Streak, OnceEverPerPlayer):
     idstring = 'totalKillsSmall'
     name = 'Family Friendly Fun'
@@ -496,6 +579,7 @@ class TotalKillsSmall(Streak, OnceEverPerPlayer):
     streakTarget = 50
     messages = incrementTrigger.messages
 
+@availableAchievements.register
 class TotalKillsMedium(Streak, OnceEverPerPlayer):
     idstring = 'totalKillsMedium'
     name = 'We All Fall Down'
@@ -505,6 +589,7 @@ class TotalKillsMedium(Streak, OnceEverPerPlayer):
     streakTarget = 100
     messages = incrementTrigger.messages
 
+@availableAchievements.register
 class ShoppingSpree(Streak, OncePerPlayerPerGame):
     idstring = 'multiUpgradesSmall'
     name =  'Shopping Spree'
@@ -515,6 +600,7 @@ class ShoppingSpree(Streak, OncePerPlayerPerGame):
     streakTarget = 2
     messages = incrementTrigger.messages + resetTrigger.messages
 
+@availableAchievements.register
 class SmartShopper(Streak, OncePerPlayerPerGame):
     idstring = 'multiUpgradesMedium'
     name = 'Smart Shopper'
@@ -524,6 +610,7 @@ class SmartShopper(Streak, OncePerPlayerPerGame):
     streakTarget = 5
     messages = incrementTrigger.messages
 
+@availableAchievements.register
 class BulletsSmall(Streak, OncePerPlayerPerGame):
     idstring = 'bulletsSmall'
     name = 'Machine Gunner'
@@ -534,6 +621,7 @@ class BulletsSmall(Streak, OncePerPlayerPerGame):
     streakTarget = 100
     messages = incrementTrigger.messages + resetTrigger.messages
 
+@availableAchievements.register
 class BulletsMedium(Streak, OncePerPlayerPerGame):
     idstring = 'bulletsMedium'
     name = 'Ammunition Overdrive'
@@ -543,6 +631,61 @@ class BulletsMedium(Streak, OncePerPlayerPerGame):
     streakTarget = 500
     messages = incrementTrigger.messages
 
+@availableAchievements.register
+class WinnerTiny(Streak, OnceEverPerPlayer):
+    idstring = 'winnerTiny'
+    name = 'Trosnoth Newbie'
+    description = 'Win your first game of Trosnoth (minimum 6 players)'
+    afterGame = True
+
+    incrementTrigger = WinTrigger
+    streakTarget = 1
+    messages = incrementTrigger.messages
+
+@availableAchievements.register
+class WinnerSmall(Streak, OnceEverPerPlayer):
+    idstring = 'winnerSmall'
+    name = 'Trosnoth Amateur'
+    description = 'Win 5 games of Trosnoth (minimum 6 players)'
+    afterGame = True
+
+    incrementTrigger = WinTrigger
+    streakTarget = 5
+    messages = incrementTrigger.messages
+
+@availableAchievements.register
+class WinnerMedium(Streak, OnceEverPerPlayer):
+    idstring = 'winnerMedium'
+    name = 'Trosnoth Consultant'
+    description = 'Win 10 games of Trosnoth (minimum 6 players)'
+    afterGame = True
+
+    incrementTrigger = WinTrigger
+    streakTarget = 10
+    messages = incrementTrigger.messages
+
+@availableAchievements.register
+class WinnerLarge(Streak, OnceEverPerPlayer):
+    idstring = 'winnerLarge'
+    name = 'Trosnoth Professional'
+    description = 'Win 20 games of Trosnoth (minimum 6 players)'
+    afterGame = True
+
+    incrementTrigger = WinTrigger
+    streakTarget = 20
+    messages = incrementTrigger.messages
+
+@availableAchievements.register
+class WinnerHuge(Streak, OnceEverPerPlayer):
+    idstring = 'winnerHuge'
+    name = 'Trosnoth Expert'
+    description = 'Win 50 games of Trosnoth (minimum 6 players)'
+
+    incrementTrigger = WinTrigger
+    streakTarget = 50
+    messages = incrementTrigger.messages
+
+@availableAchievements.register
 class AssistTags(Streak, OncePerPlayerPerGame):
     idstring = 'assistTags'
     name = 'Credit to Team'
@@ -552,6 +695,7 @@ class AssistTags(Streak, OncePerPlayerPerGame):
     streakTarget = 5
     messages = incrementTrigger.messages
 
+@availableAchievements.register
 class QuickKillSmall(QuickKillAchievement, NoLimit):
     requiredValue = 2
     timeWindow = 5
@@ -560,6 +704,7 @@ class QuickKillSmall(QuickKillAchievement, NoLimit):
     name = 'Double Kill'
     description = 'Kill two enemies within five seconds (no grenades)'
 
+@availableAchievements.register
 class QuickKillMedium(QuickKillAchievement, NoLimit):
     requiredValue = 3
     timeWindow = 5
@@ -568,6 +713,7 @@ class QuickKillMedium(QuickKillAchievement, NoLimit):
     name = 'Triple Kill'
     description = 'Kill three enemies within five seconds (no grenades)'
 
+@availableAchievements.register
 class GrenadeMultiKill(QuickKillAchievement, NoLimit):
     requiredValue = 3
     # Processing of achievements should happen within 0.25 secs
@@ -577,7 +723,96 @@ class GrenadeMultiKill(QuickKillAchievement, NoLimit):
     name = "It's Super Effective!"
     description = 'Kill three enemies with a single grenade'
 
+@availableAchievements.register
+class RicochetAchievement(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Bouncy Flag'
+    description = ('Kill an enemy with a bullet that has ricocheted at least '
+            'once')
+    idstring = 'ricochetKill'
+
+@availableAchievements.register
+class AccuracySmall(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Boom, Headshot'
+    description = 'Have an accuracy of 10% or higher during a game'
+    idstring = 'accuracySmall'
+
+@availableAchievements.register
+class AccuracyMedium(ListeningAchievement, OncePerPlayerPerGame):
+    name = "Sniping's a Good Job, Mate"
+    description = 'Have an accuracy of 15% or higher during a game'
+    idstring = 'accuracyMedium'
+
+@availableAchievements.register
+class AccuracyLarge(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Professionals Have Standards'
+    description = 'Have an accuracy of 20% or higher during a game'
+    idstring = 'accuracyLarge'
+
+@availableAchievements.register
+class StatScore(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Master of the Internet'
+    description = 'Reach a stats score of 1337 or higher during a game'
+    idstring = 'statScore'
+
+@availableAchievements.register
+class TotalAliveTime(ListeningAchievement, OnceEverPerPlayer):
+    name = 'Long Live the King'
+    description = 'Rack up a total of 1000 seconds of alive time'
+    idstring = 'totalAliveTime'
+
+@availableAchievements.register
+class StayingAlive(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Never Asleep on the Job'
+    description = 'Be alive at least 75% of the time'
+    idstring = 'stayingAlive'
+
+@availableAchievements.register
+class AliveStreak(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Still Alive'
+    description = 'Stay alive for at least 180 seconds in a single life'
+    idstring = 'aliveStreak'
+
+@availableAchievements.register
+class UseStarsEfficiently(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Waste Not, Want Not'
+    description = ('Use or contribute at least 50% of the stars you earn '
+            'during a game')
+    idstring = 'useStarsEfficiently'
+
+@availableAchievements.register
+class LongRangeKill(ListeningAchievement, OncePerPlayerPerGame):
+    name = 'Long-Range Ballistics'
+    description = 'Kill an enemy at the maximum range of your gun'
+    idstring = 'longRangeKill'
+
 # More complicated ones:
+@availableAchievements.register
+class TagsAndKills(OncePerPlayerPerGame):
+    name = 'All-Rounder'
+    description = 'Kill 10 people and tag 5 zones in a single life'
+    idstring = 'tagsAndKills'
+
+    status = [0, 0]
+    requiredQuantity = [10, 5]
+    messages = (TaggingZoneMsg, PlayerKilledMsg)
+
+    @checkPlayerId
+    def gotTaggingZoneMsg(self, msg):
+        self.status[1] = min(self.status[1] + 1, self.requiredQuantity[1])
+##        print "Status: %s" % repr(self.status)
+        if self.status == self.requiredQuantity:
+            self.achievementUnlocked()
+
+    def gotPlayerKilledMsg(self, msg):
+        if msg.targetId == self.player.id:
+            self.status = [0, 0]
+        elif msg.killerId == self.player.id:
+            self.status[0] = min(self.status[0] + 1, self.requiredQuantity[0])
+##            print "Status: %s" % repr(self.status)
+            if self.status == self.requiredQuantity:
+                self.achievementUnlocked()
+
+@availableAchievements.register
 class DestroyStars(KilledSomeoneAchievement, IncrementingAchievement,
         OnceEverPerPlayer):
     idstring = 'destroyStars'
@@ -588,6 +823,7 @@ class DestroyStars(KilledSomeoneAchievement, IncrementingAchievement,
     def killedSomeone(self, msg):
         self.increment(msg.stars)
 
+@availableAchievements.register
 class EarnStars(IncrementingAchievement, OnceEverPerPlayer):
     idstring = 'earnStars'
     name = 'Stimulus Package'
@@ -597,6 +833,7 @@ class EarnStars(IncrementingAchievement, OnceEverPerPlayer):
     requiredQuantity = 100
     messages = incrementTrigger.messages
 
+@availableAchievements.register
 class BuyEveryUpgrade(PersistedChecklistAchievement):
     idstring = 'buyEveryUpgrade'
     name = 'Technology Whiz'
@@ -606,8 +843,10 @@ class BuyEveryUpgrade(PersistedChecklistAchievement):
 
     @checkPlayerId
     def gotPlayerHasUpgradeMsg(self, msg):
-        self.addItem(msg.upgradeType)
+        if msg.reasonId == 'B':
+            self.addItem(msg.upgradeType)
 
+@availableAchievements.register
 class UseMinimapDisruption(OncePerPlayerPerGame):
     idstring = 'minimapDisruption'
     name = 'The Rader Appears to be... Jammed!'
@@ -616,10 +855,11 @@ class UseMinimapDisruption(OncePerPlayerPerGame):
 
     @checkPlayerId
     def gotPlayerHasUpgradeMsg(self, msg):
-        if msg.upgradeType == 'm':
+        if msg.upgradeType == 'm' and msg.reasonId == 'B':
             self.achievementUnlocked()
 
-class EarlyAbandon(OncePerPlayerPerGame):
+#disabled @availableAchievements.register
+class AbandonEarly(OncePerPlayerPerGame):
     idstring = 'abandonEarly'
     name = 'Reduce, Reuse, Recycle'
     description = 'Abandon a turret or phase shift before it runs out'
@@ -634,6 +874,7 @@ class EarlyAbandon(OncePerPlayerPerGame):
                 self.player.upgradeGauge >= 3):
             self.achievementUnlocked()
 
+@availableAchievements.register
 class GoodManners(OncePerPlayerPerGame):
     idstring = 'goodManners'
     name = 'Good Manners'
@@ -646,6 +887,7 @@ class GoodManners(OncePerPlayerPerGame):
         if msg.kind == 'o' and msg.text.decode().lower().strip() == 'gg':
             self.achievementUnlocked()
 
+@availableAchievements.register
 class DarkZoneKills(IncrementingAchievement, KilledSomeoneAchievement,
         OncePerPlayerPerGame):
     idstring = 'darkZoneKills'
@@ -660,6 +902,56 @@ class DarkZoneKills(IncrementingAchievement, KilledSomeoneAchievement,
                 and not self.player.ghost):
             self.increment()
 
+@availableAchievements.register
+class NinjaKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
+    idstring = 'ninjaKill'
+    name = "Ninjas Can't Catch You if You're on Fire"
+    description = 'Kill an invisible ninja'
+
+    def killedSomeone(self, msg):
+        if msg.upgradeType == 'n':
+            self.achievementUnlocked()
+
+@availableAchievements.register
+class AntiDisruption(KilledSomeoneAchievement, OncePerPlayerPerGame):
+    idstring = 'antiDisruption'
+    name = "Back Online"
+    description = 'Kill an enemy who is using a Minimap Disruption'
+
+    def killedSomeone(self, msg):
+        if msg.upgradeType == 'm':
+            self.achievementUnlocked()
+
+@availableAchievements.register
+class ChokepointSpam(IncrementingAchievement, KilledSomeoneAchievement,
+        OncePerPlayerPerGame):
+
+    idstring = 'chokepointSpam'
+    name = "Chokepoint Checkmate"
+    description = 'Kill three players without moving or dying (no upgrades)'
+    requiredQuantity = 3
+
+    x = y = 0
+
+    messages = (PlayerKilledMsg, ShotFiredMsg)
+
+    # Strictly speaking, the player is still able to move or die, but if they
+    # don't shoot from exactly the same spot it won't count anyway.
+
+    @checkPlayerId
+    def gotShotFiredMsg(self, msg):
+        if msg.xpos != self.x or msg.ypos != self.y:
+            self.reset()
+
+        self.x = msg.xpos
+        self.y = msg.ypos
+
+    def killedSomeone(self, msg):
+        if self.player.upgrade is None:
+            self.increment()
+
+
+#disabled @availableAchievements.register
 class PhaseShiftAbsorb(IncrementingAchievement, OncePerPlayerPerGame):
     idstring = 'phaseShiftAbsorb'
     name = "Can't Touch This"
@@ -677,23 +969,7 @@ class PhaseShiftAbsorb(IncrementingAchievement, OncePerPlayerPerGame):
         if msg.upgradeType == 'h':
             self.reset()
 
-class RicochetAchievement(KilledSomeoneAchievement, OncePerPlayerPerGame):
-    name = 'Bouncy Flag'
-    description = ('Kill an enemy with a bullet that has ricocheted at least '
-            'once')
-    idstring = 'ricochetKill'
-
-    def killedSomeone(self, msg):
-        try:
-            shot = self.world.getShot(self.player.id, msg.shotId)
-        except Abort:
-            # This doesn't yet work, because the world deletes the shot before
-            # we get here
-            pass
-        else:
-            if shot.bounced:
-                self.achievementUnlocked()
-
+@availableAchievements.register
 class KillEnemyWithStars(KilledSomeoneAchievement, OncePerPlayerPerGame):
     name = 'Stop Right There, Criminal Scum'
     description = 'Kill an enemy holding five stars or more'
@@ -703,6 +979,7 @@ class KillEnemyWithStars(KilledSomeoneAchievement, OncePerPlayerPerGame):
         if msg.stars >= 5:
             self.achievementUnlocked()
 
+@availableAchievements.register
 class KillAsRabbit(KilledSomeoneAchievement, OncePerPlayerPerGame):
     name = 'Never Surrender'
     description = 'Kill an enemy after losing a game'
@@ -711,9 +988,10 @@ class KillAsRabbit(KilledSomeoneAchievement, OncePerPlayerPerGame):
 
     def killedSomeone(self, msg):
         if (self.player.team is not None and
-                self.player.isEnemyTeam(self.world.winningTeam)):
+                self.player.isEnemyTeam(self.world.getWinningTeam())):
             self.achievementUnlocked()
 
+@availableAchievements.register
 class RabbitKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
     name = 'Icing on the Cake'
     description = 'Kill an enemy after winning a game'
@@ -721,11 +999,12 @@ class RabbitKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
     afterGame = True
 
     def killedSomeone(self, msg):
-        if (self.world.winningTeam == self.player.team and
+        if (self.world.getWinningTeam() == self.player.team and
                 self.player.isEnemyTeam(self.world.getPlayer(msg.targetId
                 ).team)):
             self.achievementUnlocked()
 
+#disabled @availableAchievements.register
 class TurretKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
     name = 'Sentry Mode Activated'
     description = 'Kill an enemy as a turret'
@@ -736,6 +1015,7 @@ class TurretKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
                 == 't'):
             self.achievementUnlocked()
 
+@availableAchievements.register
 class FirstKill(OncePerGame):
     name = 'First Blood'
     description = 'Make the first kill of the game'
@@ -752,6 +1032,7 @@ class FirstKill(OncePerGame):
                 players = set([killer])
                 self.achievementUnlocked(players)
 
+@availableAchievements.register
 class FirstTag(OncePerGame):
     name = 'Game On'
     description = 'Tag or assist in the tagging of the first zone in a game'
@@ -768,6 +1049,22 @@ class FirstTag(OncePerGame):
                     helpers.add(player)
             self.achievementUnlocked(helpers)
 
+@availableAchievements.register
+class FirstRespawn(OncePerGame):
+    name = 'Finger on the Trigger'
+    description = 'Be the first person to respawn (minimum 6 players)'
+    idstring = 'firstRespawn'
+    messages = (RespawnMsg,)
+
+    def gotRespawnMsg(self, msg):
+        if not self.unlocked:
+            if len(self.world.players) < 6:
+                self.achievementUnlocked([])
+            else:
+                player = self.world.getPlayer(msg.playerId)
+                self.achievementUnlocked([player])
+
+@availableAchievements.register
 class LastTag(OncePerGame):
     name = 'And the Dirt is Gone'
     description = 'Tag or assist in the tagging of the final zone'
@@ -785,6 +1082,7 @@ class LastTag(OncePerGame):
                     winners.add(player)
             self.achievementUnlocked(winners)
 
+@availableAchievements.register
 class Domination(KilledSomeoneAchievement, OncePerPlayerPerGame):
     name = 'Domination'
     description = 'Kill the same enemy five times in a single game'
@@ -801,8 +1099,9 @@ class Domination(KilledSomeoneAchievement, OncePerPlayerPerGame):
         if self.playerKills[nick] == self.requiredQuantity:
             self.achievementUnlocked()
 
+@availableAchievements.register
 class MinimapDisruptionTag(OncePerPlayerPerGame):
-    name = 'Tag in the Dark'
+    name = 'Under the Radar'
     description = 'Tag a zone while the enemy\'s minimap is disrupted'
     idstring = 'disruptionTag'
     messages = (TaggingZoneMsg,)
@@ -815,6 +1114,7 @@ class MinimapDisruptionTag(OncePerPlayerPerGame):
                 self.achievementUnlocked()
                 break
 
+@availableAchievements.register
 class MutualKill(NoLimit):
     name = 'Eye for an Eye'
     description = 'Kill an enemy at the same time he kills you'
@@ -853,6 +1153,7 @@ class MutualKill(NoLimit):
     def gotRemovePlayerMsg(self, msg):
         self.tryRemove(msg.playerId)
 
+@availableAchievements.register
 class SnatchedZone(OncePerPlayerPerGame):
     idstring = 'snatched'
     name = 'Snatched from the Jaws'
@@ -885,6 +1186,7 @@ class SnatchedZone(OncePerPlayerPerGame):
             # Couldn't have tagged if we'd been half a second later
             self.achievementUnlocked()
 
+@availableAchievements.register
 class KilledByNobody(KilledSomeoneAchievement, OncePerPlayerPerGame):
     name = 'And Who Deserves the Blame?'
     description = 'Die with no-one being awarded a kill'
@@ -895,6 +1197,7 @@ class KilledByNobody(KilledSomeoneAchievement, OncePerPlayerPerGame):
         if msg.targetId == self.player.id and msg.killerId == '\x00':
             self.achievementUnlocked()
 
+@availableAchievements.register
 class ShieldRevenge(OncePerPlayerPerGame):
     name = 'Shields Up, Weapons Online'
     description = 'Kill the enemy who shot your shield within ten seconds'
@@ -913,7 +1216,7 @@ class ShieldRevenge(OncePerPlayerPerGame):
         if (msg.targetId == self.player.id and self.player.upgrade is not None
                 and self.player.upgrade.upgradeType == 's'):
             # Our shield got hit
-            self.shieldKillers.append((self.world.getGameTime(),
+            self.shieldKillers.append((self.world.getElapsedTime(),
                     self.world.getPlayer(msg.shooterId)))
 
     def gotPlayerKilledMsg(self, msg):
@@ -927,7 +1230,7 @@ class ShieldRevenge(OncePerPlayerPerGame):
 
     def killedSomeone(self, msg):
         target = self.world.getPlayer(msg.targetId)
-        currentTime = self.world.getGameTime()
+        currentTime = self.world.getElapsedTime()
         for time, attacker in list(self.shieldKillers):
             if currentTime - time > self.timeWindow:
                 self.shieldKillers.remove((time, attacker))
@@ -935,6 +1238,7 @@ class ShieldRevenge(OncePerPlayerPerGame):
                 self.achievementUnlocked()
                 break
 
+@availableAchievements.register
 class ShieldThenKill(OncePerPlayerPerGame):
     name = 'Armour-Piercing Bullets'
     description = 'Kill an enemy after destroying their shields'
@@ -954,7 +1258,7 @@ class ShieldThenKill(OncePerPlayerPerGame):
         if (msg.shooterId == self.player.id and target.upgrade is not None and
                 target.upgrade.upgradeType == 's'):
             # We hit their shield
-            self.shieldsKilled.append((self.world.getGameTime(), target))
+            self.shieldsKilled.append((self.world.getElapsedTime(), target))
 
     def gotPlayerKilledMsg(self, msg):
         if msg.targetId == self.player.id:
@@ -967,7 +1271,7 @@ class ShieldThenKill(OncePerPlayerPerGame):
 
     def killedSomeone(self, msg):
         target = self.world.getPlayer(msg.targetId)
-        currentTime = self.world.getGameTime()
+        currentTime = self.world.getElapsedTime()
         for time, victim in list(self.shieldsKilled):
             if currentTime - time > self.timeWindow:
                 self.shieldsKilled.remove((time, victim))
@@ -975,6 +1279,28 @@ class ShieldThenKill(OncePerPlayerPerGame):
                 self.achievementUnlocked()
                 break
 
+@availableAchievements.register
+class ShieldDefense(OncePerPlayerPerGame):
+    name = 'Strategic Deployment'
+    description = 'Activate a shield just before being shot'
+    idstring = 'shieldDefense'
+
+    timeWindow = 0.2 # seconds
+    messages = (PlayerHasUpgradeMsg, ShotAbsorbedMsg)
+    shieldTime = None
+
+    @checkPlayerId
+    def gotPlayerHasUpgradeMsg(self, msg):
+        if msg.upgradeType == 's':
+            self.shieldTime = self.world.getElapsedTime()
+
+    def gotShotAbsorbedMsg(self, msg):
+        if msg.targetId == self.player.id and self.shieldTime is not None:
+            currentTime = self.world.getElapsedTime()
+            if (currentTime - self.shieldTime) <= self.timeWindow:
+                self.achievementUnlocked()
+
+@availableAchievements.register
 class LastManStanding(OncePerTeamPerGame):
     name = 'Never Give Up'
     description = 'Be the last surviving member of your team'
@@ -992,57 +1318,99 @@ class LastManStanding(OncePerTeamPerGame):
 
     def check(self):
         remaining = []
+
         for player in self.world.players:
             if player.team == self.team:
                 remaining.append(player)
+
         if len(remaining) == 1:
             self.achievementUnlocked([remaining[0]])
 
 ##################
 # Leader Killing #
 ##################
-leaderAchievements = {'kjohns': 'Vespene Geyser Exhausted',
-    'jbartl': 'The Bigger They Are...', 'mwalla': "School's Out",
-    'jredmo': 'Kernel Panic', 'tirwin': 'Uncomfortably Energetic',
-    'dmengl': "Trolling Ain't Easy", 'shorn': 'Employee of the Month',
-    'adonal': 'Drum Solo', 'dcatch': 'Jerakeen Genocide',
-    'jowen': 'Zergling Rush', 'lsharl': 'Knight of Catan'}
+leaderAchievements = {
+                      'adonal': 'Drum Solo',
+                      'dcatch': 'Jerakeen Genocide',
+                      'dmenge': "Trolling Ain't Easy",
+                      'jbartl': 'The Bigger They Are...',
+                      'jowen':  'Zergling Rush',
+                      'jredmo': 'Kernel Panic',
+                      'khethe': 'Vespene Geyser Exhausted',
+                      'kjohns': 'All Your Bass are Belong to Us',
+                      'lsharl': 'Knight of Catan',
+                      'mbrand': 'How do I shot web?',
+                      'mwalla': "School's Out",
+                      'shorn':  'Employee of the Month',
+                      'tirwin': 'Uncomfortably Energetic',
+                      'tryan':  'Nerf Warfare'
+                      }
+
+leaderNames =        {'adonal': 'ErbaneOrb',
+                      'dcatch': 'Kindred',
+                      'dmenge': 'Cricket',
+                      'jbartl': 'Hrodga',
+                      'jowen':  'Nobody',
+                      'jredmo': 'Avatar',
+                      'khethe': 'Scarecrow',
+                      'kjohns': 'Kain',
+                      'lsharl': 'Dr. Strangelove',
+                      'mbrand': 'PorridgeLord',
+                      'mwalla': 'Jupator',
+                      'shorn':  'Clamburger',
+                      'tirwin': 'Bundy',
+                      'tryan':  'Acne'
+                      }
 
-additionalLeaders = {
-    'gsearl': 'Galapagos',
-    'dholme': 'Jyolanar',
-    'mdaman': 'MaZ',
-}
 leaderAchievements = additionalLeaders = {} # We are not on camp.
 
 def getLeaderDescription(name):
-    return 'Kill %s three times in a single game' % (name,)
+    return 'Kill %s three times' % (leaderNames[name],)
 def stripPunctuation(nick):
     exclude = set(string.punctuation + ' ')
     return ''.join(ch for ch in nick if ch not in exclude)
 
-class LeaderMultiKill(KilledSomeoneAchievement, IncrementingAchievement,
+for username, achievementName in leaderAchievements.iteritems():
+    @availableAchievements.register
+    class LeaderMultiKill(KilledSomeoneAchievement, IncrementingAchievement,
+            OnceEverPerPlayer):
+        requiredQuantity = 3
+        idstring = 'leader' + stripPunctuation(username)
+        name = achievementName
+        description = getLeaderDescription(username)
+        username = username
+
+        def killedSomeone(self, msg):
+            player = self.world.getPlayer(msg.targetId)
+            if player.user is None:
+                return
+            username = player.user.username
+            if username == self.username:
+                self.increment()
+
+#disabled @availableAchievements.register
+class DeveloperKills(KilledSomeoneAchievement, IncrementingAchievement,
         OnceEverPerPlayer):
-    requiredQuantity = 3
-    def __init__(self, player, world, outPlug, leaderName, achievementName):
-        self.idstring = 'leader' + stripPunctuation(leaderName)
-        super(LeaderMultiKill, self).__init__(player, world, outPlug)
-        self.leaderName = leaderName
-        self.name = achievementName
-        self.description = getLeaderDescription(self.leaderName)
+    idstring = 'developerKills'
+    name = 'Developer Destroyer'
+    description = 'Rack up 10 developer kills'
+    requiredQuantity = 10
+
+    developers = ('jbartl', 'adonal', 'shorn',
+                  'hrodga', 'erbaneorb', 'clamburger')
 
     def killedSomeone(self, msg):
         player = self.world.getPlayer(msg.targetId)
         if player.user is None:
             return
-        nick = stripPunctuation(player.user.username)
-        if nick.lower() == self.leaderName.lower():
+        nick = stripPunctuation(player.user.username).lower()
+        if nick in self.developers:
             self.increment()
 
-
 def lower(string):
     return string.lower()
 
+#disabled @availableAchievements.register
 class KillEveryLeader(KilledSomeoneAchievement, PersistedChecklistAchievement):
     idstring = 'killEveryLeader'
     name = 'Leaders vs Campers'
@@ -1058,36 +1426,3 @@ class KillEveryLeader(KilledSomeoneAchievement, PersistedChecklistAchievement):
         nick = stripPunctuation(player.user.username).lower()
         if nick in self.requiredItems:
             self.addItem(nick)
-
-concreteAchievements = (MultiTagSmall, MultiTagMedium, MultiTagLarge,
-    TotalTagsSmall, TotalTagsMedium, MultiKillSmall, MultiKillMedium,
-    MultiKillLarge, TotalKillsSmall, TotalKillsMedium, ShoppingSpree,
-    SmartShopper, FirstTag, KillEnemyWithStars, RicochetAchievement, FirstKill,
-    Domination, DestroyStars, EarnStars, BulletsSmall, BulletsMedium,
-    AssistTags, KillAsRabbit, RabbitKill, UseMinimapDisruption, BuyEveryUpgrade,
-    KillEveryLeader, MutualKill, GrenadeMultiKill, QuickKillSmall,
-    QuickKillMedium, SnatchedZone, TurretKill, PhaseShiftAbsorb, EarlyAbandon,
-    GoodManners, MinimapDisruptionTag, LastTag, DarkZoneKills, KilledByNobody,
-    ShieldRevenge, ShieldThenKill, LastManStanding)
-
-achievementDict = {}
-for achv in concreteAchievements:
-    achievementDict[achv.idstring] = achv
-for idstring, name in leaderAchievements.iteritems():
-    achievementDict['leader'+idstring] = (lambda player, world, outPlug,
-            idstring=idstring, name=name: LeaderMultiKill(player, world,
-            outPlug, idstring, name))
-
-
-# Get the name and description, given an idstring
-def getAchievementDetails(idstring):
-    for achievementClass in concreteAchievements:
-        if achievementClass.idstring == idstring:
-            if achievementClass.name == "":
-                return idstring, achievementClass.description
-            else:
-                return achievementClass.name, achievementClass.description
-    for leaderName, achievementName in leaderAchievements.iteritems():
-        if 'leader'+leaderName == idstring:
-            return achievementName, getLeaderDescription(leaderName)
-    return None, None
