@@ -20,33 +20,24 @@ import logging
 
 from trosnoth.utils.components import Component, Plug, handler
 from trosnoth.messages import AddPlayerMsg, ChatMsg, JoinSuccessfulMsg
-from trosnoth.model.universe_base import GameState
 from trosnoth.utils.utils import timeNow
 
-from trosnoth.gamerecording.achievementlist import (concreteAchievements,
-        LeaderMultiKill, leaderAchievements, OncePerGame, OncePerTeamPerGame,
-        stripPunctuation, getLeaderDescription)
+from trosnoth.gamerecording.achievementlist import availableAchievements
 
 log = logging.getLogger('achievements')
 
 class PlayerAchievements(object):
     '''Tracks achievements for a single player.'''
+    achievementDefs = availableAchievements
+
     def __init__(self, player, world, outPlug):
         self.player = player
         self.world = world
         self.achievements = []
         self.achievementMap = {}
 
-        for classType in concreteAchievements:
-            if issubclass(classType, OncePerGame) or issubclass(classType,
-                    OncePerTeamPerGame):
-                pass
-            else:
-                self._addAchievement(classType(self.player, world, outPlug))
-
-        for leaderName, achievementName in leaderAchievements.iteritems():
-            self._addAchievement(LeaderMultiKill(self.player, world, outPlug,
-                    leaderName, achievementName))
+        for classType in self.achievementDefs.perPlayer:
+            self._addAchievement(classType(self.player, world, outPlug))
 
     def _addAchievement(self, achievement):
         self.achievements.append(achievement)
@@ -76,14 +67,7 @@ def mightApply(world, achievement):
     Returns True iff this achievement might apply, based on whether or not the
     game is on.
     '''
-    if world.gameState == GameState.Ended:
-        gameRunning = False
-    elif world.gameState == GameState.InProgress:
-        gameRunning = True
-    else:
-        return False
-
-    return gameRunning ^ achievement.afterGame
+    return world.gameIsInProgress() ^ achievement.afterGame
 
 class AchievementManager(Component):
     '''Listens for messages and triggers achievements accordingly.'''
@@ -91,9 +75,11 @@ class AchievementManager(Component):
     outPlug = Plug()
 
     SAVE_PERIOD = 20
+    achievementDefs = availableAchievements
 
     def __init__(self, world, filename):
         Component.__init__(self)
+
         self.world = world
         self.filename = filename.replace('replace-this', 'players')
         self.metaFilename = filename.replace('replace-this', 'definitions')
@@ -101,19 +87,18 @@ class AchievementManager(Component):
 
         self._metaSave()
 
-        # All players recorded this game, indexed by nick.
+        # All players recorded this game, indexed by user id.
         self.allPlayers = {}
 
         self.oncePerGameAchievements = []
         self.oncePerTeamPerGameAchievements = []
-        for classType in concreteAchievements:
-            if issubclass(classType, OncePerGame):
-                self.oncePerGameAchievements.append(classType(world,
-                        self.outPlug))
-            elif issubclass(classType, OncePerTeamPerGame):
-                for team in world.teams:
-                    self.oncePerTeamPerGameAchievements.append(classType(world,
-                            team, self.outPlug))
+        for classType in self.achievementDefs.oncePerGame:
+            self.oncePerGameAchievements.append(classType(world,
+                    self.outPlug))
+        for classType in self.achievementDefs.oncePerTeamPerGame:
+            for team in world.teams:
+                self.oncePerTeamPerGameAchievements.append(classType(world,
+                        team, self.outPlug))
 
     def _metaSave(self):
         '''Saves information about the achievements themselves to a JSON file.
@@ -122,26 +107,10 @@ class AchievementManager(Component):
 
         allAchievements = {}
 
-        for classType in concreteAchievements:
-            if issubclass(classType, OncePerGame):
-                allAchievements.update(classType(None, None).describe())
-            else:
-                allAchievements.update(classType(None, None, None).describe())
+        for classType in self.achievementDefs.all:
+            allAchievements.update(classType.describe())
 
-        for leaderName, achievementName in leaderAchievements.iteritems():
-            idstring = 'leader' + stripPunctuation(leaderName)
-            description = getLeaderDescription(leaderName)
-            allAchievements.update({idstring: {'name': achievementName,
-                                               'description': description,
-                                               'type': 'incremental',
-                                               'requirements': 3,
-                                               'keepProgress': False}})
-
-        file = open(self.metaFilename, 'w')
-        jsonString = json.dumps(allAchievements, indent = 4)
-        file.write(jsonString)
-        file.flush()
-        file.close()
+        json.dump(allAchievements, open(self.metaFilename, 'w'), indent=4)
 
     @inPlug.defaultHandler
     def passOn(self, msg):
