@@ -37,9 +37,9 @@ htmlDir = os.path.join(gameDir, 'htmlStats')
 achvDir = os.path.join(gameDir, 'achievements')
 achvExt = '.trosachv'
 
-def makeGameRecorder(world, layoutDatabase, idManager):
+def makeGameRecorder(world, gameController, layoutDatabase, idManager):
     gameRecorder = GameRecorder(world)
-    w = WorldSettingsResponder(world, layoutDatabase)
+    w = WorldSettingsResponder(world, gameController, layoutDatabase)
 
     gameRecorder.inPlug.connectPlug(w.agentEvents)
     gameRecorder.outPlug.connectPlug(w.agentRequests)
