@@ -44,11 +44,11 @@ class ReplayRecorder(Component):
         self.world = world
         self.file = open(self.filename, 'w')
         self.numTilFlush = self.FLUSH_AFTER
-        self.timeStarted = self.world.getGameTime()
+        self.timeStarted = self.world.getElapsedTime()
         self.stopped = False
 
     def timeStamp(self):
-        return self.world.getGameTime() - self.timeStarted
+        return self.world.getElapsedTime() - self.timeStarted
 
     @inPlug.defaultHandler
     def gotMessage(self, msg):
@@ -63,9 +63,10 @@ class ReplayRecorder(Component):
             self.file.flush()
 
     def stop(self):
-        self.file.flush()
-        self.file.close()
-        self.stopped = True
+        if not self.stopped:
+            self.file.flush()
+            self.file.close()
+            self.stopped = True
 
 from twisted.internet import reactor
 
@@ -90,7 +91,7 @@ class ReplayPlayer(Component):
 
     def begin(self):
         self.loadMessages(self.MESSAGES_TO_LOAD)
-        self.timeStarted = self.world.getGameTime()
+        self.timeStarted = self.world.getElapsedTime()
         reactor.callLater(0, self.tick)
 
 
@@ -117,7 +118,7 @@ class ReplayPlayer(Component):
     def tick(self):
         while len(self.messagesToReplay) > 0:
             time, msg = self.messagesToReplay[0]
-            if time <= self.world.getGameTime():
+            if time <= self.world.getElapsedTime():
                 self.outPlug.send(msg)
                 del self.messagesToReplay[0]
             else:
