@@ -21,12 +21,11 @@ from trosnoth.utils.components import Component, Plug, handler
 from trosnoth.messages import (TaggingZoneMsg, PlayerStarsSpentMsg,
         GameStartMsg, GameOverMsg, ShotFiredMsg, RespawnMsg, PlayerKilledMsg,
         PlayerHitMsg, AddPlayerMsg, RemovePlayerMsg, PlayerHasUpgradeMsg,
-        ShotAbsorbedMsg)
+        ShotAbsorbedMsg, AchievementProgressMsg)
 
 from trosnoth.utils.utils import timeNow
 from collections import defaultdict
 from trosnoth.model.upgrades import upgradeOfType
-from trosnoth.model.universe_base import GameState
 
 # Known Issues:
 #  * If a player shoots a turret, a shield, or a phase shift,
@@ -37,8 +36,9 @@ from trosnoth.model.universe_base import GameState
 
 class PlayerStatKeeper(object):
     '''Maintains the statistics for a particular player object'''
-    def __init__(self, player):
+    def __init__(self, player, plug):
         self.player = player
+        self.plug = plug
 
         # A: Recorded [A]ll game (including post-round)
         # G: Recorded during the main [G]ame only
@@ -77,18 +77,79 @@ class PlayerStatKeeper(object):
     def shotFired(self):
         self.shotsFired += 1
 
+        if self.shotsFired >= 100:
+            if self.accuracy() >= 0.10:
+                self.sendAchievementProgress('accuracySmall')
+            if self.accuracy() >= 0.15:
+                self.sendAchievementProgress('accuracyMedium')
+            if self.accuracy() >= 0.20:
+                self.sendAchievementProgress('accuracyLarge')
+
+        if self.totalPoints() >= 1337:
+            self.sendAchievementProgress('statScore')
+
+    def sendAchievementProgress(self, achievementId):
+        if self.player.id == -1:
+            # Player is no longer in game.
+            return
+        self.plug.send(AchievementProgressMsg(self.player.id, achievementId))
+
+    def _updateStreaks(self, updateAlive):
+        '''
+        updateAlive will be set to True in three situations:
+          1. if the player has just died
+          2. if the player was alive when the game ended
+          3. if the player was alive when they disconnected
+        '''
+        self.killStreak = max(self.killStreak, self.currentKillStreak)
+        self.tagStreak = max(self.tagStreak, self.currentTagStreak)
+
+        time = timeNow()
+
+        if updateAlive:
+            lastLife = time - self.lastTimeRespawned
+            self.aliveStreak = max(self.aliveStreak, lastLife)
+            self.timeAlive += lastLife
+            if lastLife >= 180:
+                self.sendAchievementProgress('aliveStreak')
+
+        elif self.lastTimeDied is not None:
+            self.timeDead += time - self.lastTimeDied
+
+        self.currentKillStreak = 0
+        self.currentTagStreak = 0
+
     def died(self, killer, stars):
         self.deaths += 1
         self.playerDeaths[killer] += 1
         self.starsWasted += stars
-        self._allStreaksOver()
-        self.lastTimeDied = timeNow()
+
+        time = timeNow()
+
+        self._updateStreaks(True)
+
+        self.lastTimeDied = time
+
+        if self.timeAlive >= 1000:
+            self.sendAchievementProgress('totalAliveTime')
+
+        if self.timeAlive >= 300 and self.timeAlive >= (self.timeAlive +
+                self.timeDead) * 0.75:
+            self.sendAchievementProgress('stayingAlive')
+
+    def respawned(self):
+        time = timeNow()
+
+        if self.lastTimeDied is not None:
+            self.timeDead += (time - self.lastTimeDied)
+
+        self.lastTimeRespawned = time
 
     def goneFromGame(self):
-        self._allStreaksOver()
+        self._updateStreaks(not self.player.ghost)
 
     def gameOver(self, winningTeamId):
-        self.saveAllStreaks()
+        self._updateStreaks(not self.player.ghost)
         winningTeam = self.player.universe.getTeam(winningTeamId)
         if winningTeam is None:
             # Draw. Do nothing
@@ -98,31 +159,21 @@ class PlayerStatKeeper(object):
         else:
             self.roundsWon += 1
 
-    def _allStreaksOver(self):
-        self.saveAllStreaks()
-        self.lastTimeSaved = None
-
-        self.currentKillStreak = 0
-        self.currentTagStreak = 0
-        self.lastTimeRespawned = None
-
     def killed(self, victim):
         self.kills += 1
         self.playerKills[victim] += 1
         self.currentKillStreak += 1
         self.starsEarned += 1
 
-    def respawned(self):
-        self.lastTimeRespawned = timeNow()
-        self.saveAllStreaks()
-        self.lastTimeDied = None
-        self.lastTimeSaved = None
+        self.killStreak = max(self.killStreak, self.currentKillStreak)
 
     def zoneTagged(self):
         self.zoneTags += 1
         self.currentTagStreak += 1
         self.starsEarned += 1
 
+        self.tagStreak = max(self.tagStreak, self.currentTagStreak)
+
     def tagAssist(self):
         self.zoneAssists += 1
 
@@ -132,9 +183,12 @@ class PlayerStatKeeper(object):
     def usedStars(self, stars):
         self.starsUsed += stars
 
+        if self.starsUsed >= 20 and self.starsUsed >= self.starsEarned * 0.5:
+            self.sendAchievementProgress('useStarsEfficiently')
+
     def upgradeUsed(self, upgradeType):
         self.upgradesUsed[upgradeType] += 1
-        self.saveAllStreaks()
+
 
     def totalPoints(self):
         points = 0
@@ -149,35 +203,16 @@ class PlayerStatKeeper(object):
     def _accuracyPoints(self):
         if self.shotsFired == 0:
             return 0
-        return ((self.shotsHit ** 2) / self.shotsFired) * 30
+        return ((self.shotsHit ** 2.) / self.shotsFired) * 30
 
     def accuracy(self):
         if self.shotsFired == 0:
             return 0
-        return self.shotsHit / self.shotsFired
-
-    def saveAllStreaks(self):
-        now = timeNow()
-        if self.player.ghost:
-            if self.lastTimeDied is not None:
-                self.timeDead += (now - self.lastTimeDied)
-        else:
-            self.killStreak = max(self.killStreak, self.currentKillStreak)
-            self.tagStreak = max(self.tagStreak, self.currentTagStreak)
-
-            if self.lastTimeRespawned is not None:
-                curAliveStreak = now - self.lastTimeRespawned
-                if self.lastTimeSaved != None:
-                    timeToAdd = now - self.lastTimeSaved
-                else:
-                    timeToAdd = curAliveStreak
-                self.aliveStreak = max(self.aliveStreak, curAliveStreak)
-                self.timeAlive += timeToAdd
-        self.lastTimeSaved = now
+        return self.shotsHit * 1. / self.shotsFired
 
     # Probably fallible, as players could rejoin with the same name.
     def statDict(self):
-        self.saveAllStreaks()
+        #self.saveAllStreaks()
         stats = {}
         for val in ('aliveStreak', 'deaths', 'killStreak', 'kills',
                 'roundsLost', 'roundsWon', 'shotsFired', 'shotsHit',
@@ -187,6 +222,9 @@ class PlayerStatKeeper(object):
             stats[val] = getattr(self, val)
         stats['bot'] = self.player.bot
         stats['team'] = self.player.teamId
+        stats['username'] = (self.player.user.username if self.player.user
+                else None)
+
         # We've stored these dicts as player objects (meaning rejoins may be
         # credited in two places)
         # Here, we combine them by aggregating on player nick (which should be
@@ -225,7 +263,7 @@ class StatKeeper(Component):
         self.winningTeamId = None
 
     def save(self, force=False):
-        if not force and self.world.gameState != GameState.InProgress:
+        if not force and self.world.gameIsInProgress():
             return
         stats = {}
         stats['players'] = {}
@@ -287,7 +325,7 @@ class StatKeeper(Component):
                 statkeeper = sl
                 statkeeper.rejoined(player)
         if statkeeper is None:
-            statkeeper = PlayerStatKeeper(player)
+            statkeeper = PlayerStatKeeper(player, self.world.eventPlug)
             self.allPlayerStatLists.append(statkeeper)
         self.playerStatList[msg.playerId] = statkeeper
         self.save()
@@ -311,6 +349,7 @@ class StatKeeper(Component):
     def shotFired(self, msg):
         self.playerStatList[msg.playerId].shotFired()
 
+
     @handler(ShotAbsorbedMsg, inPlug)
     @handler(PlayerHitMsg, inPlug)
     def playerHit(self, msg):
