@@ -170,6 +170,15 @@ class TextImage(NewImage):
         self.font = font
         self.antialias = antialias
 
+    def setColour(self, colour):
+        self.colour = colour
+
+    def setBackColour(self, colour):
+        self.bgColour = colour
+    
+    def setFont(self, font):
+        self.font = font
+
     def setText(self, text):
         self._text = str(text)
     def getText(self):
