@@ -18,9 +18,10 @@
 from trosnoth.gui.fonts import fontcache
 
 class Font(object):
-    def __init__(self, name, size):
+    def __init__(self, name, size, bold = False):
         self._name = name
         self._size = size
+        self._bold = bold
         self._app = None
         self._filename = None
 
@@ -46,7 +47,7 @@ class Font(object):
             self._app = app
         else:
             filename = self._filename
-        return fontcache.get(filename, self._size)
+        return fontcache.get(filename, self._size, self._bold)
 
     def __repr__(self):
         return "Font: %s size %d" % (self._name, self._size)
@@ -59,7 +60,8 @@ class ScaledFont(Font):
         else:
             filename = self._filename
         return fontcache.get(filename,
-                int(self._size * app.screenManager.scaleFactor + 0.5))
+                int(self._size * app.screenManager.scaleFactor + 0.5),
+                self._bold)
 
     def __repr__(self):
         return "Scaled Font: %s size %d" % (self._name, self._size)
