@@ -23,9 +23,9 @@ log = logging.getLogger('fontcache')
 
 _cache = {}
 
-def get(filename, size):
+def get(filename, size, bold = False):
     try:
-        return _cache[filename, size]
+        return _cache[filename, size, bold]
     except KeyError:
         try:
             result = pygame.font.Font(filename, size)
@@ -33,5 +33,8 @@ def get(filename, size):
             log.error('Font Cache: unable to load font %r' % (filename,))
             result = pygame.font.Font(None, size)
 
-        _cache[filename, size] = result
+        if bold:
+            result.set_bold(True)
+
+        _cache[filename, size, bold] = result
         return result
