@@ -15,6 +15,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
+import logging
+
 import pygame
 
 from framework import Element
@@ -22,6 +24,8 @@ from elements import TextElement
 from trosnoth.utils.event import Event
 from trosnoth.gui.common import AttachedPoint, Location, ScaledSize
 
+log = logging.getLogger('checkbox')
+
 class CheckBox(Element):
     def __init__(self, app, pos, text, font, colour, initValue=False,
                  hotkey=None, style='circle', fillColour=None):
@@ -89,6 +93,9 @@ class CheckBox(Element):
             self.value = val
             self.onValueChanged.execute(self)
 
+    def getValue(self):
+        return self.value
+
     def processEvent(self, event):
         box = self._getBoxRect()
         # Active events.
