@@ -28,16 +28,17 @@ class Clock(Element):
     def __init__(self, app, timer, pos, font,
                  colour = (0,0,0), bgColour = None):
         super(Clock, self).__init__(app)
-        self.timer = timer
+        self.setTimer(timer)
         self.pos = pos
         self.size = None
         self.colour = colour
         self.bgColour = bgColour
-        # Number of seconds counted down/remaining
-        self.currentVal = 0
 
         self.font = font
 
+    def setTimer(self, timer):
+        self.timer = timer
+
     # Rather than the size of this object changing as the numbers change,
     # set a fixed size.
     # For no fixed size, call with size as None
