@@ -52,7 +52,7 @@ class TwistedInteractiveConsole(InteractiveConsole):
         InteractiveConsole.__init__(self, *args, **kwargs)
         self._dummyStdOut = DummyStdOut(self.write)
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def interact(self, banner=None):
         """Closely emulate the interactive Python console.
 
@@ -88,9 +88,7 @@ class TwistedInteractiveConsole(InteractiveConsole):
                 else:
                     prompt = sys.ps1
                 try:
-                    readInput = defer.waitForDeferred(self.raw_input(prompt))
-                    yield readInput
-                    line = readInput.getResult()
+                    line = yield self.raw_input(prompt)
 
                     # Can be None if sys.stdin was redefined
                     encoding = getattr(sys.stdin, "encoding", None)
