@@ -106,12 +106,13 @@ class TextElement(Element):
     @param pos: should be an instance of trosnoth.ui.common.Location'''
 
     def __init__(self, app, text, font, pos, colour=(0,128,0),
-            anchor='topleft', shadow=False, backColour=None):
+            anchor='topleft', shadow=False, backColour=None,
+            antialias=True):
         super(TextElement, self).__init__(app)
 
         self.pos = pos
         self.anchor = anchor
-        self.image = TextImage(text, font, colour, backColour)
+        self.image = TextImage(text, font, colour, backColour, antialias)
         self.__shadow = None
         self.setShadow(shadow)
         self._surface = None
@@ -163,14 +164,19 @@ class TextElement(Element):
             self._surface = None
 
     def __getShadowPos(self):
-        height = self.image.font.getHeight(self.app)
-        x = (height / 45) + 1
+        x = self._shadowOffset
         pos = RelativeLocation(self.pos, (x, x))
         return pos
 
-    def setShadow(self, shadow):
+    def setShadow(self, shadow, offset = None):
         if shadow:
             if self.__shadow is None:
+                if offset is None:
+                    height = self.image.font.getHeight(self.app)
+                    self._shadowOffset = (height / 45) + 1
+                else:
+                    self._shadowOffset = offset
+
                 self.__shadow = TextElement(self.app, self.image.text,
                         self.image.font, self.__getShadowPos(),
                         colours.shadowDefault, self.anchor, False)
@@ -180,6 +186,11 @@ class TextElement(Element):
                 self.__shadow = None
                 self._surface = None
 
+    def setShadowOffset(self, offset):
+        if self.__shadow is not None:
+            self.setShadow(False)
+            self.setShadow(True, offset)
+
     def setShadowColour(self, colour):
         if self.__shadow is not None:
             self.__shadow.setColour(colour)
@@ -290,3 +301,17 @@ class TextButton(HoverButton):
     def setText(self, text):
         self.stdImage.text = text
         self.hvrImage.text = text
+
+    def setFont(self, font):
+        self.stdImage.setFont(font)
+        self.hvrImage.setFont(font)
+
+    def setColour(self, colour):
+        self.stdImage.setColour(colour)
+
+    def setHoverColour(self, colour):
+        self.hvrImage.setColour(colour)
+
+    def setBackColour(self, colour):
+        self.hvrImage.setBackColour(colour)
+        self.stdImage.setBackColour(colour)
\ No newline at end of file
