@@ -76,7 +76,7 @@ class CompoundElement(Element):
                 try:
                     element.tick(deltaT)
                 except Exception, e:
-                    log.exception(str(e))
+                    log.error(str(e), exc_info=True)
         except RuntimeError, e:
             if e.__str__() == 'maximum recursion depth exceeded':
                 log.error('ERROR: Infinite loop of elements:')
@@ -92,7 +92,7 @@ class CompoundElement(Element):
             try:
                 element.draw(screen)
             except Exception, e:
-                log.exception(str(e))
+                log.error(str(e), exc_info=True)
             element.drawTime.noteValue(timeNow() - t)
 
     def getSlowestChild(self):
@@ -108,17 +108,17 @@ class CompoundElement(Element):
         self.setFocus(None)
 
     def setFocus(self, element):
-        if element and element not in self.elements:
+        if element is not None and element not in self.elements:
             return
 
-        if element and element.hasFocus:
+        if element is not None and element.hasFocus:
             return
 
         for el in self.elements:
             if el.hasFocus:
                 el.hasFocus = False
                 el.lostFocus()
-        if element:
+        if element is not None:
             element.hasFocus = True
             element.gotFocus()
 
