@@ -53,8 +53,8 @@ class MenuDisplay(framework.CompoundElement):
     ACCELERATION = 500  # pix/s/s
 
     def __init__(self, app, location, size, font, manager, titleColour,
-                 stdColour, hvrColour, backColour=None, autosize=False,
-                 hidable=False, keymapping=None):
+                 stdColour, hvrColour, disabledColour=None, backColour=None,
+                 autosize=False, hidable=False, keymapping=None):
         super(MenuDisplay, self).__init__(app)
 
         self.screenSize = app.screenManager.size
@@ -67,6 +67,7 @@ class MenuDisplay(framework.CompoundElement):
         self.titleColour = titleColour
         self.stdColour = stdColour
         self.hvrColour = hvrColour
+        self.disabledColour = disabledColour or stdColour
 
         self.autosize = autosize
         self.hidable = hidable
@@ -255,21 +256,27 @@ class MenuDisplay(framework.CompoundElement):
         stdImage.set_colorkey((0,0,0))
         hvrImage.set_colorkey((0,0,0))
 
+        if item.enabled:
+            stdColour = self.stdColour
+            hvrColour = self.hvrColour
+        else:
+            stdColour = hvrColour = self.disabledColour
+
         if item.name == '---':
             # Separator.
-            pygame.draw.line(stdImage, self.stdColour, (0, itemHeight//2),
+            pygame.draw.line(stdImage, stdColour, (0, itemHeight//2),
                              (self.rect.width, itemHeight//2))
-            pygame.draw.line(hvrImage, self.hvrColour, (0, itemHeight//2),
+            pygame.draw.line(hvrImage, hvrColour, (0, itemHeight//2),
                              (self.rect.width, itemHeight//2))
         else:
             # Put caption on standard canvas.
-            img = self.font.render(self.app, item.name, True, self.stdColour)
+            img = self.font.render(self.app, item.name, True, stdColour)
             r = img.get_rect()
             r.midleft = (2, itemHeight // 2)
             stdImage.blit(img, r)
 
             # Put caption on hover canvas.
-            img = self.font.render(self.app, item.name, True, self.hvrColour)
+            img = self.font.render(self.app, item.name, True, hvrColour)
             hvrImage.blit(img, r)
 
         if item.action is not None and self.keymapping is not None:
@@ -281,16 +288,17 @@ class MenuDisplay(framework.CompoundElement):
             else:
 
                 # Put hotkey on standard canvas.
-                img = self.font.render(self.app, hkStr, True, self.stdColour)
+                img = self.font.render(self.app, hkStr, True, stdColour)
                 r = img.get_rect()
                 r.midright = (width-2, itemHeight // 2)
                 stdImage.blit(img, r)
 
                 # Put hotkey on hover canvas.
-                img = self.font.render(self.app, hkStr, True, self.hvrColour)
+                img = self.font.render(self.app, hkStr, True, hvrColour)
                 hvrImage.blit(img, r)
 
         btn = elements.HoverButton(self.app, Location((x, y), 'center'),
                                    stdImage, hvrImage)
-        btn.onClick.addListener(lambda btn: item.execute())
+        if item.enabled:
+            btn.onClick.addListener(lambda btn: item.execute())
         self.elements.append(btn)
