@@ -18,11 +18,15 @@
 '''prompt.py - contains the definition for a prompt that could be used for
 any type of input.'''
 
+import logging
+
 import pygame
 import trosnoth.gui.framework.framework as framework
 from trosnoth.utils.event import Event
 from trosnoth.gui import keyboard
 
+log = logging.getLogger('inputbox')
+
 def intValidator(char):
     return char >= '0' and char <= '9'
 
@@ -76,6 +80,9 @@ class InputBox(framework.Element):
         self.cursorIndex = len(self.value)
         self.offset = 0
 
+    def clear(self):
+        self.setValue('')
+
     def setValue(self, value):
         self.value = value
         self.offset = 0
@@ -84,6 +91,15 @@ class InputBox(framework.Element):
     def getValue(self):
         return self.value
 
+    def setFont(self, font):
+        self.font = font
+
+    def setValidator(self, validator):
+        self.validator = validator
+
+    def setMaxLength(self, length):
+        self.maxLength = length
+
     def _getSize(self):
         return self._getRect().size
 
@@ -257,6 +273,7 @@ class InputBox(framework.Element):
 
             if (event.type == pygame.MOUSEBUTTONDOWN and
                     rect.collidepoint(event.pos) and event.button == 1):
+
                 self.onClick.execute(self)
                 xOffset = event.pos[0] - rect.left
                 self.cursorIndex = self.__getCursorIndex(xOffset)
