@@ -27,7 +27,7 @@ log = logging.getLogger('scrollableCanvas')
 class ScrollBar(Element):
     defaultWidth = 15
     defaultButtonChange = 20  # number of pixels moved when button is pressed
-    defaultColour = (0, 255, 0)
+    defaultColour = (0, 192, 0)
     defaultBackColour = (128, 128, 128)
     def __init__(self, app, parent, pos, horizontal=False):
         Element.__init__(self, app)
@@ -71,12 +71,12 @@ class ScrollBar(Element):
         self.__checkThisIsValid()
         if self.horizontal:
             size = (self._getBarLength(), self._getWidth())
-            pos = (self._getTopLeftPt()[0] + self._getPosition(),
+            pos = (self._getTopLeftPt()[0] + self.getPosition(),
                     self._getTopLeftPt()[1])
         else:
             size = (self._getWidth(), self._getBarLength())
             pos = (self._getTopLeftPt()[0], self._getTopLeftPt()[1] +
-                    self._getPosition())
+                    self.getPosition())
         return pygame.rect.Rect(pos, size)
 
     def __getDrawableSize(self):
@@ -101,16 +101,16 @@ class ScrollBar(Element):
         self.pos.apply(self.app, r)
         return r
 
-    def __movedTo(self, pos):
+    def _movedTo(self, pos):
         self.__checkThisIsValid()
         if self.horizontal:
-            self._setPosition(self.originalPosition + (pos[0] -
+            self.setPosition(self.originalPosition + (pos[0] -
                     self.beingDraggedFrom[0]))
         else:
-            self._setPosition(self.originalPosition + (pos[1] -
+            self.setPosition(self.originalPosition + (pos[1] -
                     self.beingDraggedFrom[1]))
 
-        newScrollingPos = (self._getPosition() * self.__getCanvasSize() /
+        newScrollingPos = (self.getPosition() * self.__getCanvasSize() /
                 self.__getDrawableSize())
         newScrollingPos = min(newScrollingPos, self.__getCanvasSize() -
                 self.__getDrawableSize())
@@ -119,12 +119,12 @@ class ScrollBar(Element):
         else:
             self.parent.setVerticalPos(newScrollingPos)
 
-    def _getPosition(self):
+    def getPosition(self):
         # This line is useful in the event of a screen resize
-        self._setPosition(self.position)
+        self.setPosition(self.position)
         return self.position
 
-    def _setPosition(self, pos):
+    def setPosition(self, pos):
         self.position = min(max(pos, 0), self.__getDrawableSize() -
                 self._getBarLength())
 
@@ -137,12 +137,12 @@ class ScrollBar(Element):
             return None
         elif (event.type == pygame.MOUSEMOTION and self.beingDraggedFrom
                 != None):
-            self.__movedTo(event.pos)
+            self._movedTo(event.pos)
             return None
         elif (event.type == pygame.MOUSEBUTTONUP and event.button == 1
                 and self.beingDraggedFrom != None):
             self.beingDraggedFrom = None
-            self.originalPosition = self._getPosition()
+            self.originalPosition = self.getPosition()
             return None
         return event
 
@@ -165,6 +165,9 @@ class ScrollableCanvas(CompoundElement):
         self.pos = pos
         self.size = size
         self.displaySize = displaySize
+        self.backgroundColour = (64,64,64)
+
+        self._screenSize = tuple(app.screenManager.size)
 
         self.__horizontalScrollBar = ScrollBar(app, self,
                 Location(AttachedPoint((0,0), self._getRect, 'bottomleft'),
@@ -176,15 +179,16 @@ class ScrollableCanvas(CompoundElement):
         # internalPos is the position on the full canvas that we are drawing
         # from
         self.internalPos = None
-        self._setInternalPos((0,0))
+        self.setInternalPos((0,0))
 
-    def _getInternalPos(self):
-        # This line is useful in the event of screen resizes:
-        self._setInternalPos(self.internalPos)
+    def getInternalPos(self):
+        if self._screenSize != self.app.screenManager.size:
+            self._screenSize = tuple(self.app.screenManager.size)
+            self.setInternalPos(self.internalPos)
 
         return self.internalPos
 
-    def _setInternalPos(self, pos):
+    def setInternalPos(self, pos):
         self.internalPos = tuple([min(pos[i], (self._getCanvasSize()[i] -
                 self._getDrawableSize()[i])) for i in 0,1])
 
@@ -253,22 +257,22 @@ class ScrollableCanvas(CompoundElement):
         return r
 
     def setVerticalPos(self, verticalPos):
-        self._setInternalPos((self._getInternalPos()[0], verticalPos))
+        self.setInternalPos((self.getInternalPos()[0], verticalPos))
 
     def setHorizontalPos(self, horizontalPos):
-        self._setInternalPos((horizontalPos, self._getInternalPos()[1]))
+        self.setInternalPos((horizontalPos, self.getInternalPos()[1]))
 
     def _getContentsRect(self):
         return pygame.rect.Rect(self._getPt(), self._getDrawableSize())
 
     def _getOffsetCanvasRect(self):
         topleft = self._getPt()
-        offsetPt = tuple([topleft[i] - self._getInternalPos()[i] for i in 0, 1])
+        offsetPt = tuple([topleft[i] - self.getInternalPos()[i] for i in 0, 1])
         return pygame.rect.Rect(offsetPt, self._getCanvasSize())
 
     def draw(self, screen):
         contentsRect = self._getContentsRect()
-        screen.fill((64,64,64), contentsRect)
+        screen.fill(self.backgroundColour, contentsRect)
         if self._needsHorizontalScrollBar():
             self._getHorizontalScrollBar().draw(screen)
         if self._needsVerticalScrollBar():
@@ -293,10 +297,13 @@ class ScrollableCanvas(CompoundElement):
             event = self._getVerticalScrollBar().processEvent(event)
         if event:
             if (event.type == pygame.MOUSEBUTTONDOWN or event.type ==
-                    pygame.MOUSEBUTTONUP or event.type == pygame.MOUSEMOTION):
+                    pygame.MOUSEBUTTONUP):
                 if self._getContentsRect().collidepoint(event.pos):
                     CompoundElement.processEvent(self, event)
                     return None
+                elif event.type == pygame.MOUSEMOTION:
+                    CompoundElement.processEvent(self, event)
+                    return event
                 else:
                     return event
             else:
