@@ -15,13 +15,20 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-from framework import CompoundElement
-from elements import TextElement
+import logging
+
+import pygame
+
+from trosnoth.gui.framework.framework import CompoundElement
+from trosnoth.gui.framework.elements import TextElement, TextButton
+from trosnoth.gui.framework.prompt import InputBox
 from trosnoth.gui.fonts.font import Font
 from trosnoth.gui import colours
-import pygame
-from trosnoth.gui.common import Location, AttachedPoint, addPositions
+from trosnoth.gui.common import Location, AttachedPoint, addPositions, Area
+from trosnoth.utils.event import Event
+from trosnoth.utils.utils import BasicContextManager
 
+log = logging.getLogger('game')
 
 # Styles work thusly:
 # When looking for a particular attribute (say backColour),
@@ -29,11 +36,12 @@ from trosnoth.gui.common import Location, AttachedPoint, addPositions
 # its row's style object, then it its column's style object,
 # then in the table's. As soon as it finds a non-None value,
 # it will use that.
-class TableStyle(object):
+class TableStyle(BasicContextManager):
 
     def __init__(self):
         self.backColour = None
         self.foreColour = None
+        self.hoverColour = None
         self.hasShadow = None
         self.shadowColour = None
         self.font = None
@@ -50,14 +58,17 @@ class StyleUser(CompoundElement):
 
 
 class CellAggregator(StyleUser):
-    def __init__(self, app, table, index):
+    def __init__(self, app, table):
         super(CellAggregator, self).__init__(app, TableStyle())
         self._table = table
         self._cells = []
-        self.index = index
+        self.index = -1
         self.elements = self._cells
 
-    def _newCell(self, cell, index = None):
+    def setIndex(self, index):
+        self.index = index
+
+    def _newCell(self, cell, index=None):
         if index is None:
             index = len(self._cells)
         assert index <= len(self._cells)
@@ -92,9 +103,10 @@ class CellAggregator(StyleUser):
 
 class Row(CellAggregator):
     def __init__(self, app, table, index, height):
-        super(Row, self).__init__(app, table, index)
+        super(Row, self).__init__(app, table)
         assert hasattr(height, 'getVal')
         self.setHeight(height)
+        self.setIndex(index)
 
     def setHeight(self, height):
         assert hasattr(height, 'getVal')
@@ -107,13 +119,24 @@ class Row(CellAggregator):
         return self._table._getRowPt(self.index)
 
 class Column(CellAggregator):
-    def __init__(self, app, table, index, width):
-        super(Column, self).__init__(app, table, index)
+    def __init__(self, app, table, width=0):
+        super(Column, self).__init__(app, table)
         self.setWidth(width)
 
+    def onClicked(self, sender):
+        self.setFocus(sender)
+        self._table.onClicked(self)
+
+    def lostFocus(self):
+        self.setFocus(None)
+
     def setWidth(self, width):
         self._width = width
 
+    def setDefaultWidth(self, width):
+        if self._width == 0:
+            self._width = width
+
     def _getWidth(self):
         if hasattr(self._width, 'getVal'):
             return self._width.getVal(self.app)
@@ -123,23 +146,31 @@ class Column(CellAggregator):
     def _getPt(self):
         return self._table._getColPt(self.index)
 
+class TextColumn(Column):
+    def createCell(self, row):
+        return TextCell(self.app, row, self)
+
+class TextBoxColumn(Column):
+    def createCell(self, row):
+        return TextBoxCell(self.app, row, self)
+
+class TextButtonColumn(Column):
+    def createCell(self, row):
+        return TextButtonCell(self.app, row, self)
+
 class Cell(StyleUser):
     def __init__(self, app, row, column):
         super(Cell, self).__init__(app, TableStyle())
         self._row = row
         self._column = column
-        textAlign = self.styleGet('textAlign')
-        self.textElement = TextElement(self.app, "", self.styleGet('font'), Location(CellAttachedPoint((0,0), self, textAlign), textAlign), colour=self.styleGet('foreColour'))
-        self.elements = [self.textElement]
         self._changed = True
-        self._oldText = ''
 
     @property
-    def row(self):
+    def rowNum(self):
         return self._row.index
 
     @property
-    def column(self):
+    def columnNum(self):
         return self._column.index
 
     def styleUpdate(self):
@@ -161,12 +192,6 @@ class Cell(StyleUser):
 
         return None
 
-    def setText(self, text):
-        if text != self._oldText:
-            self.textElement.setText(text)
-            self._changed = True
-            self._oldText = text
-
     def _getPt(self):
         rowPt = self._row._getPt()
         colPt = self._column._getPt()
@@ -188,19 +213,17 @@ class Cell(StyleUser):
         usableSize = tuple([size[i] - padding[i]*2 for i in 0,1])
         return pygame.Rect(usablePt, usableSize)
 
+    def _boxClicked(self, sender):
+        self.setFocus(sender)
+        self._column.onClicked(self)
+
     def draw(self, surface):
         if self._changed:
             self._changed = False
-            self.textElement.setFont(self.styleGet('font'))
-            self.textElement.setColour(self.styleGet('foreColour'))
-            textAlign = self.styleGet('textAlign')
-            self.textElement.pos.anchor = textAlign
-            self.textElement.pos.point.attachedAt = textAlign
-            bgColour = self.styleGet('backColour')
-            if bgColour is not None:
-                surface.fill(bgColour, self._getRect())
-            self.textElement.setShadow(self.styleGet('hasShadow'))
-            self.textElement.setShadowColour(self.styleGet('shadowColour'))
+            self._update()
+        bgColour = self.styleGet('backColour')
+        if bgColour is not None:
+            surface.fill(bgColour, self._getRect())
 
         clipRect = surface.get_clip()
         newClipRect = clipRect.clip(self._getUsableRect())
@@ -210,21 +233,120 @@ class Cell(StyleUser):
         finally:
             surface.set_clip(clipRect)
 
+class TextCell(Cell):
+    def __init__(self, app, row, column):
+        super(TextCell, self).__init__(app, row, column)
+        textAlign = self.styleGet('textAlign')
+        self.textElement = TextElement(self.app, '', self.styleGet('font'), Location(CellAttachedPoint((0,0), self, textAlign), textAlign), colour=self.styleGet('foreColour'))
+        self.elements = [self.textElement]
+        self._oldText = ''
+
+    def setText(self, text):
+        if text != self._oldText:
+            self.textElement.setText(text)
+            self._changed = True
+            self._oldText = text
+
+    def _update(self):
+        self.textElement.setFont(self.styleGet('font'))
+        self.textElement.setColour(self.styleGet('foreColour'))
+        textAlign = self.styleGet('textAlign')
+        self.textElement.pos.anchor = textAlign
+        self.textElement.pos.point.attachedAt = textAlign
+        self.textElement.setShadow(self.styleGet('hasShadow'))
+        self.textElement.setShadowColour(self.styleGet('shadowColour'))
+
+class TextBoxCell(Cell):
+    def __init__(self, app, row, column):
+        super(TextBoxCell, self).__init__(app, row, column)
+        textAlign = self.styleGet('textAlign')
+        self.inputBox = InputBox(self.app, Area(CellAttachedPoint((0,0), self,
+                textAlign), self._getUsableRect().size, textAlign),
+                font=self.styleGet('font'), colour=self.styleGet('foreColour'))
+        self.inputBox.onEdit.addListener(self._valueChanged)
+        self.elements = [self.inputBox]
+        self._oldText = ''
+        self._readOnly = True
+        self.setReadOnly(False)
+        self.onValueChanged = Event()
+
+    def _valueChanged(self, sender):
+        self._oldText = self.inputBox.getValue()
+        self.onValueChanged.execute(self)
+
+    def setReadOnly(self, readOnly):
+        if readOnly == self._readOnly:
+            return
+        self._readOnly = readOnly
+
+        if readOnly:
+            self.inputBox.onClick.removeListener(self._boxClicked)
+        else:
+            self.inputBox.onClick.addListener(self._boxClicked)
+            self.setFocus(None)
+
+    def lostFocus(self):
+        self.setFocus(None)
+
+    def setText(self, text):
+        if text != self._oldText:
+            self.inputBox.setValue(text)
+            self._changed = True
+            self._oldText = text
+
+    def getText(self):
+        return self._oldText
+
+    def setMaxLength(self, length):
+        self.inputBox.setMaxLength(length)
+
+    def setValidator(self, validator):
+        self.inputBox.setValidator(validator)
+
+    def _update(self):
+        self.inputBox.setFont(self.styleGet('font'))
+        self.inputBox.setFontColour(self.styleGet('foreColour'))
+        bgColour = self.styleGet('backColour')
+        if bgColour is not None:
+            self.inputBox.setBackColour(bgColour)
+        textAlign = self.styleGet('textAlign')
+        self.inputBox.area.anchor = textAlign
+        self.inputBox.area.point.attachedAt = textAlign
+
+class TextButtonCell(Cell):
+    def __init__(self, app, row, column):
+        super(TextButtonCell, self).__init__(app, row, column)
+        textAlign = self.styleGet('textAlign')
+        pos = Location(CellAttachedPoint((0,0), self, textAlign), textAlign)
+        self.textButton = TextButton(self.app, pos, '', self.styleGet('font'), self.styleGet('foreColour'),
+                                     self.styleGet('hoverColour'))
+        self.elements = [self.textButton]
+        self._oldText = ''
+
+    def setOnClick(self, onClick):
+        self.textButton.onClick.addListener(onClick)
+        self._boxClicked(None)
+
+    def setText(self, text):
+        if text != self._oldText:
+            self.textButton.setText(text)
+            self._changed = True
+            self._oldText = text
+
+    def _update(self):
+        self.textButton.setFont(self.styleGet('font'))
+        self.textButton.setColour(self.styleGet('foreColour'))
+        self.textButton.setHoverColour(self.styleGet('hoverColour'))
+        self.textButton.setBackColour(self.styleGet('backColour'))
+        textAlign = self.styleGet('textAlign')
+        self.textButton.pos.anchor = textAlign
+        self.textButton.pos.point.attachedAt = textAlign
+
 
 class CellAttachedPoint(AttachedPoint):
-    def __init__(self, val, cell, attachedAt = 'topleft'):
+    def __init__(self, val, cell, attachedAt='topleft'):
         super(CellAttachedPoint, self).__init__(val, cell._getUsableRect, attachedAt)
 
-
-defaultStyle = TableStyle()
-defaultStyle.backColour = (255,255,255)
-defaultStyle.padding = (5,5)
-defaultStyle.foreColour = (0,0,0)
-defaultStyle.font = Font('FreeSans.ttf', 36)
-defaultStyle.textAlign = 'topleft'
-defaultStyle.hasShadow = False
-defaultStyle.shadowColour = colours.shadowDefault
-
 class Value(object):
     def __init__(self, value):
         self._value = value
@@ -233,30 +355,43 @@ class Value(object):
 
 class Table(StyleUser):
     def __init__(self, app, pos, rows=None, columns=None):
+        defaultStyle = TableStyle()
+        defaultStyle.backColour = (255,255,255)
+        defaultStyle.padding = (5,5)
+        defaultStyle.foreColour = (0,0,0)
+        defaultStyle.hoverColour = (0,0,0)
+        defaultStyle.font = Font('FreeSans.ttf', 36)
+        defaultStyle.textAlign = 'topleft'
+        defaultStyle.hasShadow = False
+        defaultStyle.shadowColour = colours.shadowDefault
         super(Table, self).__init__(app, defaultStyle)
         self.pos = pos
 
         self._columns = []
         self._rows = []
-        # Bind self.elements to self._rows (arbitrarily chosen,
-        # rather than self._columns)
-        self.elements = self._rows
+        self.elements = self._columns
 
         # Style Settings:
         self._borderColour = (192,192,192)
         self._borderWidth = 10
         self._defaultWidth = 200
-        self._defaultHeight = Value(60)
+        self.setDefaultHeight(Value(60))
 
         if rows is not None:
             self.addRows(rows)
 
         if columns is not None:
-            self.addColumns(columns)
+            self.addEmptyColumns(columns)
 
         self._sizes = {}
         self._appSize = None
 
+    def onClicked(self, sender):
+        self.setFocus(sender)
+
+    def setDefaultHeight(self, height):
+        self._defaultHeight = height
+
     def styleUpdate(self):
         '''
         Informs the table that the style of some part of the table has changed.
@@ -284,31 +419,39 @@ class Table(StyleUser):
         newRow = Row(self.app, self, index, self._defaultHeight)
         self._rows.insert(index, newRow)
         for x in range(0, len(self._columns)):
-            newCell = Cell(self.app, newRow, self._columns[x])
+            newCell = self._columns[x].createCell(newRow)
             newRow._newCell(newCell, x)
             self._columns[x]._newCell(newCell, index)
 
         for x in range(index+1, len(self._rows)):
             self._rows[x]._anotherInsertedBefore()
         self._sizes = {}
+        return newRow
 
-    def addColumn(self, index = None):
+    def addColumn(self, column, index=None):
         if index is None:
             index = len(self._columns)
-        assert index <= len(self._columns)
-        # Create and Insert
-        newColumn = Column(self.app, self, index, self._defaultWidth)
-        self._columns.insert(index, newColumn)
-        for x in range(0, len(self._rows)):
-            newCell = Cell(self.app, self._rows[x], newColumn)
-            newColumn._newCell(newCell, x)
-            self._rows[x]._newCell(newCell, index)
+        if index > len(self._columns):
+            raise ValueError('invalid index %r' % (index,))
 
-        for x in range(index+1, len(self._columns)):
-            self._columns[x]._anotherInsertedBefore()
+        column.setIndex(index)
+        column.setDefaultWidth(self._defaultWidth)
+
+        self._columns.insert(index, column)
+        for rowIndex, row in enumerate(self._rows):
+            newCell = Cell(self.app, row, column)
+            column._newCell(newCell, rowIndex)
+            row._newCell(newCell, index)
+
+        for column in self._columns[index+1:]:
+            column._anotherInsertedBefore()
         self._sizes = {}
 
-    def addRows(self, count, firstIndex = None):
+    def addColumns(self, columns):
+        for column in columns:
+            self.addColumn(column)
+
+    def addRows(self, count, firstIndex=None):
         if count < 1:
             return
 
@@ -319,17 +462,20 @@ class Table(StyleUser):
             if index is not None:
                 index += 1
 
-    def addColumns(self, count, firstIndex = None):
+    def addEmptyColumns(self, count, firstIndex=None):
         if count < 1:
             return
 
         index = firstIndex
 
         for i in range(count):
-            self.addColumn(index)
+            self.addColumn(TextColumn(self.app, self, self._defaultWidth), index)
             if index is not None:
                 index += 1
 
+    def clear(self):
+        while self._rows:
+            self.delRow(self.rowCount() - 1)
 
     def delRow(self, index):
         del self._rows[index]
