@@ -40,7 +40,7 @@ class ITimer(framework.Element):
 
 class CountdownTimer(ITimer):
     '''@param amount    Amount of time to count down'''
-    def __init__(self, countTo, highest = "days"):
+    def __init__(self, countTo=0, highest = "days"):
         super(CountdownTimer,self).__init__()
         self.countTo = countTo
         self.counted = 0
