@@ -57,11 +57,12 @@ class MenuItem(object):
     @param name     the caption of the menu item
     @param action   a callable for when the item is selected
     '''
-    def __init__(self, name, action=None, listener=None):
+    def __init__(self, name, action=None, listener=None, enabled=True):
         self.name = name
         self.onClick = Event()      # (MenuItem)
         self.onAction = Event()     # (action)
         self.action = action
+        self.enabled = enabled
         if listener is not None:
             self.onClick.addListener(listener)
 
@@ -85,10 +86,17 @@ class Menu(object):
         for item in items:
             item.onAction.addListener(self.onAction.execute)
 
-    def removeItem(self, item):
-        self.items.remove(item)
+    def popItem(self, index):
+        item = self.items.pop(index)
         item.onAction.removeListener(self.onAction.execute)
+        return item
+
+    def removeItem(self, item):
+        self.popItem(self.items.index(item))
 
     def insertItem(self, pos, item):
         self.items.insert(pos, item)
         item.onAction.addListener(self.onAction.execute)
+
+    def appendItem(self, item):
+        self.insertItem(len(self.items), item)
