@@ -15,12 +15,15 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
+import logging
 import os
 import pygame
 
 import trosnoth.data.music as music
 from trosnoth.data import getPath
 
+log = logging.getLogger(__name__)
+
 # NO_MORE_MUSIC must not collide with any pygame events.
 NO_MORE_MUSIC = 234
 
@@ -30,7 +33,10 @@ class MusicManager(object):
     '''Manages the music.'''
 
     def __init__(self):
-        pygame.mixer.init(44100)
+        try:
+            pygame.mixer.init(44100)
+        except:
+            log.error('Could not initialise audio.')
         self.index = 0
         self.volume = 100
         self.filenames = []
@@ -49,6 +55,9 @@ class MusicManager(object):
         self.filenames.append(filename)
 
     def playMusic(self):
+        if not pygame.mixer.get_init():
+            return
+
         curFile = self.filenames[self.index]
         pygame.mixer.music.load(getPath(music, curFile))
         pygame.mixer.music.play(0)
@@ -68,12 +77,19 @@ class MusicManager(object):
         self.setVolume(self.volume)
 
     def stopMusic(self):
-        pygame.mixer.music.stop()
+        if pygame.mixer.get_init():
+            pygame.mixer.music.stop()
 
     def isMusicPlaying(self):
+        if not pygame.mixer.get_init():
+            return False
+
         return pygame.mixer.music.get_busy()
 
     def setVolume(self, volume):
+        if not pygame.mixer.get_init():
+            return
+
         self.volume = volume
         # Valid range is 0.0 - 1.0
         decimalVolume = volume / 100.0
