@@ -22,6 +22,10 @@ from trosnoth.data import getPath
 class SoundAction(object):
     def __init__(self, filename, startChannelIndex, numChannels):
         assert numChannels > 0
+        if not pygame.mixer.get_init():
+            self.channels = []
+            return
+
         self.sound = pygame.mixer.Sound(getPath(sound, filename))
         endChannelIndex = startChannelIndex+numChannels
         self.currentIndex = 0
@@ -30,6 +34,9 @@ class SoundAction(object):
         self.channelVolume = 1
 
     def play(self, volume=1):
+        if not pygame.mixer.get_init():
+            return
+
         finalVol = volume * self.channelVolume
         if finalVol < 0.01:
             return
@@ -55,6 +62,9 @@ class SoundPlayer(object):
         self._channelsUsed = 0
 
     def addSound(self, filename, action, maxSimultaneous = 8):
+        if not pygame.mixer.get_init():
+            return
+
         self.sounds[action] = SoundAction(filename, self._channelsUsed,
                 maxSimultaneous)
         # In case a sound is added after the volume has been set:
@@ -65,6 +75,9 @@ class SoundPlayer(object):
             pygame.mixer.set_num_channels(self._channelsUsed)
 
     def play(self, action, volume = 1):
+        if not pygame.mixer.get_init():
+            return
+
         self.sounds[action].play(volume)
 
     def setSoundVolume(self, action, val):
