@@ -45,8 +45,6 @@ default_game_keys = VirtualKeySet((
 
     ('abandon', pygame.K_m),
     ('chat', pygame.K_RETURN),
-    ('captain', pygame.K_c),
-    ('team ready', pygame.K_y),
     ('leaderboard', pygame.K_p),
     ('toggle interface', pygame.K_DELETE),
 
