@@ -64,6 +64,11 @@ class RemoveCollectableStarMsg(NetworkMessage):
     fields = 'starId', 'playerId'
     packspec = 'cc'
 
+class StarReboundedMsg(NetworkMessage):
+    idString = 'StRb'
+    fields = 'starId', 'xpos', 'ypos', 'xvel', 'yvel'
+    packspec = 'cffff'
+
 class PlayerUpdateMsg(NetworkMessage):
     '''
     attached may be:
@@ -110,8 +115,8 @@ class PlayerKilledMsg(NetworkMessage):
         O: Off the map mercy killing
     '''
     idString = 'Dead'
-    fields = 'targetId', 'killerId', 'shotId', 'stars', 'deathType'
-    packspec = 'ccIfc'
+    fields = 'targetId', 'killerId', 'shotId', 'stars', 'upgradeType', 'deathType'
+    packspec = 'ccIfcc'
 
 class PlayerHitMsg(NetworkMessage):
     '''
@@ -139,26 +144,21 @@ class ChangeNicknameMsg(NetworkMessage):
     fields = 'playerId', 'nickname'
     packspec = 'c*'
 
-class SetCaptainMsg(NetworkMessage):
-    idString = 'CapN'
-    fields = 'playerId'
-    packspec = 'c'
-
-class TeamIsReadyMsg(NetworkMessage):
-    idString = 'TRed'
-    fields = 'playerId'
-    packspec = 'c'
-
 class PlayerIsReadyMsg(NetworkMessage):
     idString = 'Redy'
-    fields = 'playerId', 'ready'
-    packspec = 'c?'
+    fields = 'playerId', 'ready', 'tournament'
+    packspec = 'c??'
 
 class SetPreferredTeamMsg(NetworkMessage):
     idString = 'TmNm'
     fields = 'playerId', 'name'
     packspec = 'c*'
 
+class PreferredTeamSelectedMsg(NetworkMessage):
+    idString = 'TmNm'
+    fields = 'playerId', 'tournament', 'name'
+    packspec = 'c?*'
+
 class SetPreferredSizeMsg(NetworkMessage):
     idString = 'Size'
     fields = 'playerId', 'halfMapWidth', 'mapHeight'
@@ -182,6 +182,11 @@ class GameOverMsg(NetworkMessage):
     fields = 'teamId', 'timeOver'
     packspec = 'cB'
 
+class ReturnToLobbyMsg(NetworkMessage):
+    idString = 'Loby'
+    fields = ()
+    packspec = ''
+
 class SetGameModeMsg(NetworkMessage):
     idString = 'Mode'
     fields = 'gameMode'
@@ -202,6 +207,16 @@ class StartingSoonMsg(NetworkMessage):
     fields = 'delay'
     packspec = 'd'
 
+class ChangeTimeLimitMsg(NetworkMessage):
+    idString = 'Time'
+    fields = 'timeLimit'
+    packspec = 'd'
+
+class ChangePlayerLimitsMsg(NetworkMessage):
+    idString = 'PlNo'
+    fields = 'maxPerTeam', 'maxTotal'
+    packspec = 'II'
+
 ####################
 # Players
 ####################
@@ -384,16 +399,18 @@ class FireShoxwaveMsg(NetworkMessage):
 ####################
 
 class DeleteUpgradeMsg(NetworkMessage):
+    '''
+    deleteReason may be:
+        'D' - player died
+        'T' - upgrade an out of time
+        'A' - abandoned
+        'S' - shield that was shot
+        'X' - removed by server
+    '''
     idString = 'DUpg'
     fields = 'playerId', 'upgradeType', 'deleteReason'
     packspec = 'ccc'
 
-# deleteReasons:
-# D: player died
-# T: upgrade ran out
-# A: abandoned
-# S: shield that was shot
-
 class BuyUpgradeMsg(NetworkMessage):
     '''
     Signal from interface that a buy has been requested.
@@ -411,10 +428,15 @@ class PlayerHasUpgradeMsg(NetworkMessage):
     Signal from validator that the player now has the specified upgrade.
     Recipients of this message should not decrement player star counts except
     due to receipt of a PlayerStarsSpentMsg.
+
+    reasonId may be:
+        'B' - upgrade was bought by player
+        'S' - upgrade was given by server
+        'X' - reason unknown
     '''
     idString = 'GotU'
-    fields = 'playerId', 'upgradeType'
-    packspec = 'cc'
+    fields = 'playerId', 'upgradeType', 'reasonId'
+    packspec = 'ccc'
 
 class PlayerStarsSpentMsg(NetworkMessage):
     idString = 'Spnt'
@@ -441,10 +463,27 @@ class CannotBuyUpgradeMsg(NetworkMessage):
     packspec = 'ccI'
 
 class GrenadeExplosionMsg(NetworkMessage):
-    idString = 'shot'
+    idString = 'BOOM'
     fields = 'xpos', 'ypos'
     packspec = 'ff'
 
+class GrenadeReboundedMsg(NetworkMessage):
+    idString = 'UhOh'
+    fields = 'playerId', 'xpos', 'ypos', 'xvel', 'yvel'
+    packspec = 'cffff'
+
+class UpgradeChangedMsg(NetworkMessage):
+    '''
+    A message for the clients that informs them of a change in an upgrade stat.
+    statType may be:
+        'S' - star cost
+        'T' - time limit
+        'X' - explosion radius
+    '''
+    idString = 'UpCh'
+    fields = 'upgradeType', 'statType', 'newValue'
+    packspec = 'ccf'
+
 ############################
 # Achievements
 ############################
@@ -453,6 +492,11 @@ class AchievementUnlockedMsg(NetworkMessage):
     idString = 'Achm'
     fields = 'playerId', 'achievementId'
     packspec = 'c*'
+    
+class AchievementProgressMsg(NetworkMessage):
+    idString = 'AchP'
+    fields = 'playerId', 'achievementId'
+    packspec = 'c*'
 
 ############################
 # Connection
@@ -477,6 +521,7 @@ tcpMessages = set([
     ChatMsg,
     GameStartMsg,
     GameOverMsg,
+    ReturnToLobbyMsg,
     SetGameModeMsg,
     SetTeamNameMsg,
     StartingSoonMsg,
@@ -505,4 +550,6 @@ tcpMessages = set([
     RemoveCollectableStarMsg,
     PlayerHasElephantMsg,
     ChangeNicknameMsg,
+    ChangeTimeLimitMsg,
+    UpgradeChangedMsg
 ])
