@@ -24,15 +24,17 @@ from trosnoth.messages import (ChatMsg, TaggingZoneMsg, PlayerStarsSpentMsg,
         KillShotMsg, RespawnRequestMsg, CannotRespawnMsg, RespawnMsg,
         PlayerKilledMsg, PlayerUpdateMsg, PlayerHitMsg, JoinRequestMsg,
         AddPlayerMsg, CannotJoinMsg, JoinSuccessfulMsg, UpdatePlayerStateMsg,
-        AimPlayerAtMsg, RemovePlayerMsg, SetCaptainMsg, TeamIsReadyMsg,
+        AimPlayerAtMsg, RemovePlayerMsg, PreferredTeamSelectedMsg,
         QueryWorldParametersMsg, SetWorldParametersMsg, WorldResetMsg,
         RequestMapBlockLayoutMsg, MapBlockLayoutMsg, ConnectionLostMsg,
         NotifyUDPStatusMsg, GrenadeExplosionMsg, ZoneStateMsg,
         AchievementUnlockedMsg, PlayerIsReadyMsg, SetPreferredTeamMsg,
         SetPreferredSizeMsg, SetPreferredDurationMsg, SetPlayerTeamMsg,
-        CreateCollectableStarMsg, RemoveCollectableStarMsg,
+        CreateCollectableStarMsg, RemoveCollectableStarMsg, ReturnToLobbyMsg,
         WorldParametersReceived, PlayerHasElephantMsg, ChangeNicknameMsg,
-        FireShoxwaveMsg, MarkZoneMsg, SetGameSpeedMsg)
+        FireShoxwaveMsg, MarkZoneMsg, SetGameSpeedMsg, AchievementProgressMsg,
+        ChangeTimeLimitMsg, ChangePlayerLimitsMsg, GrenadeReboundedMsg,
+        StarReboundedMsg, UpgradeChangedMsg)
 
 REQUEST_TIMEOUT = 2
 
@@ -100,6 +102,8 @@ class AgentFilter(Component):
 
     # Events from game to agent which are always relevant.
     @handler(PlayerHasElephantMsg, gameEvents)
+    @handler(ChangeTimeLimitMsg, gameEvents)
+    @handler(ChangePlayerLimitsMsg, gameEvents)
     @handler(DeleteUpgradeMsg, gameEvents)
     @handler(GameStartMsg, gameEvents)
     @handler(TaggingZoneMsg, gameEvents)
@@ -115,9 +119,9 @@ class AgentFilter(Component):
     @handler(PlayerKilledMsg, gameEvents)
     @handler(PlayerHitMsg, gameEvents)
     @handler(ShotAbsorbedMsg, gameEvents)
+    @handler(AchievementProgressMsg, gameEvents)
     @handler(GameOverMsg, gameEvents)
-    @handler(SetCaptainMsg, gameEvents)
-    @handler(TeamIsReadyMsg, gameEvents)
+    @handler(ReturnToLobbyMsg, gameEvents)
     @handler(StartingSoonMsg, gameEvents)
     @handler(SetTeamNameMsg, gameEvents)
     @handler(SetGameModeMsg, gameEvents)
@@ -131,7 +135,7 @@ class AgentFilter(Component):
     @handler(AchievementUnlockedMsg, gameEvents)
     @handler(WorldResetMsg, gameEvents)
     @handler(PlayerIsReadyMsg, gameEvents)
-    @handler(SetPreferredTeamMsg, gameEvents)
+    @handler(PreferredTeamSelectedMsg, gameEvents)
     @handler(SetPreferredSizeMsg, gameEvents)
     @handler(SetPreferredDurationMsg, gameEvents)
     @handler(CreateCollectableStarMsg, gameEvents)
@@ -140,6 +144,9 @@ class AgentFilter(Component):
     @handler(WorldParametersReceived, gameEvents)
     @handler(FireShoxwaveMsg, gameEvents)
     @handler(MarkZoneMsg, gameEvents)
+    @handler(GrenadeReboundedMsg, gameEvents)
+    @handler(StarReboundedMsg, gameEvents)
+    @handler(UpgradeChangedMsg, gameEvents)
     def passToAgent(self, msg):
         self.agentEvents.send(msg)
 
@@ -147,8 +154,6 @@ class AgentFilter(Component):
     # except checking for correct player id.
     @handler(ShootMsg, agentRequests)
     @handler(DeleteUpgradeMsg, agentRequests)
-    @handler(SetCaptainMsg, agentRequests)
-    @handler(TeamIsReadyMsg, agentRequests)
     @handler(UpdatePlayerStateMsg, agentRequests)
     @handler(AimPlayerAtMsg, agentRequests)
     @handler(ChatMsg, agentRequests)
