@@ -15,21 +15,24 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
+import logging
 import random
 from trosnoth.model.universe import Universe
+from trosnoth.model.gameController import GameController
 from trosnoth.model.map import MapLayout
 from trosnoth.messages import (AddPlayerMsg, QueryWorldParametersMsg,
         SetWorldParametersMsg, RequestMapBlockLayoutMsg, MapBlockLayoutMsg,
-        RequestPlayersMsg, PlayerUpdateMsg, RequestZonesMsg, SetCaptainMsg,
-        ZoneStateMsg, PlayerHasUpgradeMsg, TeamIsReadyMsg, WorldResetMsg,
-        WorldParametersReceived, SetPreferredTeamMsg, SetPreferredDurationMsg,
-        SetPreferredSizeMsg, PlayerIsReadyMsg, RequestGameModeMsg,
-        RequestGameSpeedMsg, SetGameSpeedMsg, SetGameModeMsg)
-from trosnoth.utils.components import (Component, Plug, queueMessage,
-        handler)
+        RequestPlayersMsg, PlayerUpdateMsg, RequestZonesMsg, ZoneStateMsg,
+        PlayerHasUpgradeMsg, WorldResetMsg, WorldParametersReceived,
+        PreferredTeamSelectedMsg, SetPreferredDurationMsg, SetPreferredSizeMsg,
+        PlayerIsReadyMsg, RequestGameModeMsg, RequestGameSpeedMsg,
+        SetGameSpeedMsg, SetGameModeMsg, SetTeamNameMsg)
+from trosnoth.utils.components import Component, Plug, queueMessage, handler
 from trosnoth.utils.twist import WeakCallLater
 from trosnoth.utils.unrepr import unrepr
 
+log = logging.getLogger(__name__)
+
 TIMEOUT = 5
 
 class Isolator(Component):
@@ -51,8 +54,8 @@ class Isolator(Component):
     def __init__(self, layoutDatabase, useRealWorld=True):
         Component.__init__(self)
         if useRealWorld:
-            self.world = Universe(layoutDatabase=layoutDatabase,
-                    clientOptimised=True)
+            self.world = Universe(clientOptimised=True)
+            self.gameController = GameController(layoutDatabase, self.world)
         else:
             self.world = None
         self.layoutDatabase = layoutDatabase
@@ -83,7 +86,7 @@ class Isolator(Component):
         data = unrepr(msg.settings)
         self.mapSpec = mapSpec = data.pop('worldMap')
         if self.world is not None:
-            self.world.applyWorldParameters(data)
+            self.gameController.applyWorldParameters(data)
 
             self.gotParams = True
             keys = MapLayout.unknownBlockKeys(self.layoutDatabase, mapSpec)
@@ -163,6 +166,7 @@ class Isolator(Component):
     def passToAgent(self, msg):
         if self.world is not None:
             queueMessage(self.world.orderPlug, msg)
+            queueMessage(self.gameController.orderPlug, msg)
         self.agentEvents.send(msg)
 
     @agentRequests.defaultHandler
@@ -184,8 +188,9 @@ class WorldSettingsResponder(Component):
     agentRequests = Plug()
     gameRequests = Plug()
 
-    def __init__(self, world, layoutDatabase):
+    def __init__(self, world, gameController, layoutDatabase):
         Component.__init__(self)
+        self.gameController = gameController
         self.world = world
         self.layoutDatabase = layoutDatabase
 
@@ -199,10 +204,13 @@ class WorldSettingsResponder(Component):
 
     @handler(QueryWorldParametersMsg, agentRequests)
     def queryWorldParams(self, msg):
-        data = self.world._getWorldParameters()
+        data = self.gameController._getWorldParameters()
         data['worldMap'] = self.world.map.layout.toString()
         self.agentEvents.send(SetWorldParametersMsg(msg.tag,
                 repr(data)))
+        for team in self.world.teams:
+            self.agentEvents.send(SetTeamNameMsg(team.id,
+                    team.teamName.encode('utf-8')))
 
     @handler(RequestMapBlockLayoutMsg, agentRequests)
     def queryMapBlock(self, msg):
@@ -225,20 +233,17 @@ class WorldSettingsResponder(Component):
                     player.currentZone.id, player.ghost, player.bot,
                     player.nick.encode()))
             self.agentEvents.send(player.makePlayerUpdate())
-            self.agentEvents.send(SetPreferredTeamMsg(player.id,
-                    player.preferredTeam.encode()))
-            self.agentEvents.send(SetPreferredSizeMsg(player.id,
-                    player.preferredSize[0], player.preferredSize[1]))
-            self.agentEvents.send(SetPreferredDurationMsg(player.id,
-                    player.preferredDuration))
-            self.agentEvents.send(PlayerIsReadyMsg(player.id,
-                    player.readyToStart))
+            if not player.bot:
+                self.agentEvents.send(PreferredTeamSelectedMsg(player.id,
+                        player.prefersTournamentTeam,
+                        player.preferredTeam.encode()))
+                self.agentEvents.send(SetPreferredSizeMsg(player.id,
+                        player.preferredSize[0], player.preferredSize[1]))
+                self.agentEvents.send(SetPreferredDurationMsg(player.id,
+                        player.preferredDuration))
+                self.agentEvents.send(PlayerIsReadyMsg(player.id,
+                        player.readyToStart, player.readyForTournament))
 
-        for team in self.world.teams:
-            if team.captain is not None:
-                self.agentEvents.send(SetCaptainMsg(team.captain.id))
-                if team.ready:
-                    self.agentEvents.send(TeamIsReadyMsg(team.captain.id))
         self.queryUpgrades()
 
     @handler(RequestZonesMsg, agentRequests)
@@ -252,7 +257,10 @@ class WorldSettingsResponder(Component):
             marks = ''
             for team, value in zone.markedBy.iteritems():
                 if value:
-                    marks += team.id
+                    if team is not None:
+                        marks += team.id
+                    else:
+                        marks += '\x00'
             self.agentEvents.send(ZoneStateMsg(zone.id, owner, dark, marks))
 
     @handler(RequestGameModeMsg, agentRequests)
@@ -267,4 +275,4 @@ class WorldSettingsResponder(Component):
         for player in self.world.players:
             if player.upgrade is not None:
                 self.agentEvents.send(PlayerHasUpgradeMsg(player.id,
-                        player.upgrade.upgradeType))
+                        player.upgrade.upgradeType, 'X'))
