@@ -270,31 +270,43 @@ class MapBlock(object):
         return self.defn.rect.collidepoint(*point)
 
     def _getBlockLeft(self):
-        i, j = self.defn.indices
-        if j == 0:
+        try:
+            i, j = self.defn.indices
+            if j == 0:
+                return None
+            return self.universe.zoneBlocks[i][j-1]
+        except IndexError:
             return None
-        return self.universe.zoneBlocks[i][j-1]
     blockLeft = property(_getBlockLeft)
 
     def _getBlockRight(self):
-        i, j = self.defn.indices
-        if j >= len(self.universe.zoneBlocks[i]) - 1:
+        try:
+            i, j = self.defn.indices
+            if j >= len(self.universe.zoneBlocks[i]) - 1:
+                return None
+            return self.universe.zoneBlocks[i][j+1]
+        except IndexError:
             return None
-        return self.universe.zoneBlocks[i][j+1]
     blockRight = property(_getBlockRight)
 
     def _getBlockAbove(self):
-        i, j = self.defn.indices
-        if i == 0:
+        try:
+            i, j = self.defn.indices
+            if i == 0:
+                return None
+            return self.universe.zoneBlocks[i-1][j]
+        except IndexError:
             return None
-        return self.universe.zoneBlocks[i-1][j]
     blockAbove = property(_getBlockAbove)
 
     def _getBlockBelow(self):
-        i, j = self.defn.indices
-        if i >= len(self.universe.zoneBlocks) - 1:
+        try:
+            i, j = self.defn.indices
+            if i >= len(self.universe.zoneBlocks) - 1:
+                return None
+            return self.universe.zoneBlocks[i+1][j]
+        except IndexError:
             return None
-        return self.universe.zoneBlocks[i+1][j]
     blockBelow = property(_getBlockBelow)
 
     def getZoneAtPoint(self, x, y):
@@ -315,7 +327,7 @@ class MapBlock(object):
             return
 
         shot.registeredHit = True
-        player.hurtBy(shooterId, shot.id, 'S')
+        player.hurtBy(shooterId, shot, 'S')
 
     def collideTrajectory(self, pt, origin, trajectory, tolerance):
         '''Returns true if pt lies within a distance of tolerance from the
