@@ -36,7 +36,9 @@ class PhysicsConstants(object):
 
     def _standard(self, pace=1, fireRate=1, gravity=1, jumpHeight=1,
             respawnRate=1, shotSpeed=1, shotLength=1, bounce=False,
-            respawnHealth=1):
+            respawnHealth=1, shooting=True):
+
+        self.shooting = shooting
 
         # Speed that shots travel at.
         self.shotSpeed = 600 * shotSpeed                      # pix/s
@@ -67,6 +69,9 @@ class PhysicsConstants(object):
         self.grenadeGravity = 300 * gravity
         self.grenadeInitVel = 400 * pace
 
+    def setModeNoShots(self):
+        self._standard(shooting=False)
+
     def setModeLightning(self):
         self._standard(pace=1.75, fireRate=2)
 
