@@ -261,7 +261,7 @@ class Corner(Obstaclish):
 def playerHitJumpable(self, player):
     '''This obstacle has been hit by player.'''
     if player.universe.physics.playerBounce:
-        player._jumpTime = player.maxJumpTime
+        player._jumpTime = player.universe.physics.playerMaxJumpTime
         player.yVel = -player.yVel
         player.detachFromEverything()
     else:
