@@ -56,7 +56,7 @@ class WorldPhysics(PhysicsConstants):
 
             # Check for change of zones.
             newZone = newBlock.getZoneAtPoint(newX, newY)
-            if not unit.attemptZoneEntry(newZone):
+            if not unit.canEnterZone(newZone):
                 return lastObstacle
 
         # Check for player/shot collisions along the path.
