@@ -22,12 +22,13 @@ import struct
 from trosnoth.utils.checkpoint import checkpoint
 
 from trosnoth.model.unit import Unit
-from trosnoth.model.universe_base import GameState
+from trosnoth.model.universe_base import NeutralTeamId, NoPlayer
 from trosnoth.model.map import MapLayout
 from trosnoth.utils.math import distance
 
 from trosnoth.messages import (DeleteUpgradeMsg, PlayerUpdateMsg,
-        SetPlayerTeamMsg, PlayerKilledMsg, PlayerHitMsg, ShotAbsorbedMsg)
+        SetPlayerTeamMsg, PlayerKilledMsg, PlayerHitMsg, ShotAbsorbedMsg,
+        AchievementProgressMsg)
 from trosnoth.utils.network import compress_boolean
 
 RESPAWN_CAMP_TIME = 1.0
@@ -93,7 +94,9 @@ class Player(Unit):
 
         # Preferences during the voting phase.
         self.preferredTeam = ''
+        self.prefersTournamentTeam = False
         self.readyToStart = False
+        self.readyForTournament = False
         self.preferredSize = (0, 0)
         self.preferredDuration = 0
 
@@ -200,7 +203,7 @@ class Player(Unit):
         return self.team == team
 
     def inRespawnableZone(self):
-        return self.team is None or self.currentZone.orbOwner == self.team
+        return (self.team is None or self.currentZone.orbOwner == self.team)
 
     @property
     def teamId(self):
@@ -214,16 +217,6 @@ class Player(Unit):
             return self.universe.rogueTeamName
         return self.team.teamName
 
-    def isTeamCaptain(self):
-        if self.team is None:
-            return False
-        return self.team.captain == self
-
-    def teamHasNoCaptain(self):
-        if self.team is None:
-            return False
-        return self.team.captain is None
-
     def isEnemyTeam(self, team):
         '''
         Returns True iff the given team is an enemy team of this player. It is
@@ -238,7 +231,7 @@ class Player(Unit):
         iu = self.invulnerableUntil
         if iu is None:
             return False
-        elif self.universe.getGameTime() > iu:
+        elif self.universe.getElapsedTime() > iu:
             self.invulnerableUntil = None
             return False
         return True
@@ -327,14 +320,13 @@ class Player(Unit):
                 'dead': dead,
                 'stars': stars,
                 'upgrade': upgrade,
+                'bot': self.bot
         }
 
     def removeFromGame(self):
         '''Called by network client when server says this player has left the
         game.'''
-        # Remove myself from the register of players.
-        if self.team is not None:
-            self.team.playerHasLeft(self)
+        
         if self.upgrade:
             self.upgrade.delete()
 
@@ -356,20 +348,21 @@ class Player(Unit):
     def continueOffMap(self):
         if not self.ghost:
             # Player is off the map: mercy killing.
-            self.deathDetected('\x00', 0, 'O')
+            self.deathDetected(NoPlayer, 0, 'O')
         return False
 
-    def attemptZoneEntry(self, newZone):
+    def canEnterZone(self, newZone):
         if newZone == self.currentZone:
             return True
         universe = self.universe
-        if (universe.gameState in (GameState.PreGame, GameState.Starting)
+        if (not universe.state.canLeaveFriendlyTerritory()
                 and newZone is None and self.ghost):
             # Pre-game ghost cannot enter purple zones.
             return False
 
-        if (newZone is not None and universe.gameState in (GameState.PreGame,
-                GameState.Starting) and newZone.orbOwner != self.team):
+        if (newZone is not None and
+                not universe.state.canLeaveFriendlyTerritory()
+                and newZone.orbOwner != self.team):
             # Disallowed zone change.
             return False
 
@@ -567,10 +560,6 @@ class Player(Unit):
             obstacle.hitByPlayer(self)
             self.universe.playerIsDirty(self.id)
 
-        # Check for zone tag.
-        if self.currentZone.orbOwner != self.team:
-            self.universe.checkTag(self)
-
         self._ignore = None
 
     def setAttachedObstacle(self, obstacle):
@@ -655,9 +644,6 @@ class Player(Unit):
             obstacle.hitByPlayer(self)
             self.universe.playerIsDirty(self.id)
 
-        # Check for zone tag.
-        if self.currentZone.orbOwner != self.team:
-            self.universe.checkTag(self)
         self._ignore = None
 
     def updateLivingPlayer(self, deltaT):
@@ -711,7 +697,12 @@ class Player(Unit):
         self._setStarsForDeath()
         checkpoint('Player: died')
 
-    def hurtBy(self, shooterId, shotId, deathType):
+    def hurtBy(self, shooterId, shot, deathType):
+        if shot is not None:
+            shotId = shot.id
+        else:
+            shotId = 0
+
         if self.phaseshift or self.turret:
             if shotId != 0:
                 self.universe.eventPlug.send(ShotAbsorbedMsg(self.id,
@@ -723,19 +714,33 @@ class Player(Unit):
                     self.upgrade.upgradeType, 'S'))
         elif self.health <= 1:
             self.deathDetected(shooterId, shotId, deathType)
+            if shotId != 0:
+                if shot.bounced:
+                    self.universe.eventPlug.send(
+                        AchievementProgressMsg(shooterId, 'ricochetKill'))
+
+                if shot.timeLeft < 0.1:
+                    self.universe.eventPlug.send(
+                        AchievementProgressMsg(shooterId, 'longRangeKill'))
         else:
             self.universe.eventPlug.send(PlayerHitMsg(self.id, shooterId,
                     shotId, deathType))
 
     def deathDetected(self, shooterId, shotId, deathType):
+        if self.upgrade is None:
+            upgradeType = "-"
+        else:
+            upgradeType = self.upgrade.upgradeType;
+
         self.universe.eventPlug.send(PlayerKilledMsg(self.id,
-                    shooterId, shotId, self.stars, deathType))
+                    shooterId, shotId, self.stars, upgradeType, deathType))
         if self.upgrade is not None and self.upgrade.goneWhenDie:
             self.universe.eventPlug.send(DeleteUpgradeMsg(self.id,
                     self.upgrade.upgradeType, 'D'))
 
-        if self.universe.gameState == GameState.Ended and self.team is not None:
-            self.universe.eventPlug.send(SetPlayerTeamMsg(self.id, '\x00'))
+        if self.universe.state.becomeNeutralWhenDie() and self.team is not None:
+            self.universe.eventPlug.send(SetPlayerTeamMsg(self.id,
+                    NeutralTeamId))
 
     def _setStarsForDeath(self):
         self.stars = self.universe.PLAYER_RESET_STARS
@@ -744,7 +749,8 @@ class Player(Unit):
         self.setAttachedObstacle(None)
         self.ghost = False
         self.health = self.universe.physics.playerRespawnHealth
-        self.invulnerableUntil = self.universe.getGameTime() + RESPAWN_CAMP_TIME
+        self.invulnerableUntil = (self.universe.getElapsedTime() +
+                RESPAWN_CAMP_TIME)
         self.respawnGauge = 0
         self.pos = self.currentZone.defn.pos
         i, j = MapLayout.getMapBlockIndices(self.pos[0], self.pos[1])
