@@ -44,8 +44,6 @@ class GrenadeShot(Unit):
         self.xVel = self.universe.physics.grenadeInitVel * sin(angle)
         self.yVel = -self.universe.physics.grenadeInitVel * cos(angle)
 
-        universe.addGrenade(self)
-
         try:
             block = player.currentMapBlock
         except IndexError, e:
@@ -55,9 +53,6 @@ class GrenadeShot(Unit):
         self.setMapBlock(block)
         checkpoint('Grenade created')
 
-    def delete(self):
-        self.universe.removeGrenade(self)
-
     def isSolid(self):
         return True
 
@@ -80,7 +75,7 @@ class GrenadeShot(Unit):
 
             obstacle = self.universe.physics.moveUnit(self, deltaX, deltaY)
             if obstacle is not None:
-                self.rebound(obstacle)
+                self.requestRebound(obstacle)
 
             # v = u + at
             vFinal = self.yVel + self.universe.physics.grenadeGravity * deltaT
@@ -102,7 +97,7 @@ class GrenadeShot(Unit):
         except Exception, e:
             log.exception(str(e))
 
-    def rebound(self, obstacle):
+    def requestRebound(self, obstacle):
         '''
         The grenade has hit an obstacle
         '''
@@ -111,8 +106,14 @@ class GrenadeShot(Unit):
         dif = shotAngle - obsAngle
         final = obsAngle - dif
         speed = sqrt(self.xVel ** 2 + self.yVel **2) * 0.9
-        self.xVel = speed * cos(final)
-        self.yVel = speed * sin(final)
+        xVel = speed * cos(final)
+        yVel = speed * sin(final)
+        self.universe.sendGrenadeRebound(self.player, self.pos, xVel, yVel)
+
+    def doRebound(self, reboundMessage):
+        self.pos = (reboundMessage.xpos, reboundMessage.ypos)
+        self.xVel = reboundMessage.xvel
+        self.yVel = reboundMessage.yvel
 
 
 class Shot(Unit):
