@@ -39,7 +39,7 @@ class CollectableStar(Unit):
         self.universe = universe
         self.id = id
         self.team = team
-        self.creationTime = self.universe.getGameTime()
+        self.creationTime = self.universe.getElapsedTime()
         self.registeredHit = False
 
         # Place myself.
@@ -95,14 +95,13 @@ class CollectableStar(Unit):
         '''Called by this player's universe when this player should update
         its position. deltaT is the time that's passed since its state was
         current, measured in seconds.'''
-
         try:
             deltaX = self.xVel * deltaT
             deltaY = self.yVel * deltaT
 
             obstacle = self.universe.physics.moveUnit(self, deltaX, deltaY)
             if obstacle is not None:
-                self.rebound(obstacle)
+                self.requestRebound(obstacle)
 
             # v = u + at
             vFinal = self.yVel + self.gravity * deltaT
@@ -121,11 +120,17 @@ class CollectableStar(Unit):
         except Exception, e:
             log.exception(str(e))
 
-    def rebound(self, obstacle):
+    def requestRebound(self, obstacle):
         obsAngle = obstacle.getAngle()
         shotAngle = atan2(self.yVel, self.xVel)
         dif = shotAngle - obsAngle
         final = obsAngle - dif
         speed = sqrt(self.xVel ** 2 + self.yVel **2) * 0.9
-        self.xVel = speed * cos(final)
-        self.yVel = speed * sin(final)
+        xVel = speed * cos(final)
+        yVel = speed * sin(final)
+        self.universe.sendStarRebound(self, xVel, yVel)
+
+    def gotRebound(self, msg):
+        self.pos = msg.xpos, msg.ypos
+        self.xVel = msg.xvel
+        self.yVel = msg.yvel
\ No newline at end of file
