@@ -21,8 +21,6 @@ class Team(object):
         self.world = world
         self.numOrbsOwned = 0
 
-        self.captain = None
-        self.ready = False
         self.usingMinimapDisruption = False
 
         if (not isinstance(teamID, str)) or len(teamID) != 1:
@@ -31,8 +29,10 @@ class Team(object):
 
         if teamID == 'A':
             self.teamName = 'Blue players'
-        else:
+        elif teamID == 'B':
             self.teamName = 'Red players'
+        else:
+            self.teamName = '%s Team' % (teamID,)
 
     def __str__(self):
         return self.teamName
@@ -49,22 +49,6 @@ class Team(object):
         '''Returns whether the team has lost the game (True = Lost Game)'''
         return self.numOrbsOwned == 0
 
-    def canSetCaptain(self):
-        return self.captain == None
-
-    def setCaptain(self, player):
-        assert(self.canSetCaptain())
-        assert(player.team == self)
-        self.captain = player
-
-    def teamReady(self):
-        self.ready = True
-
-    def playerHasLeft(self, player):
-        if self.captain == player:
-            self.captain = None
-            self.ready = False
-
     @staticmethod
     def setOpposition(teamA, teamB):
         teamA.opposingTeam = teamB
