@@ -41,7 +41,7 @@ class Unit(object):
         '''
         return True
 
-    def attemptZoneEntry(self, zone):
+    def canEnterZone(self, zone):
         '''
         Called when this unit attempts to enter the given zone. Should return
         True or False to indicate whether this zone entry is allowed.
