@@ -25,7 +25,6 @@ import random
 from trosnoth.utils.utils import timeNow
 from trosnoth.utils.math import distance
 from trosnoth.utils.checkpoint import checkpoint
-from trosnoth.model.universe_base import GameState, GameRules
 from trosnoth.model.upgrades import upgradeOfType, allUpgrades
 from trosnoth.model.physics import WorldPhysics
 
@@ -38,29 +37,24 @@ from trosnoth.model.team import Team
 # Component message passing
 from trosnoth.utils.components import Component, handler, Plug
 from trosnoth.utils.network import expand_boolean
-from trosnoth.utils.twist import WeakCallLater, WeakLoopingCall
-from trosnoth.messages import (ChatMsg, TaggingZoneMsg, ShotFiredMsg,
-        PlayerUpdateMsg, KillShotMsg, CannotRespawnMsg, RespawnMsg,
-        PlayerKilledMsg, PlayerHitMsg, GameStartMsg, GameOverMsg, SetCaptainMsg,
-        TeamIsReadyMsg, StartingSoonMsg, SetTeamNameMsg, SetGameModeMsg,
-        AddPlayerMsg, CannotJoinMsg, JoinSuccessfulMsg, UpdatePlayerStateMsg,
-        AimPlayerAtMsg, RemovePlayerMsg, PlayerHasUpgradeMsg, ShotAbsorbedMsg,
-        PlayerStarsSpentMsg, DeleteUpgradeMsg, CannotBuyUpgradeMsg,
-        SetPlayerTeamMsg, ConnectionLostMsg, GrenadeExplosionMsg,
+from trosnoth.utils.twist import WeakCallLater
+from trosnoth.messages import (TaggingZoneMsg, ShotFiredMsg, PlayerUpdateMsg,
+        KillShotMsg, RespawnMsg, PlayerKilledMsg, PlayerHitMsg, GameStartMsg,
+        GameOverMsg, SetTeamNameMsg, SetGameModeMsg, AddPlayerMsg,
+        JoinSuccessfulMsg, UpdatePlayerStateMsg, AimPlayerAtMsg,
+        RemovePlayerMsg, PlayerHasUpgradeMsg, ShotAbsorbedMsg,
+        PlayerStarsSpentMsg, DeleteUpgradeMsg, SetPlayerTeamMsg,
         AchievementUnlockedMsg, ZoneStateMsg, WorldResetMsg,
-        QueryWorldParametersMsg, PlayerIsReadyMsg, SetPreferredTeamMsg,
-        SetPreferredDurationMsg, SetPreferredSizeMsg, CreateCollectableStarMsg,
-        RemoveCollectableStarMsg, PlayerHasElephantMsg, ChangeNicknameMsg,
-        FireShoxwaveMsg, MarkZoneMsg, SetGameSpeedMsg, NotifyUDPStatusMsg)
+        CreateCollectableStarMsg, RemoveCollectableStarMsg,
+        PlayerHasElephantMsg, ChangeNicknameMsg, FireShoxwaveMsg, MarkZoneMsg,
+        SetGameSpeedMsg, GrenadeReboundedMsg, StarReboundedMsg,
+        UpgradeChangedMsg)
 
 DIRTY_DELAY = 0.05
 UPDATE_DELAY = 1        # PlayerUpdateMsg is sent for each player this often.
-TICK_PERIOD = 0.02
 
 DEFAULT_TEAM_NAME_1 = 'Defaulticons'
 DEFAULT_TEAM_NAME_2 = 'Standardators'
-
-GAME_STATE_CHECK_PERIOD = 5
 STALEMATE_CHECK_PERIOD = 1
 COLLECTABLE_STAR_LIFETIME = 12
 NO_COLLECTABLE_STARS = False        # For debugging.
@@ -93,9 +87,8 @@ class Universe(Component):
 
     PLAYER_RESET_STARS = 0
 
-    def __init__(self, halfMapWidth=3, mapHeight=2, gameDuration=0,
-            authTagManager=None, voting=False, layoutDatabase=None,
-            clientOptimised=False, gameName=None, onceOnly=False):
+    def __init__(self, halfMapWidth=3, mapHeight=2, authTagManager=None,
+            clientOptimised=False, gameName=None):
         '''
         halfMapWidth:   is the number of columns of zones in each team's
                         territory at the start of the game. There will always
@@ -114,28 +107,17 @@ class Universe(Component):
         '''
         super(Universe, self).__init__()
 
+        self.frameRate = 0
         self.playerWithElephant = None
         self.gameName = gameName
         self.clientOptimised = clientOptimised
         self.physics = WorldPhysics(self)
-        self.gameDuration = gameDuration
-        self.layoutDatabase = layoutDatabase
-        self.onceOnly = onceOnly
         if authTagManager is None:
             self.authManager = None
         else:
             self.authManager = authTagManager.authManager
 
-        # Initialise
-        if voting:
-            self.gameState = GameState.Ended
-        else:
-            self.gameState = GameState.PreGame
-        self.gameRules = GameRules.Normal
-        self.winningTeam = None
-        self.zonesToReset = []
         self.playerWithId = {}
-        self.huntTeamWithZones = None
         self.teamWithId = {'\x00' : None}
 
         # Create Teams:
@@ -156,35 +138,37 @@ class Universe(Component):
         self.zones = self.map.zones
         self.zoneBlocks = self.map.zoneBlocks
 
+        self._state = None
+
         self.players = set()
         self.playerUpdateTimes = {}
         self.grenades = set()
         self.collectableStars = {}      # starId -> CollectableStar
         self.shots = set()
-        self.gameOverTime = None
         self.gameMode = 'Normal'
         self.rogueTeamName = 'Rogue'
-
-        self._lastTime = timeNow()
-        self._gameTime = 0.0
+        self._elapsedTime = 0.0
         self._gameSpeed = 1.0
-        self._lastStalemateCheck = self._gameTime
-        self._lastGameStateCheck = self._gameTime
-        self.lastZoneTagged = self._gameTime
-        self._loop = None
-        self._startClock()
+        self._lastStalemateCheck = self._elapsedTime
+        self.lastZoneTagged = self._elapsedTime
+        self._gameInProgress = False
+        self._winningTeam = None
 
-    @handler(CannotBuyUpgradeMsg, orderPlug)
-    @handler(CannotJoinMsg, orderPlug)
-    @handler(CannotRespawnMsg, orderPlug)
-    @handler(ConnectionLostMsg, orderPlug)
-    @handler(ChatMsg, orderPlug)
-    @handler(GrenadeExplosionMsg, orderPlug)
-    @handler(QueryWorldParametersMsg, orderPlug)
-    @handler(NotifyUDPStatusMsg, orderPlug)
+    @orderPlug.defaultHandler
     def ignore(self, msg):
         pass
 
+    def getElapsedTime(self):
+        return self._elapsedTime
+
+    # Called when we receive parameters of the game
+    # from the server
+    def _setElapsedTime(self, time):
+        self._elapsedTime = time
+
+    # TODO: this seems a bit out of place?
+    # Marking zones doesn't seem to be related to
+    # positions of everyone on the map.
     @handler(MarkZoneMsg, orderPlug)
     def markZone(self, msg):
         try:
@@ -195,6 +179,7 @@ class Universe(Component):
         except Abort:
             pass
 
+    # TODO: this seems out-of-place
     @handler(AchievementUnlockedMsg, orderPlug)
     def achievementUnlocked(self, msg):
         try:
@@ -204,17 +189,6 @@ class Universe(Component):
         except Abort:
             pass
 
-    def reset(self, layout, gameDuration=None):
-        if gameDuration is not None:
-            self.gameDuration = gameDuration
-        self.setLayout(layout)
-
-        for player in self.players:
-            self._resetPlayer(player)
-
-        self.gameState = GameState.PreGame
-        self.eventPlug.send(WorldResetMsg())
-
     def _resetPlayer(self, player):
         zone = self.selectZone(player.teamId)
         player.pos = zone.defn.pos
@@ -228,7 +202,7 @@ class Universe(Component):
             block = self.zoneBlocks[i][j]
         except IndexError, e:
             log.exception(str(e))
-            raise IndexError, 'player start position is off the map'
+            return
         player.setMapBlock(block)
 
     def selectZone(self, teamId):
@@ -250,18 +224,15 @@ class Universe(Component):
     def gotWorldReset(self, msg):
         self.grenades = set()
         self.shots = set()
-        self.gameOverTime = None
-        self._gameTime = 0.0
-        self._lastStalemateCheck = self._gameTime
-        self._lastGameStateCheck = self._gameTime
-        self.lastZoneTagged = self._gameTime
-        self.gameState = GameState.PreGame
+        self._elapsedTime = 0.0
+        self._lastStalemateCheck = self._elapsedTime
+        self.lastZoneTagged = self._elapsedTime
 
     @handler(JoinSuccessfulMsg, orderPlug)
     def joinSuccessful(self, msg):
         try:
             player = self.getPlayer(msg.playerId)
-            if self.authManager is not None:
+            if self.authManager is not None and not player.bot:
                 player.user = self.authManager.getUserByName(msg.username)
         except Abort:
             pass
@@ -367,95 +338,26 @@ class Universe(Component):
             except Abort():
                 pass
 
-    def getTimeLeft(self):
-        if self.gameState in (GameState.PreGame, GameState.Starting):
-            return None
-        elif self.gameState == GameState.InProgress:
-            return self.gameDuration - self._gameTime
-        elif self.gameState == GameState.Ended:
-            return self.gameDuration - self.gameOverTime
-
     @handler(GameOverMsg, orderPlug)
     def gameOver(self, msg):
-        self.gameOverTime = self._gameTime
-        self.gameState = GameState.Ended
-        self.winningTeam = self.teamWithId.get(msg.teamId, None)
         for player in self.players:
             player.readyToStart = False
+            player.readyForTournament = False
             if player.ghost and player.team is not None:
                 # Give a chance for stats to save
                 WeakCallLater(0.5, self.eventPlug, 'send',
                         SetPlayerTeamMsg(player.id, '\x00'))
 
-    def setHuntedMode(self, winningTeam):
-        if winningTeam == None:
-            huntTeamWithZones = random.choice(self.teams)
-        else:
-            huntTeamWithZones = winningTeam
-        self.setHuntTeamWithZones(huntTeamWithZones)
-
-    def setHuntTeamWithZones(self, huntTeamWithZones):
-        log.debug('Trying to hunt with %s' % (huntTeamWithZones,))
-        # Firstly, if somehow all players are on the one zone, swap one to the
-        # other team
-        if len(self.players) < 4:
-            return
-        allTeams = set([])
-        for player in self.players:
-            allTeams.add(player.team)
-        if len(allTeams) == 0:
-            # No players in the game
-            pass
-        elif len(allTeams) == 1:
-            player = random.choice(self.players)
-            team = list(allTeams)[0].opposingTeam
-            self.eventPlug.send(SetPlayerTeamMsg(player.id, team.id))
-            WeakCallLater(3, self, 'setHuntedMode', team)
-        else:
-            # Now, check by who is alive and dead on each team
-            playersWithoutZones = 0
-            playersWithZones = 0
-            playersToSpawn = []
-            playersToSwapTeam = []
-            for player in self.players:
-                if player.ghost:
-                    playersToSpawn.append(player)
-                    if player.team == huntTeamWithZones:
-                        playersToSwapTeam.append(player)
-                else:
-                    if player.team != huntTeamWithZones:
-                        playersWithoutZones += 1
-                    else:
-                        playersWithZones += 1
-            if playersWithoutZones < 2: # Having only one player won't work
-                WeakCallLater(3, self, 'setHuntedMode',
-                        huntTeamWithZones.opposingTeam)
-            else:
-                log.debug('Hunting!')
-                self.huntTeamWithZones = huntTeamWithZones
-                self.gameRules = GameRules.Hunting
-                if playersWithZones == 0:
-                    # We're about to swap everyone onto the one team. Stop this!
-                    playersToSwapTeam.remove(random.choice(playersToSwapTeam))
-
-                for p in playersToSwapTeam:
-                    self.eventPlug.send(SetPlayerTeamMsg(p.id,
-                            huntTeamWithZones.opposingTeam.id))
-                for p in playersToSpawn:
-                    self.eventPlug.send(RespawnMsg(p.id, player.currentZone.id))
-                for zone in self.zones:
-                    self.eventPlug.send(ZoneStateMsg(zone.id,
-                            self.huntTeamWithZones.id, True, ''))
-
+        self._gameInProgress = False
 
+    def gameIsInProgress(self):
+        return self._gameInProgress
 
     @handler(GameStartMsg, orderPlug)
     def gameStart(self, msg):
-        time = msg.timeLimit
-        self.gameDuration = time
-        self.lastZoneTagged = self.startTime = self._gameTime
-        self.gameState = GameState.InProgress
-        self._gameTime = 0
+        self.lastZoneTagged = self.startTime = self._elapsedTime
+        self._elapsedTime = 0
+        self._gameInProgress = True
 
     @handler(AddPlayerMsg, orderPlug)
     def addPlayer(self, msg):
@@ -526,28 +428,14 @@ class Universe(Component):
         except Abort:
             pass
 
-    def _startClock(self):
-        if self._loop is not None:
-            self._loop.stop()
-        self._loop = WeakLoopingCall(self, 'tick')
-        self._loop.start(TICK_PERIOD, False)
-
-    def stopClock(self):
-        if self._loop is not None:
-            self._loop.stop()
-            self._loop = None
-
-    def tick(self):
-        '''Advances all players and shots to their new positions.'''
-        t = timeNow()
-        deltaT = t - self._lastTime
+    def advanceEverything(self, deltaT):
+        '''Advance the state of the game by deltaT seconds'''
         try:
             self.frameRate = 1. / deltaT
         except ZeroDivisionError:
             self.frameRate = 0
-        self._lastTime = t
         gameDeltaT = (deltaT * self._gameSpeed)
-        self._gameTime += gameDeltaT
+        self._elapsedTime += gameDeltaT
 
         # Update the player and shot positions.
         for s in (list(self.shots) + list(self.players) + list(self.grenades) +
@@ -555,53 +443,80 @@ class Universe(Component):
             s.reset()
             s.update(gameDeltaT)
 
-        for zone in self.zonesToReset:
-            zone.taggedThisTime = False
-        self.zonesToReset = []
-
         if not self.clientOptimised:
-            self.checkForResult()
+            self.updateZoneOwnership()
             self.processPlayerUpdates()
-            self.processGameState()
             self.checkStalemate()
+            self.updateElephant()
             self.updateCollectableStars()
 
-    def getGameTime(self):
-        '''The amount of time since the start of the game.
-        This is influnced by any game speed factors
-        '''
-        return self._gameTime
 
     def updateCollectableStars(self):
-        for star in self.collectableStars.values():
-            if star.creationTime < self._gameTime - COLLECTABLE_STAR_LIFETIME:
-                self.eventPlug.send(RemoveCollectableStarMsg(star.id, '\x00'))
+        try:
+            for star in self.collectableStars.values():
+                if (star.creationTime < self._elapsedTime -
+                        COLLECTABLE_STAR_LIFETIME):
+                    self.eventPlug.send(RemoveCollectableStarMsg(star.id, '\x00'))
+        except:
+            log.error('Error updating collectable stars', exc_info=True)
+
+    def collectableStarsCanAppear(self):
+        return not NO_COLLECTABLE_STARS and self._gameInProgress
+
+    def canBuyUpgrades(self):
+        return self.state.canBuyUpgrades()
+
+    def canRespawn(self):
+        return self.state.canRespawn()
+
+    def becomeNeutralWhenJoinOrDie(self):
+        return self.state.becomeNeutralWhenJoinOrDie()
+
+    def canShoot(self):
+        return self.state.canShoot()
+
+    def canRename(self):
+        return self.state.canRename()
+
+    def setWinningTeamById(self, teamId):
+        self._winningTeam = self.teamWithId.get(teamId, None)
+
+    def getWinningTeam(self):
+        return self._winningTeam
 
     def checkStalemate(self):
-        if NO_COLLECTABLE_STARS:
-            return
-        if self._gameTime < self._lastStalemateCheck + STALEMATE_CHECK_PERIOD:
-            return
-        self._lastStalemateCheck = self._gameTime
+        try:
+            if NO_COLLECTABLE_STARS:
+                return
+            if (self._elapsedTime < self._lastStalemateCheck +
+                    STALEMATE_CHECK_PERIOD):
+                return
+            self._lastStalemateCheck = self._elapsedTime
 
-        ePlayerId = '\x00'
-        if self.playerWithElephant:
-            ePlayerId = self.playerWithElephant.id
-        if isinstance(ePlayerId, str) and len(ePlayerId) == 1:
-            self.eventPlug.send(PlayerHasElephantMsg(ePlayerId))
-        else:
-            log.error('ePlayerId is %r but should be single char', ePlayerId)
+            if not self.collectableStarsCanAppear():
+                return
 
-        if self.gameState != GameState.InProgress:
-            return
+            prob = 0.5 ** ((self._elapsedTime - self.lastZoneTagged) /
+                    ZONE_TAG_STAR_HALF_LIFE)
+            if random.random() > prob:
+                team = random.choice(self.teams)
+                zone = self.selectZone(team.id)
+                x, y = zone.defn.randomPosition()
+                self.eventPlug.send(CreateCollectableStarMsg('0', team.id, x, y))
+        except:
+            log.error('Error checking for stalemate', exc_info=True)
 
-        prob = 0.5 ** ((self._gameTime - self.lastZoneTagged) /
-                ZONE_TAG_STAR_HALF_LIFE)
-        if random.random() > prob:
-            team = random.choice(self.teams)
-            zone = self.selectZone(team.id)
-            x, y = zone.defn.randomPosition()
-            self.eventPlug.send(CreateCollectableStarMsg('0', team.id, x, y))
+    def updateElephant(self):
+        try:
+            ePlayerId = '\x00'
+            if self.playerWithElephant:
+                ePlayerId = self.playerWithElephant.id
+            if isinstance(ePlayerId, str) and len(ePlayerId) == 1:
+                self.eventPlug.send(PlayerHasElephantMsg(ePlayerId))
+            else:
+                log.error('ePlayerId is %r but should be single char', ePlayerId)
+        except:
+            log.error('Error updating elephant', exc_info=True)
 
     @handler(CreateCollectableStarMsg, orderPlug)
     def createCollectableStar(self, msg):
@@ -625,214 +540,78 @@ class Universe(Component):
             pass
 
     def processPlayerUpdates(self):
-        playerIds = set(self.playerWithId.keys())
-        now = timeNow()
-        for pId, time in list(self.playerUpdateTimes.iteritems()):
-            if pId not in playerIds:
-                del self.playerUpdateTimes[pId]
-                continue
-            playerIds.remove(pId)
-            if now > time:
-                self._sendPlayerUpdate(pId)
+        try:
+            playerIds = set(self.playerWithId.keys())
+            now = timeNow()
+            for pId, time in list(self.playerUpdateTimes.iteritems()):
+                if pId not in playerIds:
+                    del self.playerUpdateTimes[pId]
+                    continue
+                playerIds.remove(pId)
+                if now > time:
+                    self._sendPlayerUpdate(pId)
 
-        for pId in playerIds:
-            # Not yet in dict.
-            self.playerUpdateTimes[pId] = now + UPDATE_DELAY
+            for pId in playerIds:
+                # Not yet in dict.
+                self.playerUpdateTimes[pId] = now + UPDATE_DELAY
+        except:
+            log.error('Error in player updates', exc_info=True)
 
-    def checkForResult(self):
-        if not self.gameState == GameState.InProgress:
-            return
-
-        # Check first for timeout
-        if self.gameDuration > 0 and self.getTimeLeft() <= 0:
-            if self.teams[0].numOrbsOwned > self.teams[1].numOrbsOwned:
-                self.eventPlug.send(GameOverMsg(self.teams[0].id, True))
-                checkpoint('Universe: Out of time, blue wins')
-            elif self.teams[1].numOrbsOwned > self.teams[0].numOrbsOwned:
-                self.eventPlug.send(GameOverMsg(self.teams[1].id, True))
-                checkpoint('Universe: Out of time, red wins')
-            else:
-                self.eventPlug.send(GameOverMsg('\x00', True))
-                checkpoint('Universe: Out of time, game drawn')
-            return
+    def updateZoneOwnership(self):
+        try:
+            for zone in self.zones:
+                result = zone.playerWhoTaggedThisZone()
+                if result != None:
+                    tagger, team = result
+                    if tagger == None:
+                        assert team == None
+                        self.eventPlug.send(TaggingZoneMsg(zone.id, '\x00', '\x00'))
+                    else:
+                        self.eventPlug.send(TaggingZoneMsg(zone.id, tagger.id,
+                                tagger.team.id))
+                    self._killTurret(tagger, zone)
+        except:
+            log.error('Error in zone ownership check', exc_info=True)
 
+    def teamWithAllZones(self):
         # Now check for an all zones win.
         team2Wins = self.teams[0].isLoser()
         team1Wins = self.teams[1].isLoser()
         if team1Wins and team2Wins:
             # The extraordinarily unlikely situation that all
             # zones have been neutralised in the same tick
-            self.eventPlug.send(GameOverMsg('\x00', False))
+            return 'Draw'
             checkpoint('Universe: Draw due to all zones neutralised')
         elif team1Wins:
-            self.eventPlug.send(GameOverMsg(self.teams[0].id, False))
+            return self.teams[0]
             checkpoint('Universe: Zones captured, blue wins')
         elif team2Wins:
-            self.eventPlug.send(GameOverMsg(self.teams[1].id, False))
+            return self.teams[1]
             checkpoint('Universe: Zones captured, red wins')
-
-    def processGameState(self):
-        if self.gameState != GameState.Ended or self.onceOnly:
-            return
-        if self._gameTime < self._lastGameStateCheck + GAME_STATE_CHECK_PERIOD:
-            return
-        self._lastGameStateCheck = self._gameTime
-        if self.layoutDatabase is None:
-            return
-
-        readyPlayerCount = 0
-        actualPlayerNum = 0
-        for player in self.players:
-            if not player.bot:
-                actualPlayerNum += 1
-                if player.readyToStart:
-                    readyPlayerCount += 1
-        if actualPlayerNum <= 1:
-            return
-        if not readyPlayerCount >= 0.8 * actualPlayerNum:
-            return
-
-        result = self._getNewTeams()
-        if result is None:
-            return
-        teamName1, players1, teamName2, players2 = result
-
-        size = self._getNewSize(min(len(players1), len(players2)))
-        duration = self._getNewDuration(size)
-
-        # Set team names.
-        self.teams[0].teamName = teamName1
-        self.teams[1].teamName = teamName2
-
-        # Set player teams.
-        for player in players1:
-            team = self.teams[0]
-            self.eventPlug.send(SetPlayerTeamMsg(player.id, team.id))
-            player.team = team
-        for player in players2:
-            team = self.teams[1]
-            self.eventPlug.send(SetPlayerTeamMsg(player.id, team.id))
-            player.team = team
-
-        layout = self.layoutDatabase.generateRandomMapLayout(size[0], size[1])
-        self.reset(layout, duration)
-        WeakCallLater(0.1, self, '_startSoon', 12)
-
-    def _getNewDuration(self, mapSize):
-        results = {}
-        for player in self.players:
-            if not player.bot:
-                duration = player.preferredDuration
-                results[duration] = results.get(duration, 0) + 1
-        items = results.items()
-        items.sort(key=lambda (duration, count): count)
-        items.reverse()
-        duration, count = items[0]
-        if results.get(0, 0) != count:
-            return duration
-
-        # Decide default based on map size.
-        if mapSize == (3, 2):
-            return 45 * 60
-        elif mapSize == (1, 1):
-            return 10 * 60
-        elif mapSize == (5, 1):
-            return 20 * 60
         else:
-            return min(7200, 2 * 60 * (mapSize[0]*2+1) * (mapSize[1]*1.5))
+            return None
 
-    def _getNewSize(self, teamSize):
-        '''
-        Returns a new map size based on what players vote for, with the
-        defaults (if most people select Auto) being determined by the size of
-        the smaller team.
-        '''
-        results = {}
-        for player in self.players:
-            if not player.bot:
-                size = player.preferredSize
-                results[size] = results.get(size, 0) + 1
-        items = results.items()
-        items.sort(key=lambda (size, count): count)
-        items.reverse()
-        size, count = items[0]
-        if results.get((0, 0), 0) != count:
-            return size
-
-        # Decide size based on player count.
-        if teamSize <= 3:
-            return (1, 1)
-        elif teamSize <= 4:
-            return (5, 1)
+    def teamWithMoreZones(self):
+        if self.teams[0].numOrbsOwned > self.teams[1].numOrbsOwned:
+            return self.teams[0]
+        elif self.teams[1].numOrbsOwned > self.teams[0].numOrbsOwned:
+            return self.teams[1]
         else:
-            return (3, 2)
+            return None
 
-    def _getNewTeams(self):
+    def getTeamPlayerCounts(self):
         '''
-        Returns (teamName1, players1, teamName2, players2) based on what teams
-        people have selected as their preferred teams.
+        Returns a mapping from team id to number of players currently on that
+        team.
         '''
-        desiredTeams = self._getDesiredTeams()
-        if len(desiredTeams) == 1:
-            if desiredTeams[0][0] != '':
-                return # Everyone is on one team.
-            teamName1 = ''
-            teamName2 = ''
-            players1 = []
-            players2 = []
-            others = list(self.players)
-        else:
-            fairLimit = (len(self.players) + 1) / 2
-            teamName1, players1 = desiredTeams[0]
-            if len(players1) > fairLimit:
-                # Require every player on the disadvantaged team to be ready.
-                for player in self.players:
-                    if player.preferredTeam != teamName1:
-                        if not player.readyToStart:
-                            return
-            teamName2, players2 = desiredTeams[1]
-            others = []
-            for teamName, players in desiredTeams[2:]:
-                others.extend(players)
-
-        if teamName1 == '':
-            if teamName2 != DEFAULT_TEAM_NAME_1:
-                teamName1 = DEFAULT_TEAM_NAME_1
-            else:
-                teamName1 = DEFAULT_TEAM_NAME_2
-        if teamName2 == '':
-            if teamName1 != DEFAULT_TEAM_NAME_1:
-                teamName2 = DEFAULT_TEAM_NAME_1
-            else:
-                teamName2 = DEFAULT_TEAM_NAME_2
-
-        random.shuffle(others)
-        for player in others:
-            count1 = len(players1)
-            count2 = len(players2)
-            if count1 > count2:
-                players2.append(player)
-            elif count2 > count1:
-                players1.append(player)
-            else:
-                random.choice([players1, players2]).append(player)
-
-        return teamName1, players1, teamName2, players2
-
-    def _getDesiredTeams(self):
-        '''
-        Returns a sorted sequence of duples of the form (teamName, players)
-        where teamName is a unicode/string and players is a list of players. The
-        sequence will be sorted from most popular to least popular.
-        '''
-        results = {}
+        playerCounts = {}
         for player in self.players:
-            teamName = player.preferredTeam
-            results.setdefault(teamName, []).append(player)
-        items = results.items()
-        items.sort(key=lambda (teamName, players): len(players))
-        items.reverse()
-        return items
+            playerCounts[player.teamId] = playerCounts.get(player.teamId, 0
+                    ) + 1
+        return playerCounts
+
+    def getTeamToJoin(self, preferredTeamId):
+        return self.state.getTeamToJoin(preferredTeamId, self)
 
     def getTeamName(self, id):
         if id == '\x00':
@@ -990,81 +769,9 @@ class Universe(Component):
         except KeyError:
             return None
 
-    def checkTag(self, tagger):
-        '''Checks to see if the player has touched the orb of its currentZone
-        If so, it sends a TaggingZoneMsg'''
-        if self.clientOptimised:
-            return
-        if tagger.team is None:
-            return
-
-        # How zone tagging works (more or less)
-        # 1. In its _move procedure, if a player is local, it will call checkTag
-        # 2. If it is close enough to the orb, and it has the numeric
-        #    advantage, and it onws at least one adjacent zone, it has tagged
-        #    the zone
-        # 3. The zone is checked again to see if the opposing team has also
-        #    tagged it
-        #    - If so, the zone is rendered neutral (if it's already neutral,
-        #    nothing happens).
-        #    - If not, the zone is considered to be tagged by the team.
-        # 4. If any zone ownership change has been made, the server is informed
-        #    however, no zone allocation is performed yet.
-        # 5. The server should ensure that the zone hasn't already been tagged
-        #    (such as in the situation of two players form the one team tag
-        #    a zone simultaneously), as well as checking zone numbers,
-        #    before telling all clients of the zone change.
-        # 6. The individual clients recalculate zone ownership based on the
-        #    zone change
-
-        zone = tagger.currentZone
-        if tagger.team == zone.orbOwner:
-            return
-
-        # If the tagging player has a phase shift, the zone will not be tagged.
-        if tagger.phaseshift:
-            return
-
-        # If the game is over, zone tagging is not allowed.
-        if self.gameState == GameState.Ended:
-            return
-
-        # Ensure that the zone has not already been checked in this tick:
-        if zone is None or zone.taggedThisTime:
-            return
-
-
-        # Radius from orb (in pixels) that counts as a tag.
-        tagDistance = 35
-        xCoord1, yCoord1 = tagger.pos
-        xCoord2, yCoord2 = zone.defn.pos
-
-        dist = ((xCoord1 - xCoord2) ** 2 + (yCoord1 - yCoord2) ** 2) ** 0.5
-        if dist < tagDistance:
-            attackingTeams = zone.getAttackingTeams()
-            if tagger.team in attackingTeams:
-                if (len(attackingTeams) > 1 and
-                        zone.checkForOpposingTag(tagger.team)):
-                    # The other team has also tagged it
-                    if zone.orbOwner != None:
-                        self.eventPlug.send(TaggingZoneMsg(zone.id, '\x00',
-                                '\x00'))
-                        checkpoint('Universe: two teams tag same zone')
-                else:
-                    # This team is the only to have tagged it
-                    self.eventPlug.send(TaggingZoneMsg(zone.id, tagger.id,
-                            tagger.team.id))
-                    checkpoint('Universe: zone tagged')
-
-                self._killTurret(tagger, zone)
-                zone.taggedThisTime = True
-                self.zonesToReset.append(zone)
-            else:
-                checkpoint('Universe: failed attempt to tag zone')
-
     def _killTurret(self, tagger, zone):
         if zone.turretedPlayer is not None:
-            zone.turretedPlayer.deathDetected(tagger.id, '\x00', 'T')
+            zone.turretedPlayer.deathDetected(tagger.id, 0, 'T')
             checkpoint('Universe: turret killed by tagging zone')
 
     @handler(PlayerKilledMsg, orderPlug)
@@ -1088,24 +795,6 @@ class Universe(Component):
                     self.eventPlug.send(PlayerHasElephantMsg(killer.id))
                 else:
                     self.returnElephantToOwner()
-
-            if self.gameRules == GameRules.Hunting:
-                # Hunt mode
-                if target.team != self.huntTeamWithZones:
-                    i = 0
-                    # Check number of remaining players on that team
-                    for player in self.players:
-                        if (player.team != self.huntTeamWithZones and player is
-                                not target):
-                            i += 1
-                    if i > 1:
-                        self.eventPlug.send(SetPlayerTeamMsg(target.id,
-                                self.huntTeamWithZones.id))
-                    else:
-                        # The second last player on that team just died; change
-                        # all zones now
-                        self.setHuntTeamWithZones(
-                                self.huntTeamWithZones.opposingTeam)
             if shot is not None:
                 self.removeShot(shot)
         except Abort:
@@ -1129,7 +818,7 @@ class Universe(Component):
             player = self.playerWithId[msg.playerId]
         zone = self.map.zoneWithId[msg.zoneId]
         zone.tag(player)
-        self.lastZoneTagged = self._gameTime
+        self.lastZoneTagged = self._elapsedTime
 
     @handler(ZoneStateMsg, orderPlug)
     def zoneOwned(self, msg):
@@ -1177,6 +866,15 @@ class Universe(Component):
         except Abort:
             pass
 
+    @handler(UpgradeChangedMsg, orderPlug)
+    def changeUpgrade(self, msg):
+        for upgradeClass in allUpgrades:
+            if upgradeClass.upgradeType == msg.upgradeType:
+                if msg.statType == 'S':
+                    upgradeClass.requiredStars = msg.newValue
+                elif msg.statType == 'T':
+                    upgradeClass.timeRemaining = msg.newValue
+
     def addGrenade(self, grenade):
         self.grenades.add(grenade)
 
@@ -1189,50 +887,8 @@ class Universe(Component):
     def removeCollectableStar(self, star):
         del self.collectableStars[star.id]
 
-    @handler(SetCaptainMsg, orderPlug)
-    def setCaptain(self, msg):
-        try:
-            player = self.getPlayer(msg.playerId)
-            if player.team is None:
-                return
-            player.team.captain = player
-        except Abort:
-            pass
-
-    @handler(TeamIsReadyMsg, orderPlug)
-    def teamReady(self, msg):
-        try:
-            player = self.getPlayer(msg.playerId)
-            if player.team is not None and not player.team.ready:
-                player.team.ready = True
-                if self._gameShouldStart():
-                    WeakCallLater(0.1, self, '_startSoon')
-        except Abort:
-            pass
-
-    def _gameShouldStart(self):
-        '''Checks to see if both teams are ready'''
-        if self.gameState != GameState.PreGame:
-            return False
-
-        for team in self.teams:
-            if not team.ready:
-                return False
-        return True
-
-    def _startSoon(self, delay=7):
-        self.eventPlug.send(StartingSoonMsg(delay))
-        WeakCallLater(delay, self.eventPlug, 'send',
-                GameStartMsg(self.gameDuration))
-
-    @handler(StartingSoonMsg, orderPlug)
-    def startingSoon(self, msg):
-        self.gameState = GameState.Starting
-
-        # Kick all AI players now, the game is about to begin.
-        for p in list(self.players):
-            if p.bot:
-                self.eventPlug.send(RemovePlayerMsg(p.id))
+    def setGameState(self, state):
+        self.state = state
 
     @handler(KillShotMsg, orderPlug)
     @handler(ShotAbsorbedMsg, orderPlug)
@@ -1294,17 +950,6 @@ class Universe(Component):
             'teamName1': self.teams[1].teamName,
         }
 
-        if self.gameState == GameState.PreGame:
-            result['gameState'] = "PreGame"
-        elif self.gameState == GameState.Starting:
-            result['gameState'] = 'Starting'
-        elif self.gameState == GameState.InProgress:
-            result['gameState'] = "InProgress"
-            result['timeRunning'] = self._gameTime
-            result['timeLimit'] = self.gameDuration
-        else:
-            result['gameState'] = "Ended"
-
         return result
 
     def applyWorldParameters(self, params):
@@ -1312,19 +957,30 @@ class Universe(Component):
             self.teams[0].teamName = params['teamName0']
         if 'teamName1' in params:
             self.teams[1].teamName = params['teamName1']
-        gs = params.get('gameState', None)
-        if gs == 'PreGame':
-            self.gameState = GameState.PreGame
-        elif gs == 'Starting':
-            self.gameState = GameState.Starting
-        elif gs == 'InProgress':
-            self.gameState = GameState.InProgress
-            if 'timeRunning' in params:
-                self._gameTime = params['timeRunning']
-            if 'timeLimit' in params:
-                self.gameDuration = params['timeLimit']
-        elif gs == 'Ended':
-            self.gameState = GameState.Ended
+
+    def sendGrenadeRebound(self, owningPlayer, pos, xVel, yVel):
+        msg = GrenadeReboundedMsg(owningPlayer.id, pos[0], pos[1], xVel, yVel)
+        self.eventPlug.send(msg)
+
+    @handler(GrenadeReboundedMsg, orderPlug)
+    def gotGrenadeRebounded(self, msg):
+        player = self.getPlayer(msg.playerId)
+        if player.upgrade is not None:
+            if player.upgrade.upgradeType == 'g':
+                player.upgrade.gr.doRebound(msg)
+
+    def sendStarRebound(self, star, xVel, yVel):
+        msg = StarReboundedMsg(star.id, star.pos[0], star.pos[1], xVel, yVel)
+        self.eventPlug.send(msg)
+
+    @handler(StarReboundedMsg, orderPlug)
+    def gotStarRebounded(self, msg):
+        try:
+            star = self.collectableStars[msg.starId]
+        except KeyError:
+            return
+        else:
+            star.gotRebound(msg)
 
     def setLayout(self, layout):
         self.zoneWithDef = {}
@@ -1340,38 +996,6 @@ class Universe(Component):
         for upgradetype in allUpgrades:
             upgradetype.requiredStars = 1
 
-    @handler(PlayerIsReadyMsg, orderPlug)
-    def gotPlayerIsReady(self, msg):
-        try:
-            player = self.getPlayer(msg.playerId)
-            player.readyToStart = msg.ready
-        except Abort:
-            pass
-
-    @handler(SetPreferredTeamMsg, orderPlug)
-    def gotPreferredTeam(self, msg):
-        try:
-            player = self.getPlayer(msg.playerId)
-            player.preferredTeam = msg.name.decode()
-        except Abort:
-            pass
-
-    @handler(SetPreferredSizeMsg, orderPlug)
-    def gotPreferredSize(self, msg):
-        try:
-            player = self.getPlayer(msg.playerId)
-            player.preferredSize = (msg.halfMapWidth, msg.mapHeight)
-        except Abort:
-            pass
-
-    @handler(SetPreferredDurationMsg, orderPlug)
-    def gotPreferredDuration(self, msg):
-        try:
-            player = self.getPlayer(msg.playerId)
-            player.preferredDuration = msg.duration
-        except Abort:
-            pass
-
     @handler(ChangeNicknameMsg, orderPlug)
     def changeNickname(self, msg):
         try:
@@ -1379,3 +1003,9 @@ class Universe(Component):
             player.nick = msg.nickname.decode()
         except Abort:
             pass
+
+    def isTournamentTeam(self, teamName):
+        if not self.authManager:
+            return False
+        return teamName in self.authManager.tournamentSettings.teamNames
+
