@@ -17,11 +17,10 @@
 
 from trosnoth.utils.utils import new
 
+NeutralTeamId = '\x00'
+NoPlayer = '\x00'
+
 class GameState(object):
     '''Enumeration for the state of the game.'''
-    PreGame, Starting, InProgress, Ended = new(4)
-
-class GameRules(object):
-    '''Enumeration for which set of game rules to use.'''
-    Normal, Hunting = new(2)
+    PreGame, Lobby, Starting, InProgress, Ended = new(5)
 
