@@ -167,19 +167,21 @@ class MinimapDisruption(Upgrade):
     iconPath = 'upgrade-minimap.png'
 
     def use(self):
-        self.player.team.usingMinimapDisruption = True
-        checkpoint('Minimap disruption used (client side)')
+        if self.player.team is not None:
+            self.player.team.usingMinimapDisruption = True
+            checkpoint('Minimap disruption used (client side)')
 
     def delete(self):
-        self.player.team.usingMinimapDisruption = False
-        super(MinimapDisruption, self).delete()
+        if self.player.team is not None:
+            self.player.team.usingMinimapDisruption = False
+            super(MinimapDisruption, self).delete()
 
-GRENADE_BLAST_RADIUS = 512
+GRENADE_BLAST_RADIUS = 448
 
 @registerUpgrade
 class Grenade(Upgrade):
     upgradeType = 'g'
-    requiredStars = 5
+    requiredStars = 7
     timeRemaining = 2.5
     goneWhenDie = False
     numShots = 40
@@ -201,6 +203,7 @@ class Grenade(Upgrade):
             self.gr.delete()
 
         self.gr = GrenadeShot(self.player.universe, self.player)
+        self.player.universe.addGrenade(self.gr)
 
         super(Grenade, self).use()
         checkpoint('Grenade used (client side)')
@@ -208,7 +211,7 @@ class Grenade(Upgrade):
     def delete(self):
         '''Performs any necessary tasks to remove this upgrade from the game'''
         if self.gr is not None:
-            self.gr.delete()
+            self.universe.removeGrenade(self.gr)
             self.gr = None
         super(Grenade, self).delete()
         checkpoint('Grenade deleted (client side)')
@@ -225,7 +228,7 @@ class Grenade(Upgrade):
             dist = ((player.pos[0] - xpos) ** 2 +
                     (player.pos[1] - ypos) ** 2) ** 0.5
             if dist <= GRENADE_BLAST_RADIUS:
-                player.hurtBy(killerId, 0, 'G')
+                player.hurtBy(killerId, None, 'G')
 
 @registerUpgrade
 class Ricochet(Upgrade):
@@ -252,8 +255,8 @@ class Ricochet(Upgrade):
 class Ninja (Upgrade):
     '''allows you to become invisible to all players on the opposing team'''
     upgradeType = 'n'
-    requiredStars = 6
-    timeRemaining = 15
+    requiredStars = 5
+    timeRemaining = 25
     name = 'Ninja'
     action = 'phase shift' # So old phase shift hotkeys trigger ninja.
     order = 40
@@ -279,7 +282,7 @@ class Shoxwave(Upgrade):
     '''
     upgradeType = 'w'
     requiredStars = 7
-    timeRemaining = 60
+    timeRemaining = 45
     name = 'Shoxwave'
     action = 'shoxwave'
     order = 80
@@ -298,8 +301,8 @@ class Shoxwave(Upgrade):
 @registerUpgrade
 class MachineGun(Upgrade):
     upgradeType = 'x'
-    requiredStars = 15
-    timeRemaining = 20
+    requiredStars = 10
+    timeRemaining = 30
     name = 'Machine Gun'
     action = 'turret'   # So that old turret hotkeys trigger machine gun.
     order = 10
