@@ -20,19 +20,21 @@ import random
 from trosnoth.utils.components import Component, handler, Plug
 from trosnoth.messages import (ChatMsg, BuyUpgradeMsg, PlayerStarsSpentMsg,
         PlayerHasUpgradeMsg, CannotBuyUpgradeMsg, DeleteUpgradeMsg,
-        SetCaptainMsg, TeamIsReadyMsg, RespawnRequestMsg, CannotRespawnMsg,
+        RespawnRequestMsg, CannotRespawnMsg, PreferredTeamSelectedMsg,
         RespawnMsg, FireShotMsg, ShootMsg, JoinRequestMsg, CannotJoinMsg,
         JoinApproved, UpdatePlayerStateMsg, AimPlayerAtMsg, RemovePlayerMsg,
         PlayerIsReadyMsg, SetPreferredTeamMsg, SetPreferredDurationMsg,
-        SetPreferredSizeMsg, ChangeNicknameMsg, FireShoxwaveMsg, MarkZoneMsg)
+        SetPreferredSizeMsg, ChangeNicknameMsg, FireShoxwaveMsg, MarkZoneMsg,
+        ChangePlayerLimitsMsg)
 from trosnoth.model.universe import Abort
+from trosnoth.model.universe_base import NeutralTeamId
 from trosnoth.model.upgrades import Turret, MinimapDisruption, allUpgrades
-from trosnoth.model.universe_base import GameState
 
 EDGE_TURRET_BOUNDARY = 100
 ORB_TURRET_BOUNDARY = 150
 
 MAX_TEAM_NAME_LENGTH = 30
+MAX_PLAYER_NAME_LENGTH = 30
 MAX_MAP_HEIGHT = 10
 MAX_HALF_WIDTH = 10
 MAX_GAME_DURATION = 86400
@@ -61,12 +63,26 @@ class Validator(Component):
     def passOn(self, msg):
         self.agentEvents.send(msg)
 
+    @handler(ChangePlayerLimitsMsg, gameEvents)
+    def setPlayerLimits(self, msg):
+        self.maxPlayers = msg.maxPerTeam
+        if msg.maxTotal == 0:
+            self.maxTotalPlayers = 2 * msg.maxPerTeam
+        else:
+            self.maxTotalPlayers = min(2 * msg.maxPerTeam, self.maxTotal)
+
     @handler(MarkZoneMsg, agentRequests)
     def markZone(self, msg):
         self.gameRequests.send(msg)
 
     @handler(ShootMsg, agentRequests)
     def reqFireShot(self, msg):
+        if not self.world.physics.shooting:
+            return
+        # This call will ultimately supercede the previous
+        if not self.world.canShoot():
+            return
+
         try:
             player = self.world.getPlayer(msg.playerId)
             shotType = player.getShotType()
@@ -91,8 +107,7 @@ class Validator(Component):
                 self.agentEvents.send(CannotBuyUpgradeMsg(msg.playerId, 'A',
                         msg.tag))
                 return
-            elif self.world.gameState in (GameState.PreGame,
-                    GameState.Starting):
+            elif not self.world.canBuyUpgrades():
                 self.agentEvents.send(CannotBuyUpgradeMsg(msg.playerId, 'P',
                         msg.tag))
                 return
@@ -186,27 +201,25 @@ class Validator(Component):
             assert totalStars == 0
 
         self.gameRequests.send(PlayerHasUpgradeMsg(player.id,
-                upgrade.upgradeType))
+                upgrade.upgradeType, "B"))
 
     def isValidNick(self, nick):
-        if len(nick) < 2:
+        if len(nick) < 2 or len(nick) > MAX_PLAYER_NAME_LENGTH:
             return False
         return True
 
     @handler(JoinRequestMsg, agentRequests)
     def reqJoinGame(self, msg):
-        teamId = msg.teamId
+        preferredTeamId = msg.teamId
         nick = msg.nick.decode()
         teamPlayerCounts = self._getTeamPlayerCounts()
-        if teamId == '\x00':
-            teamId = self._autoSelectTeam(teamPlayerCounts)
 
-        if self.world.gameState == GameState.Ended:
-            teamId = '\x00'
-        elif teamPlayerCounts.get(teamId, 0) >= self.maxPlayers:
+        if len(self.world.players) >= self.maxTotalPlayers:
             self.agentEvents.send(CannotJoinMsg('F', msg.teamId, msg.tag))
             return
-        if len(self.world.players) >= self.maxTotalPlayers:
+
+        teamId = self.world.getTeamToJoin(preferredTeamId)
+        if teamId != NeutralTeamId and teamPlayerCounts.get(teamId, 0) >= self.maxPlayers:
             self.agentEvents.send(CannotJoinMsg('F', msg.teamId, msg.tag))
             return
 
@@ -214,11 +227,6 @@ class Validator(Component):
             self.agentEvents.send(CannotJoinMsg('B', msg.teamId, msg.tag))
             return
 
-        for player in self.world.players:
-            if player.nick.lower() == nick.lower():
-                self.agentEvents.send(CannotJoinMsg('N', msg.teamId, msg.tag))
-                return
-
         if self.authTagManager is not None and not msg.localBotRequest:
             user = self.authTagManager.checkAuthTag(msg.authTag)
             if user is None:
@@ -227,16 +235,24 @@ class Validator(Component):
                             msg.tag))
                     return
             else:
-                if (nick.lower() != user.username and
-                        self.authTagManager.checkUsername(nick)):
-                    self.agentEvents.send(CannotJoinMsg('N', msg.teamId,
-                            msg.tag))
-                    return
                 for player in self.world.players:
                     if player.user == user:
                         self.agentEvents.send(CannotJoinMsg('U', msg.teamId,
                                 msg.tag))
                         return
+                if (nick.lower() != user.username and
+                        self.authTagManager.checkUsername(nick)):
+                    self.agentEvents.send(CannotJoinMsg('N', msg.teamId,
+                            msg.tag))
+                    return
+
+        # Only check for duplicate nick after checking for auth-related errors.
+        for player in self.world.players:
+            if player.nick.lower() == nick.lower():
+                self.agentEvents.send(CannotJoinMsg('N', msg.teamId, msg.tag))
+                return
+
+        if self.authTagManager is not None and not msg.localBotRequest:
             self.authTagManager.discardAuthTag(msg.authTag)
             username = user.username
             user.setNick(nick)
@@ -279,37 +295,13 @@ Details:
         return self.world.selectZone(teamId).id
 
     def _getTeamPlayerCounts(self):
-        '''
-        Returns a mapping from team id to number of players currently on that
-        team.
-        '''
-        playerCounts = {}
-        for player in self.world.players:
-            playerCounts[player.teamId] = playerCounts.get(player.teamId, 0
-                    ) + 1
-        return playerCounts
-
-    def _autoSelectTeam(self, playerCounts):
-        '''
-        Returns the team id of one of the teams with the smallest number of
-        players.
-        '''
-        minCount = len(self.world.players) + 1
-        minTeams = []
-        for team in self.world.teams:
-            count = playerCounts.get(team.id, 0)
-            if count < minCount:
-                minCount = count
-                minTeams = [team.id]
-            elif count == minCount:
-                minTeams.append(team.id)
-        return random.choice(minTeams)
+        return self.world.getTeamPlayerCounts()
 
     @handler(RespawnRequestMsg, agentRequests)
     def reqRespawn(self, msg):
         try:
             player = self.world.getPlayer(msg.playerId)
-            if self.world.gameState in (GameState.PreGame, GameState.Starting):
+            if not self.world.canRespawn():
                 self.agentEvents.send(CannotRespawnMsg(msg.playerId, 'P',
                         msg.tag))
             elif not player.ghost:
@@ -368,24 +360,6 @@ Details:
         except Abort:
             pass
 
-    @handler(SetCaptainMsg, agentRequests)
-    def reqSetCaptain(self, msg):
-        try:
-            player = self.world.getPlayer(msg.playerId)
-            if player.teamHasNoCaptain():
-                self.gameRequests.send(msg)
-        except Abort:
-            pass
-
-    @handler(TeamIsReadyMsg, agentRequests)
-    def reqTeamReady(self, msg):
-        try:
-            player = self.world.getPlayer(msg.playerId)
-            if player.isTeamCaptain():
-                self.gameRequests.send(msg)
-        except Abort:
-            pass
-
     @handler(ChatMsg, agentRequests)
     def chatSent(self, msg):
         try:
@@ -405,8 +379,8 @@ Details:
         try:
             self.world.getPlayer(msg.playerId)
             teamName = msg.name.decode()[:MAX_TEAM_NAME_LENGTH]
-            self.gameRequests.send(SetPreferredTeamMsg(msg.playerId,
-                    teamName.encode()))
+            self.gameRequests.send(PreferredTeamSelectedMsg(msg.playerId,
+                    self.world.isTournamentTeam(teamName), teamName.encode()))
         except Abort:
             pass
 
@@ -433,7 +407,7 @@ Details:
 
     @handler(ChangeNicknameMsg, agentRequests)
     def reqChangeNickname(self, msg):
-        if self.world.gameState == GameState.InProgress:
+        if self.world.gameIsInProgress():
             return
         try:
             p = self.world.getPlayer(msg.playerId)
