@@ -17,6 +17,7 @@
 
 import random
 import logging
+from trosnoth.utils import math
 
 log = logging.getLogger('zone')
 
@@ -69,7 +70,6 @@ class ZoneState(object):
         orbOwner - a team
         players - a Group
         nonPlayers - a Group for ghosts
-        taggedThisTime - boolean flag used to compare whether multiple players
             have tagged the zone in one tick
         turretedPlayer - None or a Player
 
@@ -82,7 +82,7 @@ class ZoneState(object):
         self.id = zoneDef.id
 
         universe.zoneWithDef[zoneDef] = self
-
+        
         self.universe = universe
         teamIndex = zoneDef.initialOwnerIndex
         if teamIndex is None:
@@ -98,9 +98,8 @@ class ZoneState(object):
         self.players = set()
 
         # zone.nonPlayers are those players that do not count in zone
-        # calculations, such as ghosts and decoys
+        # calculations, such as ghosts
         self.nonPlayers = set()
-        self.taggedThisTime = False
         self.turretedPlayer = None
         self.frozen = False
 
@@ -112,6 +111,9 @@ class ZoneState(object):
             return 'Z---'
         return 'Z%3d' % self.id
 
+    def isNeutral(self):
+        return self.orbOwner == None
+
     def tag(self, player):
         '''This method should be called when the orb in this zone is tagged'''
         self.previousOwner = self.orbOwner
@@ -139,7 +141,7 @@ class ZoneState(object):
                     # Allow the adjacent zone to check if it is entirely
                     # surrounded by non-enemy zones
                     zone.checkAgain()
-                elif zone.orbOwner == None:
+                elif zone.isNeutral():
                     pass
                 else:
                     allGood = False
@@ -184,20 +186,40 @@ class ZoneState(object):
                 # Change zone ownership to orb ownership.
                 self.zoneOwner = self.orbOwner
 
-    def checkForOpposingTag(self, team):
-        '''In the event that a zone is tagged by a team, this procedure is
-        called to see if an opposing team also tags the orb.'''
+    def playerIsWithinTaggingDistance(self, player):
+        assert player.currentZone == self
+        tagDistance = 35
+        distance = math.distance(self.defn.pos, player.pos)
+        return distance < tagDistance
 
-        xCoord1, yCoord1 = self.defn.pos
-        tagDistance = 30
-        for player in self.players:
-            if player.team != team:
-                xCoord2, yCoord2 = player.pos
-                distance = ((xCoord1 - xCoord2) ** 2 + (yCoord1 - yCoord2) **
-                        2) ** 0.5
-                if distance < tagDistance:
-                    return True
-        return False
+    def playerWhoTaggedThisZone(self):
+        '''
+        Checks to see whether the zone has been tagged. If not, return None.
+        If it has, return a tuple of the player and team who tagged it.
+        If the zone has been rendered Neutral, this will be None, None
+        '''
+        teamsToPlayers = self.getCountedPlayersByTeam()
+        teamsWhoCanTag = self.teamsAbleToTag()
+        taggingPlayers = []
+        
+        if len(teamsWhoCanTag) == 1 and list(teamsWhoCanTag)[0] == self.orbOwner:
+            # No need to check again - we are already the owner
+            return
+        
+        for team in teamsWhoCanTag:
+            #print len(teamsWhoCanTag)
+            for player in teamsToPlayers[team]:
+                if self.playerIsWithinTaggingDistance(player):
+                    taggingPlayers.append(player)
+                    # Only allow one player from each team to have tagged it.
+                    continue
+        if len(taggingPlayers) == 2:
+            # Both teams tagged - becomes neutral
+            return (None, None)
+        elif len(taggingPlayers) == 1 and list(taggingPlayers)[0].team != self.orbOwner:
+            return (taggingPlayers[0], taggingPlayers[0].team)
+        else:
+            return None
 
     def removePlayer(self, player):
         '''Removes a player from this zone'''
@@ -221,14 +243,8 @@ class ZoneState(object):
         except KeyError, e:
             log.exception(str(e))
 
-    def getPlayerCounts(self):
-        '''
-        Returns a mapping from team to the number of players that team has in
-        the zone. Does not count rogue players.
-        This takes into account the fact that turrets do not count towards
-        numerical advantage.
-        '''
-        result = dict((team, 0) for team in self.universe.teams)
+    def getCountedPlayersByTeam(self):
+        result = dict((team, []) for team in self.universe.teams)
 
         for player in self.players:
             if player.turret:
@@ -236,7 +252,19 @@ class ZoneState(object):
                 # of reckoning whether an enemy can capture the orb
                 continue
             if player.team is not None:
-                result[player.team] += 1
+                result[player.team].append(player)
+        return result
+
+    def getPlayerCounts(self):
+        '''
+        Returns a mapping from team to the number of players that team has in
+        the zone. Does not count rogue players.
+        This takes into account the fact that turrets do not count towards
+        numerical advantage.
+        '''
+        result = {}
+        for key, value in self.getCountedPlayersByTeam().iteritems():
+            result[key] = len(value)
         return result
 
     def isCapturableBy(self, team):
@@ -245,7 +273,7 @@ class ZoneState(object):
         the given team. This takes into account both the zone location and the
         number of players in the zone.
         '''
-        return team != self.orbOwner and team in self.getAttackingTeams()
+        return team != self.orbOwner and team in self.teamsAbleToTag()
 
     def isDisputed(self):
         '''
@@ -260,10 +288,10 @@ class ZoneState(object):
                 continue
             if maxPlayers > 3:
                 if (count > 3 and team != self.orbOwner and
-                        self.adjacentToTeam(team)):
+                        self._adjacentToAnotherZoneOwnedBy(team)):
                     state = 'capturable'
             elif count > maxPlayers:
-                if team != self.orbOwner and self.adjacentToTeam(team):
+                if team != self.orbOwner and self._adjacentToAnotherZoneOwnedBy(team):
                     state = 'capturable'
                 else:
                     state = 'safe'
@@ -272,22 +300,20 @@ class ZoneState(object):
                 state  = 'disputed'
         if state == 'empty':
             for team in self.universe.teams:
-                if team != self.orbOwner and self.adjacentToTeam(team):
+                if team != self.orbOwner and self._adjacentToAnotherZoneOwnedBy(team):
                     state = 'disputed'
                     break
         return state == 'disputed'
 
-    def getAttackingTeams(self):
+    def teamsAbleToTag(self):
         '''
-        Checks whether there are any teams which are able to capture this zone.
-        If so, returns a set of all such teams, otherwise returns an empty set.
-        Does not take into account the fact that a team owns the zone. That is,
-        the orb owner may be in this set.
+        Returns the set of teams who have enough players to tag a zone;
+        ownning at least one adjacent zone (ignores current zone ownership)
         '''
-        return set([t for t in self.getWinningTeams() if
-                self.adjacentToTeam(t)])
+        return set([t for t in self.getTeamsWithEnoughNumbers() if
+                self._adjacentToAnotherZoneOwnedBy(t)])
 
-    def adjacentToTeam(self, team):
+    def _adjacentToAnotherZoneOwnedBy(self, team):
         '''
         Returns whether or not this zone is adjacent to a zone whose orb is
         owned by the given team.
@@ -298,7 +324,7 @@ class ZoneState(object):
                 return True
         return False
 
-    def getWinningTeams(self):
+    def getTeamsWithEnoughNumbers(self):
         '''
         Returns a set of all teams which have:
          (a) strict numerical advantage in this zone; or
