@@ -115,6 +115,28 @@ class ConnectToGame(amp.Command):
         GameDoesNotExist: 'NO_GAME',
     }
 
+class GetRegisteredIdentities(amp.Command):
+    '''
+    Requests a list of email addresses which are associated with this user on
+    the server.
+    '''
+    arguments = []
+    response = [('identities', amp.ListOf(amp.Unicode()))]
+    errors = {
+        NotAuthenticated: 'NO_AUTH',
+    }
+
+class SetRegisteredIdentities(amp.Command):
+    '''
+    Sets the list of email addresses which are associated with this user on the
+    server.
+    '''
+    arguments = [('identities', amp.ListOf(amp.Unicode()))]
+    response = []
+    errors = {
+        NotAuthenticated: 'NO_AUTH',
+    }
+
 class GetAuthToken(amp.Command):
     '''
     Requests the server send a random string to be used during authentication or
@@ -137,6 +159,26 @@ class PasswordAuthenticate(amp.Command):
     ]
     response = [('result', amp.Boolean())]
 
+class SetPassword(amp.Command):
+    arguments = [
+        ('password', amp.String()),
+    ]
+    response = []
+    errors = {
+        NotAuthenticated: 'NO_AUTH',
+    }
+
+class GetSupportedSettings(amp.Command):
+    '''
+    Returns a list of strings which indicate which account settings the
+    client should display to the user.
+    Possible values are 'password', 'identities'
+    '''
+    arguments = []
+    response = [
+        ('result', amp.ListOf(amp.String())),
+    ]
+
 class CreateUserWithPassword(amp.Command):
     '''
     Requests to create a user on the server using the given username and
