@@ -73,41 +73,35 @@ class AuthGame(Game):
         self.server = server
         self.id = id
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def join(self, passwordGetter):
         '''
         Attempts to connect to this game. Returns a deferred whose
         callback will be executed with (netHandler, authTag).
         '''
+        log.info('Attempting to join game on %r', self.server)
         host, port = self.server
-        p = defer.waitForDeferred(ClientCreator(reactor,
-                amp.AMP).connectTCP(host, port))
-        yield p
-        p = p.getResult()
+        p = yield ClientCreator(reactor, amp.AMP).connectTCP(host, port)
+        log.info('Connection established')
 
         try:
-            result = defer.waitForDeferred(p.callRemote(
-                    authcommands.ConnectToGame, id=self.id))
-            yield result
-            result = result.getResult()
+            result = yield p.callRemote(authcommands.ConnectToGame, id=self.id)
         except authcommands.NotAuthenticated:
-            a = defer.waitForDeferred(self.lobby._authenticate(p, host,
-                    passwordGetter))
-            yield a
-            a.getResult()
+            log.info('Not authenticated')
+            yield authenticate(p, host, passwordGetter)
 
-            result = defer.waitForDeferred(p.callRemote(
-                    authcommands.ConnectToGame, id=self.id))
-            yield result
-            result = result.getResult()
+            log.info('Retrying')
+            result = yield p.callRemote(authcommands.ConnectToGame, id=self.id)
+        finally:
+            if p.transport.connected:
+                p.transport.loseConnection()
 
-        netHandler = defer.waitForDeferred(self.lobby._connectToServer(host,
-                result['port']))
-        yield netHandler
-        netHandler = netHandler.getResult()
+        log.info('Done')
+
+        netHandler = yield self.lobby._connectToServer(host, result['port'])
+        log.info('Got netHandler')
         self.lobby.app.identitySettings.setNick(result['nick'])
-        yield (netHandler, result['authTag'])
-
+        defer.returnValue((netHandler, result['authTag']))
 
 class NonAuthGame(Game):
     def __init__(self, lobby, ipAddr, port):
@@ -115,23 +109,40 @@ class NonAuthGame(Game):
         self.ip = ipAddr
         self.port = port
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def join(self, passwordGetter):
         '''
         Attempts to connect to the given lobby.Game. Returns a deferred whose
         callback will be executed with (netHandler, authTag).
         '''
-        netHandler = defer.waitForDeferred(self.lobby._connectToServer(self.ip,
-                self.port))
-        yield netHandler
-        netHandler = netHandler.getResult()
-        yield (netHandler, 0)
+        netHandler = yield self.lobby._connectToServer(self.ip, self.port)
+        defer.returnValue((netHandler, 0))
 
 class IncorrectServerVersion(Exception):
     '''
     The server created a game of the wrong version.
     '''
 
+class AuthServerConnection(object):
+    def __init__(self, server, timeout=5):
+        self.host, self.port = server
+        self.timeout = timeout
+        self.protocol = None
+
+    def __enter__(self):
+        d = ClientCreator(reactor, amp.AMP).connectTCP(self.host, self.port,
+                timeout=self.timeout)
+        d.addCallback(self.connectionEstablished)
+        return d
+
+    def connectionEstablished(self, protocol):
+        self.protocol = protocol
+        return protocol
+
+    def __exit__(self, type, value, traceback):
+        if self.protocol and self.protocol.transport.connected:
+            self.protocol.transport.loseConnection()
+
 class Lobby(object):
     '''
     Performs tasks related to finding games and joining them.
@@ -140,7 +151,7 @@ class Lobby(object):
         self.app = app
         self.netman = netman
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def getGames(self, server, timeout=2):
         '''
         Connects to an authentication server and asks it for the available
@@ -148,23 +159,19 @@ class Lobby(object):
         with a list of lobby.Game objects.
         @param server: A tuple of (host, port).
         '''
-        host, port = server
-        p = defer.waitForDeferred(ClientCreator(reactor,
-                amp.AMP).connectTCP(host, port, timeout=timeout))
-        yield p
-        p = p.getResult()
-        result = defer.waitForDeferred(p.callRemote(authcommands.ListGames))
-        yield result
-        result = result.getResult()
-        games = []
-        for item in result['games']:
-            if (item['game'] == 'Trosnoth1' and item['version'] in
-                    validServerVersions):
-                games.append(AuthGame(self, server, item['id']))
+        with AuthServerConnection(server, timeout) as deferred:
+            p = yield deferred
 
-        yield games
+            result = yield p.callRemote(authcommands.ListGames)
+            games = []
+            for item in result['games']:
+                if (item['game'] == 'Trosnoth1' and item['version'] in
+                        validServerVersions):
+                    games.append(AuthGame(self, server, item['id']))
 
-    @defer.deferredGenerator
+            defer.returnValue(games)
+
+    @defer.inlineCallbacks
     def getOtherGames(self, server, timeout=2):
         '''
         Connects to an authentication server and asks it for the available
@@ -173,40 +180,33 @@ class Lobby(object):
         objects.
         @param server: A tuple of (host, port).
         '''
-        host, port = server
-        p = defer.waitForDeferred(ClientCreator(reactor,
-                amp.AMP).connectTCP(host, port, timeout=timeout))
-        yield p
-        p = p.getResult()
-        result = defer.waitForDeferred(p.callRemote(
-                authcommands.ListOtherGames))
-        yield result
-        result = result.getResult()
-        games = []
-        for item in result['games']:
-            if (item['game'] == 'Trosnoth1' and item['version'] in
-                    validServerVersions):
-                games.append(NonAuthGame(self, item['ip'], item['port']))
+        with AuthServerConnection(server, timeout) as deferred:
+            p = yield deferred
 
-        yield games
+            result = yield p.callRemote(authcommands.ListOtherGames)
+            games = []
+            for item in result['games']:
+                if (item['game'] == 'Trosnoth1' and item['version'] in
+                        validServerVersions):
+                    games.append(NonAuthGame(self, item['ip'], item['port']))
 
-    @defer.deferredGenerator
+            defer.returnValue(games)
+
+    @defer.inlineCallbacks
     def getMulticastGames(self):
         '''
         Returns a deferred which will be called back with a sequence of Game
         objects representing the games which could be found on the LAN by
         multicasting a game request.
         '''
-        games = defer.waitForDeferred(getMulticastGames())
-        yield games
-        games = games.getResult()
+        games = yield getMulticastGames()
 
         result = []
         for (ip, port), gameInfo in games:
             result.append(NonAuthGame(self, ip, port))
-        yield result
+        defer.returnValue(result)
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def registerGame(self, server, netHandler, timeout=2):
         '''
         Registers the game running on the given ServerNetHandler with the given
@@ -215,24 +215,19 @@ class Lobby(object):
         successful.
         @param server: A tuple of (host, port).
         '''
-        host, port = server
+        with AuthServerConnection(server, timeout) as deferred:
+            p = yield deferred
 
-        p = defer.waitForDeferred(ClientCreator(reactor,
-                amp.AMP).connectTCP(host, port, timeout=timeout))
-        yield p
-        p = p.getResult()
-        result = defer.waitForDeferred(p.callRemote(authcommands.RegisterGame,
-                game='Trosnoth1', version=serverVersion,
-                port=netHandler.netman.getTCPPort()))
-        yield result
-        try:
-            result = result.getResult()
-        except:
-            yield False
+            try:
+                yield p.callRemote(authcommands.RegisterGame,
+                        game='Trosnoth1', version=serverVersion,
+                        port=netHandler.netman.getTCPPort())
+            except:
+                defer.returnValue(False)
+            else:
+                defer.returnValue(True)
 
-        yield True
-
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def startGame(self, server, passwordGetter):
         '''
         Attempts to start a game on the given server, and connects to the
@@ -242,80 +237,77 @@ class Lobby(object):
         @param passwordGetter: PasswordGetter object which is used to
             authenticate with the server if needed.
         '''
-        host, port = server
-        p = defer.waitForDeferred(ClientCreator(reactor,
-                amp.AMP).connectTCP(host, port))
-        yield p
-        p = p.getResult()
+        with AuthServerConnection(server) as deferred:
+            p = yield deferred
 
-        result = defer.waitForDeferred(p.callRemote(authcommands.CreateGame,
-                game='Trosnoth1'))
-        yield result
-        result = result.getResult()
-        if result['version'] not in validServerVersions:
-            raise IncorrectServerVersion()
+            result = yield p.callRemote(authcommands.CreateGame, game='Trosnoth1')
+            if result['version'] not in validServerVersions:
+                raise IncorrectServerVersion()
 
-        game = AuthGame(self, server, result['id'])
+            game = AuthGame(self, server, result['id'])
 
-        result = defer.waitForDeferred(game.join(passwordGetter))
-        yield result
-        yield result.getResult()
+            result = yield game.join(passwordGetter)
+            defer.returnValue(result)
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def _connectToServer(self, host, port):
         netHandler = ClientNetHandler(self.netman)
-        result = defer.waitForDeferred(netHandler.makeConnection(host, port))
-        yield result
-        result.getResult()
+        yield netHandler.makeConnection(host, port)
 
-        yield netHandler
+        defer.returnValue(netHandler)
 
-    @defer.deferredGenerator
-    def _authenticate(self, p, host, passwordGetter):
-        errorMsg = ''
-        while True:
-            if host in AUTHENTICATION_CACHE:
-                create = False
-                username, password = AUTHENTICATION_CACHE[host]
-            else:
-                result = defer.waitForDeferred(passwordGetter.getPassword(host,
-                        errorMsg))
-                yield result
-                result = result.getResult()
-                if result is None:
-                    raise AuthenticationCancelled()
-                create, username, password = result
+@defer.inlineCallbacks
+def authenticate(p, host, passwordGetter):
+    errorMsg = ''
+    for i in xrange(15):
+        if host in AUTHENTICATION_CACHE:
+            log.info('Got cached auth info')
+            create = False
+            username, password = AUTHENTICATION_CACHE[host]
+        else:
+            log.info('Checking with password getter')
+            result = yield passwordGetter.getPassword(host, errorMsg)
+            if result is None:
+                raise AuthenticationCancelled()
+            create, username, password = result
 
-            token = defer.waitForDeferred(p.callRemote(
-                    authcommands.GetAuthToken))
-            yield token
-            token = token.getResult()['token']
+        data = yield encryptPassword(p, password)
 
-            pubkey = defer.waitForDeferred(p.callRemote(
-                    authcommands.GetPublicKey))
-            yield pubkey
-            pubkey = makePublicKey(pubkey.getResult())
+        if create:
+            log.info('Creating user')
+            result = yield p.callRemote(
+                    authcommands.CreateUserWithPassword,
+                    username=str(username), password=data)
+            result = result['result']
+            if result == '':
+                log.info('Auth successful')
+                return  # Authentication successful.
+            errorMsg = result
+        else:
+            log.info('Attempting to auth')
+            result = yield p.callRemote(authcommands.PasswordAuthenticate,
+                    username=str(username), password=data)
+            result = result['result']
+            if result:
+                log.info('Auth successful')
+                return  # Authentication successful.
+            errorMsg = 'Bad username/password'
 
-            data = rsa.encrypt(token + password.encode(), pubkey)
+    # If we can't get the authentication right in 15 tries, something is
+    # wrong.
+    raise AuthenticationCancelled()
 
-            if create:
-                result = defer.waitForDeferred(p.callRemote(
-                        authcommands.CreateUserWithPassword,
-                        username=str(username), password=data))
-                yield result
-                result = result.getResult()['result']
-                if result == '':
-                    return  # Authentication successful.
-                errorMsg = result
-            else:
-                result = defer.waitForDeferred(p.callRemote(
-                        authcommands.PasswordAuthenticate,
-                        username=str(username), password=data))
-                yield result
-                result = result.getResult()['result']
-                if result:
-                    return  # Authentication successful.
-                errorMsg = 'Bad username/password'
+@defer.inlineCallbacks
+def encryptPassword(p, password):
+    log.info('Getting auth token')
+    token = yield p.callRemote(authcommands.GetAuthToken)
+    token = token['token']
+
+    log.info('Getting server public key')
+    pubkey = yield p.callRemote(authcommands.GetPublicKey)
+    pubkey = makePublicKey(pubkey)
+
+    defer.returnValue(rsa.encrypt(token + password.encode(), pubkey))
 
 AUTHENTICATION_CACHE = {}   # host -> (username, password)
 
