@@ -17,6 +17,7 @@
 
 import logging
 import struct
+import time
 
 from twisted.internet import reactor
 from twisted.internet.protocol import Factory, DatagramProtocol
@@ -96,12 +97,15 @@ class TCPReceiver(Int32StringReceiver):
         self.factory._newConnection(self)
 
     def sendMessage(self, msg):
-        self.sendString(msg.pack())
+        data = msg.pack()
+        self.sendString(data)
+        self.factory.tx += len(data)
 
     def connectionLost(self, reason):
         self.factory._lostConnection(self)
 
     def stringReceived(self, line):
+        self.factory.rx += len(line)
         if self.handler is None:
             # First message received.
             self.factory._bindConnection(self, line)
@@ -254,6 +258,11 @@ class NetworkManager(Factory):
         self._attempting = {}       # (ip, port) -> (handler, connId)
         self._lastConnectionId = 0
 
+        # For profiling.
+        self.rx = 0
+        self.tx = 0
+        self.startTime = time.time()
+
     def kill(self):
         'Stops listening for incoming connections.'
         if self._port is not None:
@@ -465,6 +474,7 @@ class NetworkManager(Factory):
         self._connections[connectionId] = connection
 
         # Send the greeting string.
+        self.tx += len(handler.greeting)
         connection.sendString(handler.greeting)
 
         # Also send the remote id.
