@@ -19,14 +19,13 @@ import logging
 
 from trosnoth.network.networkDefines import serverVersion
 from trosnoth.utils import netmsg
-from trosnoth.model.universe_base import GameState
 from trosnoth.utils.components import DynamicPlug, UnhandledMessage
 
 from trosnoth.messages import (ChatMsg, InitClientMsg,
         QueryWorldParametersMsg, RequestMapBlockLayoutMsg, RequestPlayersMsg,
         RequestUDPStatusMsg, NotifyUDPStatusMsg, DeleteUpgradeMsg,
         BuyUpgradeMsg, ShootMsg, RespawnRequestMsg, JoinRequestMsg,
-        UpdatePlayerStateMsg, AimPlayerAtMsg, SetCaptainMsg, TeamIsReadyMsg,
+        UpdatePlayerStateMsg, AimPlayerAtMsg,
         RequestZonesMsg, tcpMessages, PlayerIsReadyMsg, SetPreferredDurationMsg,
         SetPreferredTeamMsg, SetPreferredSizeMsg, RemovePlayerMsg,
         ChangeNicknameMsg, MarkZoneMsg, RequestGameSpeedMsg,
@@ -42,8 +41,6 @@ serverMsgs = netmsg.MessageCollection(
     RequestMapBlockLayoutMsg,
     ShootMsg,
     DeleteUpgradeMsg,
-    SetCaptainMsg,
-    TeamIsReadyMsg,
     UpdatePlayerStateMsg,
     AimPlayerAtMsg,
     BuyUpgradeMsg,
@@ -159,8 +156,8 @@ class ServerNetHandler(object):
         del self.connectedClients[connId]
 
         # Check for game over and no connections left.
-        if (len(self.connectedClients) == 0 and self.game.world.gameState
-                == GameState.Ended):
+        if (len(self.connectedClients) == 0 and not
+                self.game.world.gameIsInProgress()):
             # Don't shut down if local player is connected.
             for p in self.game.world.players:
                 if not p.bot:
@@ -177,5 +174,5 @@ class ServerNetHandler(object):
         # Kill server
         self.running = False
         self.game.stop()
-        self.game.world.stopClock()
+        self.game.gameController.stopClock()
         self.onShutdown.execute()
