@@ -31,11 +31,13 @@ from trosnoth.network import authcommands
 from trosnoth.network.netman import NetworkManager
 from trosnoth.network.server import ServerNetHandler
 from trosnoth.network.networkDefines import serverVersion
-from trosnoth.settings import AlertSettings, AuthServerSettings
-from trosnoth.utils.utils import hasher, timeNow, initLogging
+from trosnoth.settings import (AlertSettings, AuthServerSettings,
+        TournamentSettings)
+from trosnoth.utils.utils import hasher, timeNow, initLogging, Function
+from trosnoth.utils.unrepr import unrepr
 from trosnoth.web.server import WebServer
 
-from trosnoth.gamerecording.achievementlist import (achievementDict,
+from trosnoth.gamerecording.achievementlist import (availableAchievements,
         OnceEverPerPlayer)
 
 from twisted.protocols import amp
@@ -50,7 +52,6 @@ MAX_GAMES = 1
 GAME_NAME = 'Auth Server Game'
 GAME_KIND = 'Trosnoth1'
 
-NEW_USERS_ALLOWED = True
 GAME_VERIFY_TIME = 60
 
 class AuthenticationProtocol(amp.AMP):
@@ -137,12 +138,45 @@ class AuthenticationProtocol(amp.AMP):
         tag = server.authTagManager.newTag(self.user)
         nick = self.user.getNick()
 
+        self.factory.broadcast('%s has joined the game' % (nick,))
         return {
             'port': server.netman.getTCPPort(),
             'authTag': tag,
             'nick': nick,
         }
 
+    @authcommands.GetRegisteredIdentities.responder
+    def getRegisteredIdentities(self):
+        if self.user is None:
+            raise authcommands.NotAuthenticated()
+        return {
+            'identities': self.user.getIdentities(),
+        }
+
+    @authcommands.SetRegisteredIdentities.responder
+    def setRegisteredIdentities(self, identities):
+        if self.user is None:
+            raise authcommands.NotAuthenticated()
+        self.user.setIdentities(identities)
+        return {}
+
+    @authcommands.GetSupportedSettings.responder
+    def getSupportedSettings(self):
+        settings = ['password']
+        if self.factory.settings.showIdentityScreen:
+            settings.append('identities')
+        return {
+            'result': settings,
+        }
+
+    @authcommands.SetPassword.responder
+    def setUserPassword(self, password):
+        if self.user is None:
+            raise authcommands.NotAuthenticated()
+        password = self._decodePassword(password)
+        self.user.setPassword(password)
+        return {}
+
     @authcommands.GetAuthToken.responder
     def getAuthToken(self):
         self.token = ''.join(str(random.randrange(256)) for i in xrange(16))
@@ -193,177 +227,6 @@ class AuthenticationProtocol(amp.AMP):
             return None
         return passwordData[len(token):]
 
-class AuthenticationFactory(Factory):
-    protocol = AuthenticationProtocol
-
-    def __init__(self, dataPath=None):
-        if dataPath is None:
-            dataPath = getPath(data.user, 'authserver')
-        makeDirs(dataPath)
-        self.dataPath = dataPath
-        self.alertSettings = AlertSettings(dataPath)
-
-        self.authManager = AuthManager(dataPath)
-        self.pubKey, self.privKey = self.loadKeys()
-        self.servers = {}     # Game id -> game.
-        self.nextId = 0
-        self.theme = Theme()
-        self.registeredGames = []
-
-        try:
-            self.netman = NetworkManager(6789, 6789)
-        except CannotListenError:
-            self.netman = NetworkManager(0, None)
-            log.warning('WARNING: Could not server on port 6789.')
-        self.layoutDatabase = LayoutDatabase(self)
-
-    @property
-    def settings(self):
-        return self.authManager.settings
-
-    def getPlayerCount(self):
-        '''
-        Returns the total number of online players in all games.
-        '''
-        result = 0
-        for server in self.servers.itervalues():
-            for p in server.game.world.players:
-                if not p.bot:
-                    result += 1
-        return result
-
-    def loadKeys(self):
-        '''
-        Loads public and private keys from disk or creates them and saves them.
-        '''
-        keyPath = os.path.join(self.dataPath, 'keys')
-        try:
-            pub, priv = pickle.load(open(keyPath, 'rb'))
-        except IOError:
-            pub, priv = rsa.newkeys(self.settings.keyLength)
-            pickle.dump((pub, priv), open(keyPath, 'wb'), 2)
-
-        return pub, priv
-
-    def createGame(self):
-        '''
-        Creates a new game and returns the game id.
-        '''
-        gameId = self.nextId
-        self.nextId += 1
-        authTagMan = self.authManager.newTagManager()
-        halfWidth, height = self.settings.lobbySize
-
-        game = makeLocalGame(
-            gameName=GAME_NAME,
-            layoutDatabase=self.layoutDatabase,
-            halfMapWidth=halfWidth,
-            mapHeight=height,
-            maxPlayers=self.settings.maxPlayers,
-            authTagManager=authTagMan,
-            voting=True,
-            alertSettings=self.alertSettings,
-        )
-        server = ServerNetHandler(self.netman, game, authTagMan)
-
-        def onShutdown():
-            self.netman.removeHandler(server)
-            del self.servers[gameId]
-        server.onShutdown.addListener(onShutdown)
-        self.netman.addHandler(server)
-
-        log.info('Created game (id=%d)' % (gameId,))
-        self.servers[gameId] = server
-        return gameId
-
-    @defer.deferredGenerator
-    def registerGame(self, game, version, host, port):
-        '''
-        Registers a remote game with this server.
-        '''
-        rGame = RegisteredGame(game, version, host, port)
-        result = defer.waitForDeferred(rGame.verify())
-        yield result
-        result = result.getResult()
-        if result:
-            log.info('Registered game on %s:%s', host, port)
-            self.registeredGames.append(rGame)
-        else:
-            log.info('Failed to connect to game on %s:%s', host, port)
-            raise authcommands.PortUnreachable()
-
-    def getRegisteredGames(self):
-        '''
-        Returns a list of registered games which are running.
-        '''
-        if len(self.registeredGames) == 0:
-            return defer.succeed([])
-
-        result = []
-        d = defer.Deferred()
-        remaining = [len(self.registeredGames)]
-        def gameVerified(success, rGame):
-            if success:
-                result.append(rGame)
-            else:
-                self.registeredGames.remove(rGame)
-
-            remaining[0] -= 1
-            if remaining[0] == 0:
-                d.callback(result)
-
-        for rGame in self.registeredGames:
-            rGame.verify().addCallback(gameVerified, rGame)
-
-        return d
-
-class RegisteredGame(object):
-    '''
-    Represents a remote game that has been registered with this server.
-    '''
-    def __init__(self, game, version, host, port):
-        self.game = game
-        self.version = version
-        self.host = host
-        self.port = port
-        self.lastVerified = 0
-        self.running = True
-
-    @defer.deferredGenerator
-    def verify(self):
-        '''
-        If more than GAME_VERIFY_TIME has passed since the last verification,
-        connects to the game to check whether it is still running. Returns a
-        deferred whose callback will be executed with True or False depending on
-        whether the game is still running.
-        '''
-        t = timeNow()
-        if t - self.lastVerified < GAME_VERIFY_TIME:
-            yield self.running
-            return
-        self.lastVerified = t
-
-        p = defer.waitForDeferred(ClientCreator(reactor,
-                Protocol).connectTCP(self.host, self.port, timeout=5))
-        yield p
-        try:
-            p = p.getResult()
-        except ConnectError:
-            self.running = False
-            yield False
-            return
-
-        self.running = True
-        yield True
-
-class Theme(object):
-    '''
-    Dummy object which provides enough of the interface of app.Theme for the
-    authentication server to get by.
-    '''
-    def getPath(self, *pathBits):
-        return data.getPath(data, *pathBits)
-
 SALT = 'Trosnoth'
 class AuthManager(object):
     '''
@@ -373,6 +236,7 @@ class AuthManager(object):
         self.dataPath = dataPath
         self.tags = {}      # auth tag -> user id
         self.settings = AuthServerSettings(dataPath)
+        self.tournamentSettings = TournamentSettings(dataPath)
 
     def checkUsername(self, username):
         '''
@@ -434,6 +298,189 @@ class AuthManager(object):
         except IOError:
             return username
 
+class AuthenticationFactory(Factory):
+    protocol = AuthenticationProtocol
+    authManagerClass = AuthManager
+    networkManagerClass = NetworkManager
+
+    def __init__(self, dataPath=None):
+        if dataPath is None:
+            dataPath = getPath(data.user, 'authserver')
+        makeDirs(dataPath)
+        self.dataPath = dataPath
+        self.alertSettings = AlertSettings(dataPath)
+
+        self.authManager = self.authManagerClass(dataPath)
+        self.pubKey, self.privKey = self.loadKeys()
+        self.servers = {}     # Game id -> game.
+        self.nextId = 0
+        self.theme = Theme()
+        self.registeredGames = []
+
+        try:
+            self.netman = self.networkManagerClass(6789, 6789)
+        except CannotListenError:
+            self.netman = self.networkManagerClass(0, None)
+            log.warning('WARNING: Could not start server on port 6789.')
+        self.layoutDatabase = LayoutDatabase(self)
+
+        self.notifier = self.settings.createNotificationClient()
+        if self.notifier is not None:
+            self.notifier.startService()
+
+    @property
+    def settings(self):
+        return self.authManager.settings
+
+    def getPlayerCount(self):
+        '''
+        Returns the total number of online players in all games.
+        '''
+        result = 0
+        for server in self.servers.itervalues():
+            for p in server.game.world.players:
+                if not p.bot:
+                    result += 1
+        return result
+
+    def loadKeys(self):
+        '''
+        Loads public and private keys from disk or creates them and saves them.
+        '''
+        keyPath = os.path.join(self.dataPath, 'keys')
+        try:
+            pub, priv = pickle.load(open(keyPath, 'rb'))
+        except IOError:
+            pub, priv = rsa.newkeys(self.settings.keyLength)
+            pickle.dump((pub, priv), open(keyPath, 'wb'), 2)
+
+        return pub, priv
+
+    def createGame(self):
+        '''
+        Creates a new game and returns the game id.
+        '''
+        gameId = self.nextId
+        self.nextId += 1
+        authTagMan = self.authManager.newTagManager()
+        halfWidth, height = self.settings.lobbySize
+
+        game = makeLocalGame(
+            gameName=GAME_NAME,
+            layoutDatabase=self.layoutDatabase,
+            halfMapWidth=halfWidth,
+            mapHeight=height,
+            maxPlayers=self.settings.maxPlayers,
+            authTagManager=authTagMan,
+            alertSettings=self.alertSettings,
+        )
+        server = ServerNetHandler(self.netman, game, authTagMan)
+
+        def onShutdown():
+            self.netman.removeHandler(server)
+            del self.servers[gameId]
+        server.onShutdown.addListener(onShutdown)
+        self.netman.addHandler(server)
+
+        log.info('Created game (id=%d)' % (gameId,))
+        self.servers[gameId] = server
+        self.broadcast('a new game has been created')
+
+        return gameId
+
+    def broadcast(self, message):
+        if self.notifier is None:
+            return
+        try:
+            self.notifier.broadcast('%s: %s' % (self.settings.hostName,
+                    message))
+        except:
+            log.error('Error broadcasting message', exc_info=True)
+
+    @defer.inlineCallbacks
+    def registerGame(self, game, version, host, port):
+        '''
+        Registers a remote game with this server.
+        '''
+        rGame = RegisteredGame(game, version, host, port)
+        result = yield rGame.verify()
+        if result:
+            log.info('Registered game on %s:%s', host, port)
+            self.registeredGames.append(rGame)
+        else:
+            log.info('Failed to connect to game on %s:%s', host, port)
+            raise authcommands.PortUnreachable()
+
+    def getRegisteredGames(self):
+        '''
+        Returns a list of registered games which are running.
+        '''
+        if len(self.registeredGames) == 0:
+            return defer.succeed([])
+
+        result = []
+        d = defer.Deferred()
+        remaining = [len(self.registeredGames)]
+        def gameVerified(success, rGame):
+            if success:
+                result.append(rGame)
+            else:
+                self.registeredGames.remove(rGame)
+
+            remaining[0] -= 1
+            if remaining[0] == 0:
+                d.callback(result)
+
+        for rGame in self.registeredGames:
+            rGame.verify().addCallback(gameVerified, rGame)
+
+        return d
+
+class RegisteredGame(object):
+    '''
+    Represents a remote game that has been registered with this server.
+    '''
+    def __init__(self, game, version, host, port):
+        self.game = game
+        self.version = version
+        self.host = host
+        self.port = port
+        self.lastVerified = 0
+        self.running = True
+
+    @defer.inlineCallbacks
+    def verify(self):
+        '''
+        If more than GAME_VERIFY_TIME has passed since the last verification,
+        connects to the game to check whether it is still running. Returns a
+        deferred whose callback will be executed with True or False depending on
+        whether the game is still running.
+        '''
+        t = timeNow()
+        if t - self.lastVerified < GAME_VERIFY_TIME:
+            defer.returnValue(self.running)
+            return
+        self.lastVerified = t
+
+        try:
+            yield ClientCreator(reactor, Protocol).connectTCP(self.host,
+                    self.port, timeout=5)
+        except ConnectError:
+            self.running = False
+            defer.returnValue(False)
+            return
+
+        self.running = True
+        defer.returnValue(True)
+
+class Theme(object):
+    '''
+    Dummy object which provides enough of the interface of app.Theme for the
+    authentication server to get by.
+    '''
+    def getPath(self, *pathBits):
+        return data.getPath(data, *pathBits)
+
 class AuthTagManager(object):
     '''
     Provides a gateway for the Validator to confirm that a player has
@@ -474,6 +521,8 @@ class AuthenticatedUser(object):
     '''
     Represents a user which has been authenticated on the system.
     '''
+    achievementDefs = availableAchievements
+
     def __init__(self, authManager, username):
         self.authManager = authManager
         self.username = username = username.lower()
@@ -483,6 +532,7 @@ class AuthenticatedUser(object):
 
         if not os.path.exists(self._path):
             os.makedirs(self._path)
+
         self._loadAchievements()
         self._loadStats()
 
@@ -511,10 +561,28 @@ class AuthenticatedUser(object):
         pwdHash = hasher(SALT + password).digest()
         open(os.path.join(self._path, 'password'), 'wb').write(pwdHash)
 
+    def getInfo(self):
+        filename = os.path.join(self._path, 'info')
+        if not os.path.isfile(filename):
+            return {}
+        return unrepr(open(filename, 'r').read())
+
+    def getIdentities(self):
+        info = self.getInfo()
+        return info.get('identities', [])
+
+    def setIdentities(self, identities):
+        filename = os.path.join(self._path, 'info')
+        info = self.getInfo()
+        log.info('Updating identities of %s to %r', self.username, identities)
+        info['identities'] = identities
+        open(filename, 'wb').write(repr(info))
+
     def _loadAchievements(self):
         self.achievements = {}
-        for achv in achievementDict.iterkeys():
+        for achv in self.achievementDefs.byId.iterkeys():
             self.achievements[achv] = {'unlocked' : False}
+
         try:
             f = open(os.path.join(self._path, 'achievements'), 'rU')
         except IOError:
@@ -573,78 +641,90 @@ class AuthenticatedUser(object):
         self.lastSeen = time.time()
         self._saveStats()
 
-def startServer(port=6787, dataPath=None, manholePort=6799, password=None,
-        webPort=None):
-    pf = AuthenticationFactory(dataPath)
+class main(Function):
+    authFactory = AuthenticationFactory
+    manholeFactory = telnet.ShellFactory
 
-    factory = telnet.ShellFactory()
-    try:
-        reactor.listenTCP(manholePort, factory)
-    except CannotListenError:
-        log.error('Error starting manhole on port %d', manholePort)
-    else:
-        if password is None:
-            password = ''.join(random.choice('0123456789') for i in xrange(6))
-        factory.namespace['authfactory'] = pf
-        factory.username = 'trosnoth'
-        factory.password = password
-        log.warning('Manhole started on port %d with password %r', manholePort,
-                password)
+    def startServer(self, port=6787, dataPath=None, manholePort=6799,
+            password=None, webPort=None):
+        pf = self.authFactory(dataPath)
 
-    if webPort is not None:
-        ws = WebServer(pf, port)
+        factory = self.manholeFactory()
         try:
-            reactor.listenTCP(webPort, ws.site)
-        except CannotListenError, e:
-            log.error('Could not start web server on port %d: %s', webPort, e)
+            reactor.listenTCP(manholePort, factory)
+        except CannotListenError:
+            log.error('Error starting manhole on port %d', manholePort)
         else:
-            log.info('Started webserver on port %d', webPort)
+            if password is None:
+                password = ''.join(random.choice('0123456789') for i in xrange(6))
+            factory.namespace['authfactory'] = pf
+            def getGame():
+                games = pf.servers.values()
+                if len(games) == 0:
+                    log.info('getGame: No games available')
+                    return None
+                return games[0].game
+            factory.namespace['getGame'] = getGame
 
-    try:
-        reactor.listenTCP(port, pf)
-    except CannotListenError:
-        log.error('Error listening on port %d.', port)
-    else:
-        log.info('Started Trosnoth authentication server on port %d.', port)
-        reactor.run()
+            factory.username = 'trosnoth'
+            factory.password = password
+            log.warning('Manhole started on port %d with password %r', manholePort,
+                    password)
 
-def main():
-    from optparse import OptionParser
-    parser = OptionParser()
-    parser.add_option('-p', '--port', action='store', dest='port', default=6787,
-        help='which port to run the authentication server on')
-    parser.add_option('-D', '--datapath', action='store', dest='dataPath',
-        default=None, help='where to store the authentication server data')
-    parser.add_option('-m', '--manhole', action='store', dest='manholePort',
-        default=6799, help='which port to run the manhole on')
-    parser.add_option('--password', action='store', dest='manholePassword',
-        default=None, help='the password to use for the manhole')
-    parser.add_option('-w', '--webport', action='store', dest='webPort',
-        default='8080', help='the port on which to launch the web service. '
-        'Default is 8080. To disable web service, use --webport= with no '
-        'parameter.')
-    parser.add_option('-d', '--debug', action='store_true', dest='debug',
-        help='show debug-level messages on console')
-    parser.add_option('-l', '--log-file', action='store', dest='logFile',
-        help='file to write logs to')
+        if webPort is not None:
+            ws = WebServer(pf, port)
+            try:
+                reactor.listenTCP(webPort, ws.site)
+            except CannotListenError, e:
+                log.error('Could not start web server on port %d: %s', webPort, e)
+            else:
+                log.info('Started webserver on port %d', webPort)
 
-    options, args = parser.parse_args()
-    if len(args) > 0:
-        parser.error('no arguments expected')
-
-    initLogging(options.debug, options.logFile)
-
-    if not options.webPort:
-        webPort = None
-    else:
         try:
-            webPort = int(options.webPort)
-        except ValueError:
-            options.error('Invalid port: %r' % (options.webPort,))
+            reactor.listenTCP(port, pf)
+        except CannotListenError:
+            log.error('Error listening on port %d.', port)
+        else:
+            log.info('Started Trosnoth authentication server on port %d.', port)
+            reactor.run()
 
-    startServer(port=int(options.port), dataPath=options.dataPath,
-            manholePort=int(options.manholePort),
-            password=options.manholePassword, webPort=webPort)
+    def run(self):
+        from optparse import OptionParser
+        parser = OptionParser()
+        parser.add_option('-p', '--port', action='store', dest='port', default=6787,
+            help='which port to run the authentication server on')
+        parser.add_option('-D', '--datapath', action='store', dest='dataPath',
+            default=None, help='where to store the authentication server data')
+        parser.add_option('-m', '--manhole', action='store', dest='manholePort',
+            default=6799, help='which port to run the manhole on')
+        parser.add_option('--password', action='store', dest='manholePassword',
+            default=None, help='the password to use for the manhole')
+        parser.add_option('-w', '--webport', action='store', dest='webPort',
+            default='8080', help='the port on which to launch the web service. '
+            'Default is 8080. To disable web service, use --webport= with no '
+            'parameter.')
+        parser.add_option('-d', '--debug', action='store_true', dest='debug',
+            help='show debug-level messages on console')
+        parser.add_option('-l', '--log-file', action='store', dest='logFile',
+            help='file to write logs to')
+
+        options, args = parser.parse_args()
+        if len(args) > 0:
+            parser.error('no arguments expected')
+
+        initLogging(options.debug, options.logFile)
+
+        if not options.webPort:
+            webPort = None
+        else:
+            try:
+                webPort = int(options.webPort)
+            except ValueError:
+                options.error('Invalid port: %r' % (options.webPort,))
+
+        self.startServer(port=int(options.port), dataPath=options.dataPath,
+                manholePort=int(options.manholePort),
+                password=options.manholePassword, webPort=webPort)
 
 if __name__ == '__main__':
     main()
