@@ -73,7 +73,6 @@ function practise() {
 }
 </script>''', contents='''
   <div id="home_play_btn"><a href="/play">play</a></div>
-  <div id="home_play_special_btn"><a href="/playspecial">play special</a></div>
   <div id="home_practise_btn"><a href="/practise"
     onclick="practise(); return false;">practise</a></div>
   <div id="home_archives_btn"><a href="/archives">archives</a></div>
@@ -695,14 +694,6 @@ def message(msg, title='Trosnoth'):
             contents='<div class="central_message">%(message)s</div>' %
             {'message': cgi.escape(str(msg))})
 
-@core.addPage('/playspecial')
-def playSpecial(state, request):
-    cmd = ['trosnoth', '--play-special']
-    subprocess.Popen(cmd, stdout=subprocess.PIPE)
-    request.setResponseCode(302)
-    request.setHeader('Location', '/')
-    return message('Play special screen launched.')
-
 @core.addPage('/settings/controls')
 def controls(state, request):
     launchControls(state)
