@@ -55,7 +55,8 @@ class ThemeColours(object):
             teamNum = 0
         else:
             teamNum = ord(team.id) - 64
-        return getattr(self, 'team%d%s' % (teamNum, colourId))
+        return getattr(self, 'team%d%s' % (teamNum, colourId),
+                getattr(self, 'team0%s' % (colourId,)))
 
     sysMessageColour = teamColour('msg')
     chatColour = teamColour('chat')
@@ -65,6 +66,7 @@ class ThemeColours(object):
     miniMapGhostColour = teamColour('Mn_gh')
     miniMapStarColour = teamColour('Mn_star')
     miniMapZoneHighlight = teamColour('Mn_highlight')
+    leaderboard = teamColour('Leaderboard')
 
 def cachedProperty(fn):
     def spriteFunction(self):
@@ -112,7 +114,7 @@ class ThemeSprites(object):
     pointer = image('pointer.bmp')
     smallStar = image('smallstar.png')
     star = image('star.png')
-    grenade = image('grenade.bmp')
+    grenade = image('grenade.png')
     neutralOrb = image('greyOrb.png')
     scenery = image('scenery.png', colourkey=None)
     shoxGun3 = image('shoxgun-3.png')
@@ -225,8 +227,10 @@ class ThemeSprites(object):
             path = 'blue'
         elif teamId == 'B':
             path = 'red'
-        else:
+        elif teamId == '\x00':
             path = 'white'
+        else:
+            path = 'team' + teamId + 'Head'
         if bot:
             path += '-bot'
         return SingleImage(self.theme.loadSprite(path + '.png'))
@@ -242,6 +246,17 @@ class ThemeSprites(object):
             return self.star
         return self.theme.loadSprite(path)
 
+
+    def teamGrenade(self, team):
+        teamId = getTeamId(team)
+        if teamId == 'A':
+            path = 'blueGrenade.png'
+        elif teamId == 'B':
+            path = 'redGrenade.png'
+        else:
+            return self.grenade
+        return self.theme.loadSprite(path)
+
     @cached
     def zoneBackground(self, team):
         if team is None:
@@ -344,14 +359,16 @@ class ThemeSprites(object):
 
     @cached
     def shotAnimation(self, team):
-        if team is None:
-            shotImg1 = self.theme.loadSprite('blueShot.png')
-            shotImg2 = self.theme.loadSprite('redShot.png')
-            return Animation(0.07, timeNow, shotImg1, shotImg2)
-        elif getTeamId(team) == 'A':
+        teamId = getTeamId(team)
+
+        if teamId == 'A':
             colour = 'blue'
-        else:
+        elif teamId == 'B':
             colour = 'red'
+        elif teamId == '\x00':
+            colour = 'default'
+        else:
+            colour = 'team' + teamId
         return self.theme.makeShot(colour)
 
 class Theme(object):
@@ -402,14 +419,14 @@ class Theme(object):
 
         for fontName, defaultDetails in DEFAULT_FONTS.iteritems():
             if fontName in fontData:
-                fontFile, size = fontData[fontName]
+                fontFile, size, bold = fontData[fontName]
             else:
-                fontFile, size = defaultDetails
+                fontFile, size, bold = defaultDetails.unpack()
 
             if fontName in UNSCALED_FONTS:
-                font = Font(fontFile, size)
+                font = Font(fontFile, size, bold)
             else:
-                font = ScaledFont(fontFile, size)
+                font = ScaledFont(fontFile, size, bold)
             self.app.fonts.addFont(fontName, font)
 
     def _getFontData(self):
@@ -546,54 +563,67 @@ class Theme(object):
             images.append(self.loadTeamSprite(filename, teamId))
         return images
 
+class F(object):
+
+    def __init__(self, fontFile, size, bold = False):
+        self.fontFile = fontFile
+        self.size = size
+        self.bold = bold
+
+    def unpack(self):
+        return (self.fontFile, self.size, self.bold)
+
 DEFAULT_FONTS = {
-    'default': ('Junction.ttf', 24),
-    'defaultTextBoxFont': ('Junction.ttf', 20),
-    'unobtrusivePromptFont': ('Junction.ttf', 28),
-    'chatFont': ('Junction.ttf', 25),
+    'default': F('Junction.ttf', 24),
+    'defaultTextBoxFont': F('Junction.ttf', 20),
+    'unobtrusivePromptFont': F('Junction.ttf', 28),
+    'chatFont': F('Junction.ttf', 25),
+    'newChatFont': F('Vera.ttf', 14, True),
 
-    'winMessageFont': ('Junction.ttf', 72),
+    'winMessageFont': F('Junction.ttf', 72),
 
-    'nameFont': ('Junction.ttf', 20),
+    'nameFont': F('Junction.ttf', 20),
 
-    'hugeMenuFont': ('Junction.ttf', 54),
-    'bigMenuFont': ('Junction.ttf', 36),
-    'mainMenuFont': ('Junction.ttf', 36),
-    'serverListFont': ('Junction.ttf', 24),
-    'timerFont': ('Junction.ttf', 32),
-    'consoleFont': ('orbitron-light.ttf', 20),
-    'ampleMenuFont': ('Junction.ttf', 40),
-    'mediumMenuFont': ('Junction.ttf', 36),
-    'menuFont': ('Junction.ttf', 30),
-    'smallMenuFont': ('Junction.ttf', 20),
-    'ingameMenuFont': ('FreeSans.ttf', 12),
-    'versionFont': ('Junction.ttf', 16),
-    'scrollingButtonsFont': ('Junction.ttf', 24),
-    'zoneBarFont': ('Junction.ttf', 24),
-    'dialogButtonFont': ('KLEPTOCR.TTF', 50),
+    'hugeMenuFont': F('Junction.ttf', 54),
+    'bigMenuFont': F('Junction.ttf', 36),
+    'mainMenuFont': F('Junction.ttf', 36),
+    'serverListFont': F('Junction.ttf', 24),
+    'timerFont': F('Junction.ttf', 32),
+    'consoleFont': F('orbitron-light.ttf', 20),
+    'ampleMenuFont': F('Junction.ttf', 40),
+    'mediumMenuFont': F('Junction.ttf', 36),
+    'menuFont': F('Junction.ttf', 30),
+    'smallMenuFont': F('Junction.ttf', 20),
+    'ingameMenuFont': F('FreeSans.ttf', 12),
+    'versionFont': F('Junction.ttf', 16),
+    'scrollingButtonsFont': F('Junction.ttf', 24),
+    'zoneBarFont': F('Junction.ttf', 24),
+    'dialogButtonFont': F('KLEPTOCR.TTF', 50),
+    'serverSelectionCheckboxesFont': F('Junction.ttf', 28),
 
-    'customSizeButtonFont': ('KLEPTOCR.TTF', 100),
+    'customSizeButtonFont': F('KLEPTOCR.TTF', 100),
 
-    'messageFont': ('Junction.ttf', 20),
-    'leaderboardFont': ('KLEPTOCR.TTF', 21),
+    'messageFont': F('Junction.ttf', 20),
+    'leaderboardFont': F('KLEPTOCR.TTF', 21),
 
-    'smallNoteFont': ('Junction.ttf', 22),
-    'labelFont': ('Junction.ttf', 32),
-    'captionFont': ('Junction.ttf', 35),
-    'keymapFont': ('Junction.ttf', 20),
-    'keymapInputFont': ('Junction.ttf', 20),
+    'smallNoteFont': F('Junction.ttf', 22),
+    'labelFont': F('Junction.ttf', 32),
+    'captionFont': F('Junction.ttf', 35),
+    'keymapFont': F('Junction.ttf', 20),
+    'keymapInputFont': F('Junction.ttf', 20),
 
-    'achievementTitleFont': ('orbitron-light.ttf', 21),
-    'achievementNameFont': ('Junction.ttf', 18),
+    'achievementTitleFont': F('orbitron-light.ttf', 21),
+    'achievementNameFont': F('Junction.ttf', 18),
 
-    'connectionFailedFont': ('Junction.ttf', 32),
+    'connectionFailedFont': F('Junction.ttf', 32),
 
-    'creditsFont': ('Junction.ttf', 24),
-    'creditsH2': ('KLEPTOCR.TTF', 48),
-    'creditsH1': ('KLEPTOCR.TTF', 60),
+    'creditsFont': F('Junction.ttf', 24),
+    'creditsH2': F('KLEPTOCR.TTF', 48),
+    'creditsH1': F('KLEPTOCR.TTF', 60),
 }
 
 UNSCALED_FONTS = set([
     'nameFont',
     'ingameMenuFont',
+    'newChatFont'
 ])
