@@ -21,10 +21,11 @@ from trosnoth.gui.framework.elements import TextElement, PictureElement
 from trosnoth.gui.common import (Area, FullScreenAttachedPoint, Location,
         ScaledSize)
 import trosnoth.gui.framework.framework as framework
-from trosnoth.gamerecording.achievementlist import getAchievementDetails
+from trosnoth.gamerecording.achievementlist import availableAchievements
 from trosnoth.utils.twist import WeakLoopingCall
 
 class AchievementBox(framework.CompoundElement):
+    achievementDefs = availableAchievements
     def __init__(self, app, player, achievementId):
         super(AchievementBox, self).__init__(app)
         self.app = app
@@ -50,7 +51,7 @@ class AchievementBox(framework.CompoundElement):
                 self.height + 10), 'midbottom'), 'midtop'), self.borderColour)
 
         self.nameText = TextElement(self.app,
-                getAchievementDetails(achievementId)[0],
+                self.achievementDefs.getAchievementDetails(achievementId)[0],
                 self.fonts.achievementNameFont,
                 Location(FullScreenAttachedPoint(ScaledSize(73 / 2, -100 - 13),
                 'midbottom'), 'midbottom'), self.colours.black)
@@ -99,7 +100,8 @@ class AchievementBox(framework.CompoundElement):
             self.titleText.setText("%d ACHIEVEMENTS UNLOCKED!" %
                     len(self.achievements))
 
-        self.nameText.setText(getAchievementDetails(self.achievements[0])[0])
+        self.nameText.setText(self.achievementDefs.getAchievementDetails(
+                self.achievements[0])[0])
         self._updateImage()
 
     def _setColours(self):
