@@ -34,16 +34,17 @@ from trosnoth.trosnothgui.ingame.gamevote import GameVoteMenu, NicknameBox
 from trosnoth.trosnothgui.ingame.gauges import (TurretGauge, RespawnGauge,
         UpgradeGauge, GunGauge, StarGauge)
 from trosnoth.trosnothgui.ingame.achievementBox import AchievementBox
+from trosnoth.trosnothgui.ingame.chatBox import ChatBox
 from trosnoth.gui.framework.dialogbox import DialogResult
 from trosnoth.trosnothgui.settings.settings import SettingsMenu
 
-from trosnoth.model.upgrades import allUpgrades, Grenade
-from trosnoth.model.universe_base import GameState
+from trosnoth.model.upgrades import (allUpgrades, Grenade, Ninja, Turret,
+        Directatorship, Ricochet, Shoxwave)
 from trosnoth.model.team import Team
 from trosnoth.model.player import Player
 
-from trosnoth.messages import (BuyUpgradeMsg, DeleteUpgradeMsg, SetCaptainMsg,
-        TeamIsReadyMsg, RespawnRequestMsg, ChangeNicknameMsg, MarkZoneMsg)
+from trosnoth.messages import (BuyUpgradeMsg, DeleteUpgradeMsg,
+        RespawnRequestMsg, ChangeNicknameMsg, MarkZoneMsg)
 
 log = logging.getLogger('detailsInterface')
 
@@ -53,6 +54,7 @@ class DetailsInterface(framework.CompoundElement):
     def __init__(self, app, gameInterface):
         super(DetailsInterface, self).__init__(app)
         world = gameInterface.world
+        self.gameController = gameInterface.gameController
         self.gameInterface = gameInterface
 
         # Maximum number of messages viewable at any one time
@@ -64,9 +66,6 @@ class DetailsInterface(framework.CompoundElement):
         self.currentMessages = MessageBank(self.app, maxView, 50,
                 Location(FullScreenAttachedPoint(ScaledSize(-40,-40),
                 'bottomright'), 'bottomright'), 'right', 'bottom', font)
-        self.currentChats = MessageBank(self.app, maxView, 50,
-                Location(FullScreenAttachedPoint(ScaledSize(40,256),'topleft'),
-                'topleft'), 'left', 'top', font)
 
         # If we want to keep a record of all messages and their senders
         self.input = None
@@ -85,6 +84,8 @@ class DetailsInterface(framework.CompoundElement):
         self.settingsMenu = SettingsMenu(app, onClose=self.hideSettings,
                 showThemes=False)
 
+        self.chatBox = ChatBox(app, self.world, self.gameInterface)
+
         menuloc = Location(FullScreenAttachedPoint((0,0), 'bottomleft'),
                 'bottomleft')
         self.menuManager = mainMenu.MainMenu(self.app, menuloc, self,
@@ -93,9 +94,10 @@ class DetailsInterface(framework.CompoundElement):
         self.gameVoteMenu = GameVoteMenu(app, world,
                 onChange=self._castGameVote)
         self._gameVoteUpdateCounter = 0
-        self.elements = [self.currentMessages, self.currentChats,
-                self.upgradeDisplay, self.reloadGauge, self.starGroup,
-                self.gameVoteMenu, self.udpNotifier]
+        self.elements = [self.currentMessages, self.upgradeDisplay,
+                self.reloadGauge, self.starGroup,
+                self.gameVoteMenu, self.udpNotifier,
+                self.chatBox]
 
         self.upgradeMap = dict((upgradeClass.action, upgradeClass) for
                 upgradeClass in allUpgrades)
@@ -126,7 +128,7 @@ class DetailsInterface(framework.CompoundElement):
         self.controller.send(msg)
 
     def _updateGameVoteMenu(self):
-        if self.world.gameState == GameState.Ended and not self.world.onceOnly:
+        if self.gameController.canVote():
             self.gameVoteMenu.active = True
         else:
             self.gameVoteMenu.active = False
@@ -165,7 +167,7 @@ class DetailsInterface(framework.CompoundElement):
                 self.respawnGauge = RespawnGauge(self.app, Area(
                         FullScreenAttachedPoint(ScaledSize(0,-20),
                         'midbottom'), ScaledSize(100,30), 'midbottom'),
-                        player)
+                        player, self.gameController)
                 self.elements.append(self.respawnGauge)
         elif not player.ghost:
             self.elements.remove(self.respawnGauge)
@@ -213,7 +215,7 @@ class DetailsInterface(framework.CompoundElement):
                     self.currentUpgrade.upgradeType, random.randrange(1<<32)))
 
     def _changeNickname(self):
-        if self.world.gameState == GameState.InProgress:
+        if not self.world.canRename():
             self.newMessage('Cannot change nickname after game has started',
                     self.app.theme.colours.errorMessageColour)
             return
@@ -289,12 +291,6 @@ class DetailsInterface(framework.CompoundElement):
             elif action == 'chat':
                 self.chat()
                 self.menuManager.manager.reset()
-            elif action == 'captain':
-                self.becomeCaptain()
-                self.menuManager.manager.reset()
-            elif action == 'team ready':
-                self.teamReady()
-                self.menuManager.manager.reset()
             elif action == 'select upgrade':
                 if self.menuManager is not None:
                     self.menuManager.showBuyMenu()
@@ -313,42 +309,31 @@ class DetailsInterface(framework.CompoundElement):
         self.currentMessages.newMessage(text, colour)
 
     def newChat(self, text, sender, private):
-        if sender is self.player:
-            # Don't bother receiving messages from myself.
-            return
-
         nick = sender.nick
         if not private:
             # Message for everyone
-            extra = ': '
+            text = ": " + text
 
         elif isinstance(private, Player) and self.player is private:
             # Destined for the one player
-            extra = ' (private): '
+            text = " (private): " + text
 
         elif (isinstance(private, Team) and self.player is not None and
                 self.player.team == private):
             # Destined for the one team.
-            extra = ': '
+            text = " (team): " + text
 
         else:
             # May not have been destined for our player after all.
             return
-        allText = nick + extra + text
-        self.currentChats.newMessage(allText,
-                self.app.theme.colours.chatColour(sender.team))
 
-    def sentPublicChat(self, team, text):
-        self.currentChats.newMessage('Me (open): %s' % (text,),
-                self.app.theme.colours.chatColour(team))
+        colour = self.app.theme.colours.chatColour(sender.team)
 
-    def sentPrivateChat(self, team, nick, text):
-        self.currentChats.newMessage('Me (to_%s): %s' % (nick, text),
-                self.app.theme.colours.chatColour(team))
+        self.chatBox.newMessage(text,
+                nick, colour)
 
-    def sentTeamChat(self, team, text):
-        self.currentChats.newMessage('Me: %s' % (text,),
-                self.app.theme.colours.chatColour(team))
+    def newServerChat(self, text):
+        self.chatBox.newServerMessage(text)
 
     def endInput(self):
         if self.input:
@@ -377,14 +362,16 @@ class DetailsInterface(framework.CompoundElement):
             except ValueError:
                 pass
 
-
-
     def chat(self):
         if not self.player:
             return
 
-        self.gameInterface.gameViewer.openChat(self.player)
-
+        if self.chatBox.isOpen():
+            self.chatBox.close()
+        else:
+            pygame.key.set_repeat(300, 30)
+            self.chatBox.open()
+            self.chatBox.setPlayer(self.player)
 
     def abandon(self, player):
         '''
@@ -424,7 +411,18 @@ class DetailsInterface(framework.CompoundElement):
             upgrade = 'an upgrade'
         else:
             upgrade = str(upgrade)
-        message = '%s is using %s' % (player.nick, player.upgrade)
+
+        if type(player.upgrade) == Grenade:
+            message = '%s has thrown a %s' % (player.nick, player.upgrade)
+        elif type(player.upgrade) in [Ninja, Turret]:
+            message = '%s has become a %s' % (player.nick, player.upgrade)
+        elif type(player.upgrade) == Directatorship:
+            message = '%s has become a Directator' % player.nick
+        elif type(player.upgrade) in [Ricochet, Shoxwave]:
+            message = '%s is using %s' % (player.nick, player.upgrade)
+        else:
+            message = '%s is using a %s' % (player.nick, player.upgrade)
+
         self.newMessage(message)
 
     def upgradeDestroyed(self, player, upgrade):
@@ -435,14 +433,6 @@ class DetailsInterface(framework.CompoundElement):
         message = "%s's %s is gone" % (player.nick, upgrade)
         self.newMessage(message)
 
-    def becomeCaptain(self):
-        if self.player:
-            self.controller.send(SetCaptainMsg(self.player.id))
-
-    def teamReady(self):
-        if self.player:
-            self.controller.send(TeamIsReadyMsg(self.player.id))
-
     def showPlayerDetails(self):
         self.gameInterface.gameViewer.toggleLeaderBoard()
 
