@@ -22,9 +22,8 @@ from trosnoth.gui.framework import framework, hotkey, console
 from trosnoth.gui.framework.elements import TextElement
 from trosnoth.gui import keyboard
 from trosnoth.gui.common import Region, Screen, Location, Canvas
-from trosnoth.gui.errors import MultiWindowException
 
-from trosnoth.gamerecording.achievementlist import getAchievementDetails
+from trosnoth.gamerecording.achievementlist import availableAchievements
 
 from trosnoth.trosnothgui.ingame import viewManager
 from trosnoth.trosnothgui.ingame.replayInterface import (ReplayMenu,
@@ -47,7 +46,7 @@ from trosnoth.utils.event import Event
 from trosnoth.utils.twist import WeakCallLater
 
 from trosnoth.messages import (TaggingZoneMsg, ChatFromServerMsg, ChatMsg,
-        GameStartMsg, GameOverMsg, TeamIsReadyMsg, SetCaptainMsg,
+        GameStartMsg, GameOverMsg,
         StartingSoonMsg, ShotFiredMsg, RespawnMsg, CannotRespawnMsg,
         PlayerKilledMsg, JoinRequestMsg, JoinSuccessfulMsg, CannotJoinMsg,
         AddPlayerMsg, RemovePlayerMsg, PlayerHasUpgradeMsg,
@@ -64,7 +63,9 @@ class GameInterface(framework.CompoundElement, Component):
     controller = Plug()
     eventPlug = Plug()
 
-    def __init__(self, app, world, onDisconnectRequest=None,
+    achievementDefs = availableAchievements
+
+    def __init__(self, app, game, onDisconnectRequest=None,
             onConnectionLost=None, replay=False, authTag=0):
         framework.CompoundElement.__init__(self, app)
         Component.__init__(self)
@@ -77,19 +78,19 @@ class GameInterface(framework.CompoundElement, Component):
         self.onConnectionLost = Event()
         if onConnectionLost is not None:
             self.onConnectionLost.addListener(onConnectionLost)
-
-        self.world = world
+        self.game = game
+        self.gameController = game.gameController
+        self.world = game.world
 
         self.keyMapping = keyboard.KeyboardMapping(keymap.default_game_keys)
         self.updateKeyMapping()
-
-        self.gameViewer = viewManager.GameViewer(self.app, self, world, replay)
+        self.gameViewer = viewManager.GameViewer(self.app, self, game, replay)
         if replay:
-            self.gameMenu = ReplayMenu(self.app, world)
+            self.gameMenu = ReplayMenu(self.app, self.world)
             closeReplay = self.onDisconnectRequest.execute
             self.gameMenu.onDisconnectRequest.addListener(closeReplay)
         else:
-            self.gameMenu = GameMenu(self.app, self, world)
+            self.gameMenu = GameMenu(self.app, self, self.game)
         self.detailsInterface = DetailsInterface(self.app, self)
         self.runningPlayerInterface = None
         self.observerInterface = ObserverInterface(self.app, self)
@@ -279,10 +280,6 @@ class GameInterface(framework.CompoundElement, Component):
         if self.gameViewer.timerBar is not None:
             self.gameViewer.timerBar.kill()
             self.gameViewer.timerBar = None
-        try:
-            self.gameViewer.closeChat()
-        except MultiWindowException:
-            pass
         pygame.key.set_repeat(300, 30)
 
     def gameOver(self, winningTeam):
@@ -372,24 +369,6 @@ class GameInterface(framework.CompoundElement, Component):
             self.gameViewer.timerBar.gameFinished()
         #self.playSound('end', 1)
 
-    @handler(SetCaptainMsg, eventPlug)
-    def setCaptain(self, msg):
-        try:
-            player = self.world.getPlayer(msg.playerId)
-            self.detailsInterface.newMessage("%s is %s's captain" %
-                    (player.nick, player.team), self.getChatColour(player.team))
-        except Abort:
-            pass
-
-    @handler(TeamIsReadyMsg, eventPlug)
-    def teamIsReady(self, msg):
-        try:
-            player = self.world.getPlayer(msg.playerId)
-            self.detailsInterface.newMessage('%s is ready' %
-                    (player.team,), self.getChatColour(player.team))
-        except Abort:
-            pass
-
     def getChatColour(self, team):
         return self.app.theme.colours.chatColour(team)
 
@@ -523,23 +502,20 @@ class GameInterface(framework.CompoundElement, Component):
 
     def sendPrivateChat(self, player, targetId, text):
         self.controller.send(ChatMsg(player.id, 'p', targetId, text.encode()))
-        try:
-            nick = self.world.playerWithId[targetId].nick
-        except KeyError:
-            nick = '???'
-        self.detailsInterface.sentPrivateChat(player.team, nick, text)
 
     def sendTeamChat(self, player, text):
         self.controller.send(ChatMsg(player.id, 't', player.teamId,
                 text.encode()))
-        self.detailsInterface.sentTeamChat(player.team, text)
 
     def sendPublicChat(self, player, text):
         self.controller.send(ChatMsg(player.id, 'o', '\x00', text.encode()))
-        self.detailsInterface.sentPublicChat(player.team, text)
 
     @handler(ChatMsg, eventPlug)
     def chat(self, msg):
+        if msg.kind == 's':
+            self.detailsInterface.newServerChat(msg.text.decode())
+            return
+
         try:
             sender = self.world.getPlayer(msg.playerId)
             if msg.kind == 't':
@@ -559,7 +535,8 @@ class GameInterface(framework.CompoundElement, Component):
             player = self.world.getPlayer(msg.playerId)
         except Abort:
             return
-        achievementName = getAchievementDetails(msg.achievementId)[0]
+        achievementName = self.achievementDefs.getAchievementDetails(
+                msg.achievementId)[0]
         self.detailsInterface.newMessage('%s has unlocked "%s"!' %
                 (player.nick, achievementName),
                 self.app.theme.colours.achievementMessageColour)
