@@ -22,15 +22,14 @@ from trosnoth.gui.errors import MultiWindowException
 from trosnoth.model.universe_base import GameState
 from trosnoth.utils.twist import WeakCallLater
 
-MAX_NAME_LENGTH = 20
-
 class GameMenu(InGameMenu):
     '''This is not actually a menu any more, but rather a controller used only
     when joining the game.'''
 
-    def __init__(self, app, gameInterface, world):
+    def __init__(self, app, gameInterface, game):
         super(GameMenu, self).__init__(app)
-        self.world = world
+        self.gameController = game.gameController
+        self.world = game.world
         self.interface = gameInterface
         self.joined = False
         self.joiningInfo = None
@@ -38,16 +37,16 @@ class GameMenu(InGameMenu):
 
         self.joiningScreen = JoiningDialog(self.app, self)
 
-        self.joinGameDialog = JoinGameDialog(self.app, self, world)
+        self.joinGameDialog = JoinGameDialog(self.app, self, self.world)
         self.joinGameDialog.onClose.addListener(self._joinDlgClose)
         self.joinGameDialog.show()
 
-        if world.gameState == GameState.Ended:
+        if self.gameController.state() in (GameState.Lobby, GameState.Ended):
             WeakCallLater(0.1, self, 'gotWorldParameters')
 
     def gotWorldParameters(self):
-        if (self.joinGameDialog.showing and self.world.gameState ==
-                GameState.Ended):
+        if (self.joinGameDialog.showing and self.gameController.state() in
+                (GameState.Lobby, GameState.Ended)):
             nick = self.app.identitySettings.nick
             if nick is not None:
                 self.joinGameDialog.close()
@@ -71,7 +70,6 @@ class GameMenu(InGameMenu):
             self.interface.disconnect()
         else:
             nick = self.joinGameDialog.nickBox.value.strip()
-            nick = nick[:MAX_NAME_LENGTH]
             self.app.identitySettings.setNick(nick)
 
             team = self.joinGameDialog.selectedTeam
@@ -113,7 +111,7 @@ class GameMenu(InGameMenu):
             # Nickname is already being used.
             self.showMessage('That name is already being used!')
         elif result[0] == 'badn':
-            # Nickname is already being used.
+            # Nickname is too short or long.
             self.showMessage('That name is not allowed!')
         elif result[0] == 'auth':
             self.showMessage('You are not authorised to join!')
