@@ -26,9 +26,9 @@ from trosnoth.model.universe_base import GameState
 from trosnoth.utils.twist import WeakCallLater
 
 class GameTimer(framework.CompoundElement):
-    def __init__(self, app, world):
+    def __init__(self, app, game):
         super(GameTimer, self).__init__(app)
-        self.world = world
+        self.gameController = game.gameController
         self.app = app
 
         # Change these constants to say where the box goes
@@ -48,6 +48,7 @@ class GameTimer(framework.CompoundElement):
 
 
         self.gameTimer = None
+        self.gameClock = None
         self.timerAdjustLoop = None
         self.countingDown = None
 
@@ -56,30 +57,30 @@ class GameTimer(framework.CompoundElement):
         # Value the countdown has to get to before it starts to flash
         self.flashValue = 30
 
+
         self.loop()
 
     def loop(self):
         self.timerAdjustLoop = WeakCallLater(10, self, 'loop')
-        self.syncTimer()
+        if self.gameController.hasTimeLimit() != self.countingDown:
+            self.resetTimer()
+        else:
+            self.syncTimer()
 
     def checkState(self):
-        if self.world.gameState != GameState.InProgress:
+        if self.gameController.state() != GameState.InProgress:
             if self.running:
                 self.gameFinished()
         elif not self.running:
-            self.gameStarted(self.world.getTimeLeft())
+            self.gameStarted()
 
     def syncTimer(self):
-        self.checkState()
         if self.gameTimer is None:
             return
-        time = self.world.getTimeLeft()
 
-        if time < 0:
-            self.gameTimer.counted = -time
-        else:
-            self.gameTimer.countTo = time
-            self.gameTimer.counted = 0
+        self.gameTimer.counted = self.gameController.getGameTime()
+        if self.gameController.hasTimeLimit():
+            self.gameTimer.countTo = self.gameController.duration()
 
     def _flash(self, flashState):
         if flashState == 0:
@@ -88,28 +89,34 @@ class GameTimer(framework.CompoundElement):
             self.gameClock.setColours(self.app.theme.colours.timerFontColour)
 
     def kill(self):
-        if (self.world.gameState == GameState.InProgress and
+        if (self.gameController.state() == GameState.InProgress and
                 self.timerAdjustLoop is not None):
             self.timerAdjustLoop.cancel()
         self.gameTimer = None
 
-    def gameStarted(self, time):
+    def gameStarted(self):
         self.running = True
-
-        if time > 0:
-            self.gameTimer = timer.CountdownTimer(time, highest = 'minutes')
-            self.countingDown = True
-        else:
-            self.gameTimer = timer.Timer(startAt = -time, highest = 'minutes')
-            self.countingDown = False
-
-        self.gameClock = clock.Clock(self.app, self.gameTimer,
+        self.gameClock = clock.Clock(self.app, None, # We'll set the timer soon...
                 Location(FullScreenAttachedPoint(ScaledSize(0, 0), 'midtop'),
                 'midtop'), self.app.screenManager.fonts.timerFont,
                 self.app.theme.colours.timerFontColour)
-        self.gameTimer.start()
+                
+        self.resetTimer()
         self.elements = [self.gameClock]
 
+    def resetTimer(self):
+        if self.gameController.hasTimeLimit():
+            self.gameTimer = timer.CountdownTimer(highest = 'minutes')
+            self.countingDown = True
+        else:
+            self.gameTimer = timer.Timer(highest = 'minutes')
+            self.countingDown = False
+        if self.gameClock is not None:
+            self.gameClock.setTimer(self.gameTimer)
+            self._flash(1)
+        self.syncTimer()
+        self.gameTimer.start()
+
     def gameFinished(self):
         self.running = False
         if self.timerAdjustLoop is not None:
@@ -123,13 +130,15 @@ class GameTimer(framework.CompoundElement):
     def _getRect(self):
         return self.area.getRect(self.app)
 
-
-    def draw(self, surface):
+    def tick(self, deltaT):
+        super(GameTimer, self).tick(deltaT)
         self.checkState()
         if (self.running and self.countingDown and self.gameTimer is not None
                 and self.gameTimer.getCurTime() <= self.flashValue):
             self._flash(int((self.gameTimer.getCurTime() / self.flashCycle) %
                     2))
+
+    def draw(self, surface):
         timerBox = self._getRect()
         # Box background
         surface.fill(self.app.theme.colours.timerBackground, timerBox)
