@@ -15,6 +15,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
+import logging
+
 from trosnoth.gui.framework.menu import MenuDisplay
 from trosnoth.gui.menu.menu import MenuManager, Menu, MenuItem
 from trosnoth.gui.common import Abs, Location, Region, Canvas, ScaledSize
@@ -27,6 +29,8 @@ from trosnoth.messages import (PlayerIsReadyMsg, SetPreferredTeamMsg,
         SetPreferredDurationMsg, SetPreferredSizeMsg)
 from trosnoth.utils.event import Event
 
+log = logging.getLogger(__name__)
+
 MENU_WIDTH = 120
 class GameVoteMenu(CompoundElement):
     def __init__(self, app, world, onChange=None):
@@ -84,6 +88,7 @@ class GameVoteSubMenu(MenuDisplay):
         stdColour = (255, 255, 0)
         hvrColour = (0, 255, 255)
         backColour = (0, 64, 192)
+        disabledColour = (128, 128, 128)
         autosize = True
         hidable = True
         size = Abs(MENU_WIDTH, 10)
@@ -92,8 +97,8 @@ class GameVoteSubMenu(MenuDisplay):
 
         location = Location(Abs(0, 0))
         MenuDisplay.__init__(self, app, location, size, font, manager,
-                titleColour, stdColour, hvrColour, backColour, autosize,
-                hidable, keymapping)
+                titleColour, stdColour, hvrColour, disabledColour, backColour,
+                autosize, hidable, keymapping)
 
 class CountedMenuItem(MenuItem):
     def __init__(self, caption, action=None, listener=None):
@@ -131,18 +136,29 @@ class SelectionMenu(Menu):
 
 class ReadyMenu(SelectionMenu):
     def __init__(self, gvm, pos):
+        self.scratchItem = CountedMenuItem('Ready', (True, False))
+        self.tournamentItem = CountedMenuItem('Tournament', (False, True))
+        self.tournamentMode = False
         SelectionMenu.__init__(self, gvm, pos, [
-            CountedMenuItem('Not ready', False),
-            CountedMenuItem('Ready', True),
+            CountedMenuItem('Not ready', (False, False)),
+            self.scratchItem,
         ])
 
-    def selectItem(self, ready):
-        self.gvm.onChange.execute(PlayerIsReadyMsg(self.gvm.playerId, ready))
+    def selectItem(self, (ready, tournament)):
+        self.gvm.onChange.execute(PlayerIsReadyMsg(self.gvm.playerId, ready,
+                tournament))
         self.gvm.hideSubMenu()
 
     def updateTitle(self, player):
+        self.tournamentItem.enabled = player.prefersTournamentTeam
+
         if player.readyToStart:
-            title = 'Ready'
+            if not self.tournamentMode:
+                title = 'Ready'
+            else:
+                title = 'Scratch'
+        elif player.readyForTournament:
+            title = 'Tournament'
         else:
             title = 'Not ready'
         self.button.setText(title)
@@ -151,26 +167,28 @@ class ReadyMenu(SelectionMenu):
         values = {}
         for player in self.gvm.world.players:
             if not player.bot:
-                ready = player.readyToStart
-                values[ready] = values.get(ready, 0) + 1
+                key = player.readyToStart, player.readyForTournament
+                values[key] = values.get(key, 0) + 1
+
+        tournamentMode = (False, True) in values or self.tournamentItem.enabled
+        if tournamentMode and not self.tournamentMode:
+            self.appendItem(self.tournamentItem)
+            self.scratchItem.caption = 'Scratch'
+        elif not tournamentMode and self.tournamentMode:
+            self.popItem(2)
+            self.scratchItem.caption = 'Ready'
+        self.tournamentMode = tournamentMode
+
         for item in self.items:
             item.count = values.get(item.action, 0)
 
-DEFAULT_TEAM_NAMES = [
-    'Standardators',
-    'Defaulticons',
-]
 class TeamMenu(SelectionMenu):
     def __init__(self, gvm, pos):
         items = [
             CountedMenuItem('Any team', ''),
-        ]
-        for name in DEFAULT_TEAM_NAMES:
-            items.append(CountedMenuItem(name, name))
-        items.extend([
             MenuItem('---', None),
             MenuItem('Other...', None, listener=self._otherClicked),
-        ])
+        ]
         SelectionMenu.__init__(self, gvm, pos, items)
 
     def updateTitle(self, player):
@@ -213,8 +231,7 @@ class TeamMenu(SelectionMenu):
                 item.count = values.pop(item.action)
             else:
                 item.count = 0
-                if (item.caption not in DEFAULT_TEAM_NAMES and item.action !=
-                        ''):
+                if item.action != '':
                     killable_items.append(item)
         for item in killable_items:
             self.removeItem(item)
