@@ -16,6 +16,8 @@
 # 02110-1301, USA.
 
 from trosnoth.gui.framework import framework
+from trosnoth.model.universe_base import GameState
+from trosnoth.utils.utils import timeNow
 import pygame
 
 class GaugeBase(framework.Element):
@@ -80,13 +82,19 @@ class GaugeBase(framework.Element):
 class RespawnGauge(GaugeBase):
     '''Represents a graphical gauge to show how close to respawning a player
     is.'''
-    def __init__(self, app, area, player):
+    def __init__(self, app, area, player, gameController):
         super(RespawnGauge, self).__init__(app, area)
         self.player = player
+        self.gameController = gameController
 
     def getRatio(self):
-        return 1 - (self.player.respawnGauge /
-                self.player.universe.physics.playerRespawnTotal)
+        if self.gameController.state() == GameState.Starting:
+            secondsUntilGameStarts = self.gameController._gameStartingAt - timeNow()
+            return 1 - (secondsUntilGameStarts /
+                        self.gameController._gameStartDelay)
+        else:
+            return 1 - (self.player.respawnGauge /
+                    self.player.universe.physics.playerRespawnTotal)
 
     def getForeColour(self):
         if self.getRatio() >= 1:
@@ -175,6 +183,8 @@ class StarGauge(GaugeBase):
         self.upgrade = upgrade
 
     def getRatio(self):
+        if self.upgrade.requiredStars == 0:
+            return 1
         return self.player.getTeamStars() / (self.upgrade.requiredStars + 0.)
 
     def getForeColour(self):
