@@ -41,9 +41,10 @@ PUBLIC_UPGRADES = ['Minimap Disruption', 'Turret', 'Shield', 'Grenade']
 
 class LeaderBoard(framework.CompoundElement):
 
-    def __init__(self, app, world, gameViewer):
+    def __init__(self, app, game, gameViewer):
         super(LeaderBoard, self).__init__(app)
-        self.world = world
+        self.world = game.world
+        self.gameController = game.gameController
         self.gameViewer = gameViewer
         self.app = app
         self.scale = self.app.screenManager.scaleFactor
@@ -77,24 +78,18 @@ class LeaderBoard(framework.CompoundElement):
         self.playerTable.getColumn(1).setWidth(ScaledScalar(25))    # Star
         self.playerTable.getColumn(2).setWidth(ScaledScalar(70))    # Upgrade
 
-        self.colourLookup = {
-            'A': app.theme.colours.leaderboardBlue,
-            'B': app.theme.colours.leaderboardRed,
-            '\x00' : app.theme.colours.leaderBoardRogue,
-        }
-
         self._resetPlayers()
 
         self.elements = [self.playerTable]
         self.update(True)
 
     def _resetPlayers(self):
-        self.players = {'A': [], 'B': [], '\x00' : []}
+        self.players = dict((k, []) for k in self.world.teamWithId)
 
     def update(self, loop=False):
         self._resetPlayers()
 
-        if self.world.gameState == GameState.InProgress:
+        if self.world.gameIsInProgress():
             yPos = self.yPos
         else:
             yPos = self.yPos + int(50 * self.scale)
@@ -104,12 +99,10 @@ class LeaderBoard(framework.CompoundElement):
         try:
             pi = self.gameViewer.interface.runningPlayerInterface
             self.friendlyTeam = pi.player.team
-            self.enemyTeam = self.friendlyTeam.opposingTeam
             self.spectator = False
         except (AttributeError, KeyError):
             # This will occur if the player is not yet on a team
             self.friendlyTeam = self.world.teamWithId['A']
-            self.enemyTeam = self.world.teamWithId['B']
             self.spectator = True
 
         for player in self.world.players:
@@ -151,25 +144,31 @@ class LeaderBoard(framework.CompoundElement):
             self.playerTable.delRow(0)
 
         index = -1
-        self.indices = {}
+        self.contents = []
 
-        for team in (self.friendlyTeam, self.enemyTeam, None):
+        teamOrder = [self.friendlyTeam] + [t for t in self.world.teams if t !=
+                self.friendlyTeam]
+        if self.friendlyTeam is not None:
+            teamOrder.append(None)
+        for team in teamOrder:
             if team is None:
                 teamId = '\x00'
             else:
                 teamId = team.id
-            players = self.players[teamId]
-            if len(players) == 0:
+            # If we're in the lobby, only show real players
+            shouldShow = lambda player: teamId != '\x00' or not player['bot']
+            visiblePlayers = filter(shouldShow, self.players[teamId])
+            if len(visiblePlayers) == 0:
                 continue
 
-            index = self._addTeamHeader(team, teamId, players, index)
+            index = self._addTeamHeader(team, teamId, visiblePlayers, index)
 
             # Add a row for each player
-            for player in players:
+            for player in visiblePlayers:
                 index = self._addPlayerRow(team, teamId, player, index)
 
             # Add the 'star total' row
-            index = self._addTeamFooter(team, teamId, players, index)
+            index = self._addTeamFooter(team, teamId, visiblePlayers, index)
 
     def _addRow(self, index, height):
         index += 1
@@ -204,42 +203,37 @@ class LeaderBoard(framework.CompoundElement):
         index = self._addRow(index, ScaledScalar(self.rowHeight))
         self.playerTable[0][index].setText( self.world.getTeamName(teamId))
 
-        if self.world.gameState == GameState.InProgress:
+        if self.world.gameIsInProgress():
             self._embedTeamHeaderStar(index)
             self._writeTeamHeaderPlayerCount(players, index)
         else:
             self.playerTable[1][index].elements = []
 
-        teamColour = self.colourLookup[teamId]
-        if self.world.gameState in (GameState.PreGame, GameState.Starting):
+        teamColour = self.app.theme.colours.leaderboard(team)
+        if self.gameController.state() in (GameState.Lobby, GameState.PreGame,
+                GameState.Starting):
             if team is None:
                 pass
-            elif team.ready:
-                self._setTeamHeaderStatus(index, 'ready',
-                        self.app.theme.colours.leaderboardNormal)
-            else:
-                self._setTeamHeaderStatus(index, 'not ready',
-                        self.app.theme.colours.leaderboardDead)
-        elif self.world.gameState == GameState.Ended:
+        elif self.gameController.state() == GameState.Ended:
             if team is None:
                 pass
-            elif self.world.winningTeam == team:
+            elif self.world.getWinningTeam() == team:
                 self._setTeamHeaderStatus(index, 'has won!', teamColour)
-            elif self.world.winningTeam is not None:
+            elif self.world.getWinningTeam() is not None:
                 self._setTeamHeaderStatus(index, 'has lost.', teamColour)
             else:
                 self._setTeamHeaderStatus(index, 'has drawn!', teamColour)
 
         self.playerTable.getRow(index).style.foreColour = teamColour
-        self.indices['header' + teamId] = index
+        self.contents.append((index, 'header', team))
 
         return index
 
     def _addUpgradeText(self, team, player, index):
         if player['upgrade'] is None:
             upgradeText = ''
-        elif (team is self.enemyTeam and player['upgrade'] not in
-                PUBLIC_UPGRADES):
+        elif (team != self.friendlyTeam and player['upgrade'] not in
+                PUBLIC_UPGRADES and team is not None):
             upgradeText = ''
         else:
             if player['upgrade'] in SHORT_UPGRADE_NAMES:
@@ -251,11 +245,11 @@ class LeaderBoard(framework.CompoundElement):
         cell.setText(upgradeText)
         cell.style.foreColour = self.app.theme.colours.leaderboardNormal
 
-    def _setPlayerColour(self, teamId, player, index):
+    def _setPlayerColour(self, team, player, index):
         if player['dead']:
             colour = self.app.theme.colours.leaderboardDead
         else:
-            colour = self.colourLookup[teamId]
+            colour = self.app.theme.colours.leaderboard(team)
         self.playerTable[0][index].style.foreColour = colour
         self.playerTable[1][index].style.foreColour = colour
 
@@ -263,22 +257,18 @@ class LeaderBoard(framework.CompoundElement):
         index = self._addRow(index, ScaledScalar(self.rowHeight))
         self.playerTable[0][index].setText(player['nick'])
 
-        if self.world.gameState in (GameState.PreGame, GameState.Starting):
-            if team is not None and team.captain is not None:
-                if player['pID'] == struct.unpack('B', team.captain.id)[0]:
-                    self.playerTable[2][index].setText('captain')
-        else:
+        if self.gameController.state() not in (GameState.PreGame, GameState.Starting):
             if team is self.friendlyTeam or self.spectator:
                 self.playerTable[1][index].setText(str(player['stars']))
 
             self._addUpgradeText(team, player, index)
 
-        self._setPlayerColour(teamId, player, index)
+        self._setPlayerColour(team, player, index)
 
         return index
 
     def _addTeamFooter(self, team, teamId, players, index):
-        if (len(players) > 1 and self.world.gameState not in
+        if (len(players) > 1 and self.gameController.state() not in
                 (GameState.PreGame, GameState.Starting) and
                 team is not None):
             index = self._addRow(index, ScaledScalar(self.rowHeight))
@@ -290,7 +280,7 @@ class LeaderBoard(framework.CompoundElement):
             else:
                 noun = 'stars'
             self.playerTable[2][index].setText(noun)
-            self.indices['total'+teamId] = index
+            self.contents.append((index, 'total', team))
 
         return index
 
@@ -305,8 +295,8 @@ class LeaderBoard(framework.CompoundElement):
         # pointC = left-most point of the line shadow
         # pointD = right-most point of the line shadow
 
-        for name, index in self.indices.iteritems():
-            if name[:6] == 'header':
+        for (index, rowKind, team) in self.contents:
+            if rowKind == 'header':
                 rightMargin = int(8 * self.scale)
                 leftMargin = int(40 * self.scale)
                 bottomMargin = int(2 * self.scale)
@@ -316,19 +306,19 @@ class LeaderBoard(framework.CompoundElement):
                 pointB = (pointA[0] + self.playerTable._getSize()[0] -
                         leftMargin - rightMargin, pointA[1])
                 pointC, pointD = shift(pointA, pointB)
-                pygame.draw.line(surface, self.colourLookup[name[-1:]], pointA,
-                        pointB, 1)
+                teamColour = self.app.theme.colours.leaderboard(team)
+                pygame.draw.line(surface, teamColour, pointA, pointB, 1)
                 pygame.draw.line(surface, (0, 0, 0), pointC, pointD, 1)
 
                 pointA, pointB = shift(pointA, pointB,
                         -self.playerTable.getRow(index)._getHeight() +
                         bottomMargin, 0)
                 pointC, pointD = shift(pointA, pointB)
-                pygame.draw.line(surface, self.colourLookup[name[-1:]], pointA,
-                        pointB, 1)
+                teamColour = self.app.theme.colours.leaderboard(team)
+                pygame.draw.line(surface, teamColour, pointA, pointB, 1)
                 pygame.draw.line(surface, (0, 0, 0), pointC, pointD, 1)
 
-            if name[:5] == 'total':
+            elif rowKind == 'total':
                 bottomMargin = int(2 * self.scale)
 
                 rowPoint = self.playerTable._getRowPt(index)
