@@ -49,8 +49,6 @@ class MainMenu(MenuDisplay):
                 items=[
             MenuItem('Abandon upgrade', 'abandon'),
             MenuItem('Chat', 'chat'),
-            MenuItem('Become captain', 'captain'),
-            MenuItem('Team is ready', 'team ready'),
             MenuItem('Show/hide HUD', 'toggle interface'),
             MenuItem('Mark zone at cursor', 'mark zone'),
             MenuItem('Cancel', 'menu')
@@ -75,9 +73,9 @@ class MainMenu(MenuDisplay):
         manager.setDefaultMenu(mainMenu)
 
         super(MainMenu, self).__init__(app, location, size, font, manager,
-                                       titleColour, stdColour, hvrColour,
-                                       backColour, autosize, hidable,
-                                       keymapping)
+                titleColour, stdColour, hvrColour, None, backColour, autosize,
+                hidable, keymapping)
+
     def showBuyMenu(self):
         self.manager.reset()
         self.manager.showMenu(self.buyMenu)
