@@ -27,17 +27,19 @@ class MessageBank(framework.CompoundElement):
 
     def __init__(self, app, maxSize, maxLength, startPt, L_R, U_D, font,
             timeLimit = 10):
-        '''Parameters:
-        maxSize: the number of lines that can be displayed at any one time
-        maxLength: the maximum length of a line
-
-        startPt is the corner closest to the edge of the screen
-        that a message will get. For example, if aligned top left, this point
-        will specify the top left position of the most recent message
-
-        L_R defines the alignment of the text that appears
-        U_D defines whether the text will be coming from the top or the
-        bottom'''
+        '''
+        @param maxSize: The largest number of lines that can be displayed at
+            any given time.
+        @param maxLength: The maximum length of a line. Lines longer than this
+            will be wrapped.
+        @param startPt: The corner closest to the edge of the screen that a
+            message will get. For example, if aligned top left, this point
+            will specify the top left position of the most recent message.
+        @param L_R: the alignment of the text that appears ("left" or "right")
+        @param U_D: defines whether the text will be coming from the top or the
+            bottom ("top" or "bottom")
+        @param timeLimit: the time (in seconds) before a message disappears
+        '''
         super(MessageBank, self).__init__(app)
         self.timeLimit = timeLimit
         if maxSize <= 0:
@@ -54,9 +56,15 @@ class MessageBank(framework.CompoundElement):
         self.startPt = startPt
 
     def newMessage(self, text, colour):
+        '''
+        Adds a new message to the bank, splitting lines as necessary.
+
+        @param text: The text of the message. May contain newline characters.
+        @param colour: A tuple.
+        '''
         text = self.breakUp(text)
         if not text.__contains__('\n'):
-            self.newIndividual(Message(self.app, text, colour, self,
+            self._newIndividual(Message(self.app, text, colour, self,
                     self.timeLimit))
             return
         tempMessages = []
@@ -79,28 +87,68 @@ class MessageBank(framework.CompoundElement):
             j = len(tempMessages) - 1
 
         while j >= 0 and j < len(tempMessages):
-            self.newIndividual(tempMessages[j])
+            self._newIndividual(tempMessages[j])
             j += inc
 
-    def newIndividual(self, newMessage):
+    def _newIndividual(self, newMessage):
+        self.prepareForInsertion()
+        self.elements.append(newMessage)
+
+    def prepareForInsertion(self):
+        '''
+        Makes room for a new message.
+
+        Deletes the oldest message if the capacity has been reached and moves
+        every message up one space.
+        '''
         if self.currentLength >= self.maxSize:
             self.delMessage(True)
         for message in self.elements:
             message.downByOne()
-
-        self.elements.append(newMessage)
         self.currentLength += 1
 
+    def newColourMessage(self, parts):
+        '''
+        Adds a message with multiple colour parts to the MessageBank.
+        This method does not split / wrap lines: you have to do this yourself!
+
+        @param parts: A list containing the colour segments of the message.
+        Each segment is a tuple with the text and colour for that segment.
+        '''
+        self.prepareForInsertion()
+
+        count = 0
+        xOffset = 0
+        
+        for text, colour in parts:
+            count += 1
+
+            # Only the last item should have a break afterwards
+            if count == len(parts):
+                trailingBreak = True
+            else:
+                trailingBreak = False
+                
+            message = Message(self.app, text, colour, self, self.timeLimit,
+                    trailingBreak, xOffset)
+
+            xOffset += message._getRect().width
+
+            self.elements.append(message)
+                    
+
     def delMessage(self, popped = False):
         # We'll always be deleting the oldest message
-        self.elements.pop(0)
+        popped = self.elements.pop(0)
+        while not popped.trailingBreak:
+            popped = self.elements.pop(0)
         self.currentLength -= 1
 
     def breakUp(self, text):
         '''Breaks the text up into seperate lines of max 50 chars. If there's
         a space within 20 characters of the end, it will break there, otherwise
         it will simply cut the current word in half'''
-        maxLength = 50
+        maxLength = self.maxLength
         minLength = 30
         lineDelimiters = (' ', '\t', '-')
         i = 0
@@ -138,18 +186,25 @@ class MessageBank(framework.CompoundElement):
             self.delMessage(True)
 
 class Message(elements.TextElement):
-    '''Sprite object that every player has which indicates the player's nick.'''
-    def __init__(self, app, text, colour, messageBank, timeLimit):
+    def __init__(self, app, text, colour, messageBank, timeLimit,
+                 trailingBreak = True, xOffset = 0):
         self.messageBank = messageBank
         if not colour:
             colour = app.theme.colours.grey
+            
         super(Message, self).__init__(app, text, self.messageBank.font,
                                       self.messageBank.startPt,
                                       colour, shadow = True)
+
+        if xOffset > 0:
+            self.setPos(RelativeLocation(self.pos, (xOffset, 0)))
+            
         if colour == guiColours.shadowDefault:
             self.setShadowColour(guiColours.shadowAgainstGrey)
 
         self.timeLeft = timeLimit
+        self.xOffset = xOffset
+        self.trailingBreak = trailingBreak
 
     def __repr__(self):
         return 'Message object: "%s"' % self.getText()
