@@ -182,7 +182,7 @@ class MiniMap(framework.Element):
             for star in self.universe.collectableStars.values():
                 drawCircle(star, colours.miniMapStarColour(star.team), 2)
 
-        self.drawZoneHighlights(screen, sRect)
+            self.drawZoneHighlights(screen, sRect)
 
         # Finally, draw the border
         screen.set_clip(oldClip)
@@ -278,11 +278,10 @@ class MiniMap(framework.Element):
         else:
             playerTeam = player.team
         owner = zone.orbOwner
-
         if zone.isCapturableBy(playerTeam):
             return sprites.zoneHighlight(playerTeam, self.scale)
         if owner == playerTeam or owner is None:
-            teams = zone.getAttackingTeams()
+            teams = zone.teamsAbleToTag()
             teams.discard(playerTeam)
             if teams:
                 return sprites.zoneHighlight(teams.pop(), self.scale)
