@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2010 Joshua D Bartlett
+# Copyright (C) 2006-2011 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -26,6 +26,6 @@ class ObserverInterface(framework.CompoundElement):
         self.keyMapping = gameInterface.keyMapping
 
     def processEvent(self, event):
-        if event.type == pygame.KEYDOWN and \
-                    self.keyMapping.get(event.key, None) == 'menu':
-           self.gameInterface.showMenu()
\ No newline at end of file
+        if (event.type == pygame.KEYDOWN and self.keyMapping.get(event.key,
+                None) == 'menu'):
+           self.gameInterface.showMenu()
