@@ -76,7 +76,7 @@ class GrenadeSprite(pygame.sprite.Sprite):
     def __init__(self, app, grenade):
         super(GrenadeSprite, self).__init__()
         self.grenade = grenade
-        self.image = app.theme.sprites.grenade
+        self.image = app.theme.sprites.teamGrenade(grenade.player.team)
         self.rect = self.image.get_rect()
 
     @property
@@ -121,7 +121,7 @@ class PlayerSprite(pygame.sprite.Sprite):
 
         head = sprites.playerHead(self.player.team, self.player.bot)
         self.runningAnimation = [
-            Animation(0.1, self.player.universe.getGameTime,
+            Animation(0.1, self.player.universe.getElapsedTime,
                 *sprites.runningLegs),
             sprites.playerBody,
             head,
@@ -130,7 +130,7 @@ class PlayerSprite(pygame.sprite.Sprite):
 
         self.reversingAnimation = [
             sprites.playerBody,
-            Animation(0.1, self.player.universe.getGameTime,
+            Animation(0.1, self.player.universe.getElapsedTime,
                     *sprites.backwardsLegs),
             head,
         ]
@@ -159,7 +159,7 @@ class PlayerSprite(pygame.sprite.Sprite):
         ]
         self.fallingAnimation = self.jumpingAnimation
         self.shieldAnimation = Animation(0.15,
-                self.player.universe.getGameTime, *sprites.shieldImages)
+                self.player.universe.getElapsedTime, *sprites.shieldImages)
         self.phaseShiftAnimation = Animation(0.15, timeNow,
                 *sprites.phaseShiftImages)
         self.jammingHat = app.theme.sprites.jammingHat()
