@@ -29,7 +29,6 @@ from trosnoth.gui.framework import framework, elements
 from trosnoth.trosnothgui.ingame.leaderboard import LeaderBoard
 from trosnoth.trosnothgui.ingame.statusBar import ZoneProgressBar
 from trosnoth.trosnothgui.ingame.gameTimer import GameTimer
-from trosnoth.trosnothgui.ingame.chatBox import ChatBox
 from trosnoth.trosnothgui.ingame.universegui import UniverseGUI
 from trosnoth.model.map import MapLayout
 from trosnoth.model.universe_base import GameState
@@ -576,18 +575,20 @@ class GameViewer(framework.CompoundElement):
 
     zoneBarHeight = 25
 
-    def __init__(self, app, gameInterface, world, replay):
+    def __init__(self, app, gameInterface, game, replay):
         super(GameViewer, self).__init__(app)
         self.replay = replay
         self.interface = gameInterface
-        self.world = world
+        self.game = game
+        self.gameController = game.gameController
+        self.world = game.world
         self.worldgui = UniverseGUI(app, self, self.world)
         self.app = app
 
         self.viewManager = ViewManager(self.app, self.worldgui, replay=replay)
 
-        self.timerBar = GameTimer(app, world)
-        self.chatBox = ChatBox(self.app, self.world, self)
+        self.timerBar = GameTimer(app, game)
+        
         self.miniMap = None
         self.pregameMessage = None
         self.leaderboard = None
@@ -639,17 +640,17 @@ class GameViewer(framework.CompoundElement):
                 'topright'), 'topright')
         self.pregameMessage = PregameMessage(self.app, messagePos)
         self.zoneBar = ZoneProgressBar(self.app, self.world, self)
-        self.leaderboard = LeaderBoard(self.app, self.world, self)
+        self.leaderboard = LeaderBoard(self.app, self.game, self)
 
         self.elements = [self.viewManager, self.pregameMessage]
 
     def updatePregameMessage(self):
         text = ''
-        if self.world.gameState == GameState.InProgress:
+        if self.gameController.state() == GameState.InProgress:
             text = ''
-        elif self.world.gameState == GameState.PreGame:
-            text = 'Game will begin when both captains signal ready.'
-        elif self.world.gameState == GameState.Starting:
+        elif self.gameController.state() == GameState.PreGame:
+            text = 'Indicate "Ready" to begin'
+        elif self.gameController.state() == GameState.Starting:
             text = 'Prepare yourself...'
         else:
             pi = self.interface.runningPlayerInterface
@@ -657,7 +658,7 @@ class GameViewer(framework.CompoundElement):
                 team = pi.player.team
                 if team is None:
                     text = 'Waiting for more ready players.'
-                elif team is self.world.winningTeam:
+                elif team is self.world.getWinningTeam():
                     text = 'Congratulations!'
                 else:
                     text = 'Better luck next time!'
@@ -699,14 +700,4 @@ class GameViewer(framework.CompoundElement):
         if self.leaderboard in self.elements:
             self.elements.remove(self.leaderboard)
         else:
-            self.elements.append(self.leaderboard)
-
-    def openChat(self, player):
-        pygame.key.set_repeat(300, 30)
-        self.chatBox.reset()
-        self.chatBox.setPlayer(player)
-        self.chatBox.show()
-
-    def closeChat(self):
-        self.chatBox.close()
-
+            self.elements.append(self.leaderboard)
\ No newline at end of file
