@@ -36,8 +36,6 @@ from trosnoth.trosnothgui.pregame.playscreen import PlayAuthScreen
 from trosnoth.trosnothgui.pregame.connectionFailedDialog import (
         ConnectionFailedDialog)
 from trosnoth.trosnothgui.ingame.gameInterface import GameInterface
-from trosnoth.trosnothgui.pregame.playspecial import PlaySpecial
-from trosnoth.trosnothgui.pregame.playerNameMenu import PlayerNameMenu
 from trosnoth.trosnothgui.pregame.savedGameMenu import SavedGameMenu
 from trosnoth.trosnothgui.settings.settings import SettingsMenu
 import trosnoth.version
@@ -124,13 +122,13 @@ class Interface(framework.CompoundElement):
 
     def setupGame(self, game, authTag):
         # Create a game interface.
-        self.gi = gi = GameInterface(self.app, game.world,
+        self.gi = gi = GameInterface(self.app, game,
                 onDisconnectRequest=self._disconnectGame,
                 onConnectionLost=self.connectionLost, authTag=authTag)
         self.agentId = game.addAgent(gi.eventPlug, gi.controller)
 
     def setupReplay(self, game):
-        self.gi = gi = GameInterface(self.app, game.world,
+        self.gi = gi = GameInterface(self.app, game,
                 onDisconnectRequest=self.replayOver, replay=True)
         self.agentId = game.addAgent(gi.eventPlug, gi.controller)
 
@@ -188,6 +186,8 @@ class Interface(framework.CompoundElement):
         self.elements = [self.gi]
 
 class SingleAuthInterface(Interface):
+    connectScreenFactory = PlayAuthScreen
+
     def __init__(self, app):
         Interface.__init__(self, app)
 
@@ -198,6 +198,8 @@ class SingleAuthInterface(Interface):
             settings = app.connectionSettings
             servers = list(settings.servers)
             if not trosnoth.version.release:
+                # Check that developers aren't accidentally connecting to
+                # play.trosnoth.org.
                 servers = [s[:2] for s in servers if s[0] !=
                         'play.trosnoth.org']
             if settings.otherGames:
@@ -212,33 +214,12 @@ class SingleAuthInterface(Interface):
             self.startupInterface.begin(servers, canHost=True)
 
     def _makeStartupInterface(self):
-        return PlayAuthScreen(self.app, onFail=self.app.stop)
+        return self.connectScreenFactory(self.app, onFail=self.app.stop)
 
     def connectionLost(self, reason=None):
         Interface.connectionLost(self, reason)
         self.app.stop()
 
-class PlaySpecialInterface(Interface):
-    def __init__(self, app):
-        Interface.__init__(self, app)
-
-        name = self.app.identitySettings.nick
-        if name is None:
-            playerNameMenu = PlayerNameMenu(app)
-            playerNameMenu.onGotName.addListener(self._gotName)
-            self.elements = [self.backdrop, self.playerNameMenu]
-        else:
-            self._gotName(name)
-
-    def _gotName(self, name):
-        self.startupInterface.setName(name)
-        self.startupInterface.regainedFocus()
-        self.elements = [self.backdrop, self.startupInterface]
-
-    def _makeStartupInterface(self):
-        return PlaySpecial(self.app, onCancel=self.app.stop,
-                onInetConnect=self.connectToServer)
-
 class KeySettingsInterface(Interface):
     def _makeStartupInterface(self):
         return SettingsMenu(self.app, onClose=self.app.stop, showSound=False,
