@@ -20,16 +20,25 @@ import os
 import time
 import urllib
 
-from trosnoth.gamerecording.achievementlist import getAchievementDetails
+from trosnoth.gamerecording.achievementlist import availableAchievements
 from trosnoth.network.networkDefines import serverVersion
 from trosnoth.web.site import PageSet, ServerState, Template
 from trosnoth.version import fullVersion
 from trosnoth import data
 
-DOCTYPE = ('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" '
-        '"http://www.w3.org/TR/html4/loose.dtd">')
+DOCTYPE = ('<!DOCTYPE html>')
 pages = PageSet()
+pages.achievementDefs = availableAchievements
 
+# <script src="https://browserid.org/include.js" type="text/javascript"></script>
+# <script src="/login.js" type="text/javascript"></script>
+#  <header class="clearfix">
+#    <div id="logininfo">
+#      <div class="info">Not signed in</div>
+#      <div class="signin"><a href="javascript:loginButtonClicked()"><img
+#          src="https://browserid.org/i/sign_in_blue.png"></a></div>
+#    </div>
+#  </header>
 pageTemplate = Template('''%(docType)s
 <html>
 <head>
@@ -53,7 +62,8 @@ pageTemplate = Template('''%(docType)s
 })
 
 sendRequestJS = '''
-  function sendRequest(url, onSucceed, onFailure) {
+  function sendRequest(url, onSucceed, onFailure, method) {
+    if (!method) method = 'GET';
     if (window.XMLHttpRequest) {
         xmlhttp = new XMLHttpRequest();
     } else {
@@ -68,7 +78,7 @@ sendRequestJS = '''
         }
       }
     };
-    xmlhttp.open('GET', url, true);
+    xmlhttp.open(method, url, true);
     xmlhttp.send();
   }
 '''
@@ -177,10 +187,60 @@ def home(state, request):
     'timer': timerJS,
 })
 
+@pages.addPage('/login.js')
+def loginjs(state, request):
+    request.setHeader('Content-Type', 'text/javascript')
+    return '''
+function loginButtonClicked() {
+    navigator.id.get(function(assertion) {
+        if (assertion) {
+            // We really need to send the assertion to the server to get it
+            // verified.
+            loggedIn();
+        } else {
+            // something went wrong!  the user isn't logged in.
+            loggedOut();
+        }
+    });
+}
+
+function setSessions(val) {
+  navigator.id.sessions = val ? val : [ ];
+}
+
+function loggedIn(email) {
+  setSessions([ { email: email } ]);
+
+  document.getElementById("logininfo").innerHTML = '<div class="info">Signed in</div>';
+}
+
+function loggedOut() {
+  setSessions();
+  document.getElementById("logininfo").innerHTML = '<div class="info">Not signed in</div><div class="signin"><a href="javascript:loginButtonClicked()"><img src="https://browserid.org/i/sign_in_blue.png"></a></div>';
+}
+'''
+
+##container {
+#  margin: 45px auto 0;
 @pages.addPage('/style.css')
 def style(state, request):
     request.setHeader('Content-Type', 'text/css')
     return '''
+header {
+    background-color: #eeeeee;
+    border-bottom: 2px solid rgba(200, 230, 255, 0.5);
+    box-shadow: 0 1px 8px rgba(0, 0, 64, 0.6);
+    color: #606060;
+    display: block;
+    height: 30px;
+    left: 0;
+    opacity: 0.8;
+    position: fixed;
+    top: 0;
+    width: 100%;
+    z-index: 10;
+}
+
 h1 {
   color: #000060;
   font-family: Junction, Verdana, Helvetica, sans-serif;
@@ -189,6 +249,10 @@ h1 {
   text-shadow: #c0c0c0 1px 1px 1px;
 }
 
+header h1 {
+  margin-top: 0px;
+}
+
 h1 a {
   text-decoration: none;
 }
@@ -232,7 +296,7 @@ h2 {
 
 .achievement img {
   border: 1px solid black;
-  padding: 7px;
+  padding: 0px;
 }
 
 .achievement_name {
@@ -248,6 +312,30 @@ a img {
   border: 0px none;
   margin: 2px;
 }
+
+#logininfo {
+  float: right;
+}
+
+.info {
+  display: inline-block;
+  vertical-align: top;
+  margin-top: 7px;
+}
+
+.signin {
+  display: inline-block;
+  margin-top: 2px;
+}
+
+.clearfix:after {
+  clear: both;
+  content: ".";
+  display: block;
+  height: 0;
+  line-height: 0;
+  visibility: hidden;
+}
 '''
 
 @pages.addPage('/data/header')
@@ -268,8 +356,8 @@ def playNow(state, request):
 
 @pages.addPage('/play')
 def play(state, request):
-    hostName = state.getHostName()
-    homeAddress = state.getHomeAddress()
+    hostName = state.getHostName(request)
+    homeAddress = state.getHomeAddress(request)
     return pageTemplate.apply(
         title=cgi.escape(state.getName()),
         contents='''
@@ -370,12 +458,7 @@ def play(state, request):
     this.error = error = function(code) {
       var msg = '<p>' + downloadHTML
       msg += '<br>To play on this server, you will need ';
-      msg += 'version %(version)s or compatible.</p><p class="small">';
-      msg += 'This message is shown because the server cannot detect a ';
-      msg += 'running Trosnoth core. If this is incorrect and you are ';
-      msg += 'running Trosnoth core version %(version)s or compatible, ';
-      msg += '<a href="%(playUrl)s">click here</a> to play on %(hostName)s.'
-      msg += '</p>';
+      msg += 'version %(version)s or compatible.</p>';
       playMsg.innerHTML = msg;
       if (timeout) clearTimeout(timeout);
       onTimer.reset();
@@ -455,8 +538,14 @@ def playerProfile(state, request):
     achievements = []
     for achievement, status in user.achievements.iteritems():
         if status['unlocked']:
-            name, description = getAchievementDetails(achievement)
-            achievements.append((name, achievement, description))
+            try:
+                name, description = pages.achievementDefs.getAchievementDetails(
+                        achievement)
+            except KeyError:
+                # Achievement no longer exists.
+                pass
+            else:
+                achievements.append((name, achievement, description))
     achievements.sort()
     achStrings = []
     for name, imgId, description in achievements:
